@@ -95,7 +95,7 @@ static double bench_generic_matrix_multiply(long dims[DIMS])
 }
 
 
-static double bench_generic_add(long dims[DIMS], unsigned int flags, bool forloop)
+static double bench_generic_add(long dims[DIMS], unsigned long flags, bool forloop)
 {
 	long dimsX[DIMS];
 	long dimsY[DIMS];
@@ -104,7 +104,7 @@ static double bench_generic_add(long dims[DIMS], unsigned int flags, bool forloo
 
 	md_select_dims(DIMS, flags, dimsX, dims);
 	md_select_dims(DIMS, ~flags, dimsC, dims);
-	md_select_dims(DIMS, ~0u, dimsY, dims);
+	md_select_dims(DIMS, ~0UL, dimsY, dims);
 
 	long strsX[DIMS];
 	long strsY[DIMS];
@@ -146,13 +146,13 @@ static double bench_generic_add(long dims[DIMS], unsigned int flags, bool forloo
 }
 
 
-static double bench_generic_sum(long dims[DIMS], unsigned int flags, bool forloop)
+static double bench_generic_sum(long dims[DIMS], unsigned long flags, bool forloop)
 {
 	long dimsX[DIMS];
 	long dimsY[DIMS];
 	long dimsC[DIMS];
 
-	md_select_dims(DIMS, ~0u, dimsX, dims);
+	md_select_dims(DIMS, ~0UL, dimsX, dims);
 	md_select_dims(DIMS, flags, dimsY, dims);
 	md_select_dims(DIMS, ~flags, dimsC, dims);
 
@@ -433,7 +433,7 @@ static double bench_wavelet(long scale)
 	minsize[1] = MIN(dims[1], 16);
 	minsize[2] = MIN(dims[2], 16);
 
-	const struct operator_p_s* p = prox_wavelet_thresh_create(DIMS, dims, 6, 0u, minsize, 1.1, true);
+	const struct operator_p_s* p = prox_wavelet_thresh_create(DIMS, dims, 6, 0u, WAVELET_DAU2, minsize, 1.1, true);
 
 	complex float* x = md_alloc(DIMS, dims, CFL_SIZE);
 	md_gaussian_rand(DIMS, dims, x);
@@ -537,6 +537,45 @@ static double bench_fftmod(long scale)
 
 
 
+static double bench_generic_expand(int typ, long scale)
+{
+	long dims[DIMS] = { 1, 256 * scale, 256 * scale, 1, 1, 16, 1, 16 };
+
+	complex float* x = md_alloc(DIMS, dims, CFL_SIZE);
+
+	double tic = timestamp();
+
+	switch (typ) {
+
+		case 0:
+			md_zfill(DIMS, dims, x, 1.);
+			break;
+		case 1:
+			md_zsmul(DIMS, dims, x, x, 1.);
+			break;
+		default:
+			assert(0);
+	}
+
+	double toc = timestamp();
+
+	md_free(x);
+
+	return toc - tic;
+}
+
+
+static double bench_zfill(long scale)
+{
+	return bench_generic_expand(0, scale);
+}
+
+static double bench_zsmul(long scale)
+{
+	return bench_generic_expand(1, scale);
+}
+
+
 
 
 enum bench_indices { REPETITION_IND, SCALE_IND, THREADS_IND, TESTS_IND, BENCH_DIMS };
@@ -597,6 +636,8 @@ const struct benchmark_s {
 	{ bench_zl1norm,	"l1 norm" },
 	{ bench_copy1,		"copy 1" },
 	{ bench_copy2,		"copy 2" },
+	{ bench_zfill,		"complex fill" },
+	{ bench_zsmul,		"complex scalar multiplication" },
 	{ bench_wavelet,	"wavelet soft thresh" },
 	{ bench_mdfft,		"(MD-)FFT" },
 	{ bench_fft,		"FFT" },
@@ -604,13 +645,19 @@ const struct benchmark_s {
 };
 
 
-static const char usage_str[] = "[<output>]";
 static const char help_str[] = "Performs a series of micro-benchmarks.";
 
 
 
-int main_bench(int argc, char* argv[])
+int main_bench(int argc, char* argv[argc])
 {
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_OUTFILE(false, &out_file, "output"),
+	};
+
 	bool threads = false;
 	bool scaling = false;
 	long flags = ~0l;
@@ -622,7 +669,7 @@ int main_bench(int argc, char* argv[])
 		OPT_LONG('s', &flags, "flags", "select benchmarks"),
 	};
 
-	cmdline(&argc, argv, 0, 1, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	long dims[BENCH_DIMS] = MD_INIT_ARRAY(BENCH_DIMS, 1);
 	long strs[BENCH_DIMS];
@@ -635,8 +682,8 @@ int main_bench(int argc, char* argv[])
 
 	md_calc_strides(BENCH_DIMS, strs, dims, CFL_SIZE);
 
-	bool outp = (2 == argc);
-	complex float* out = (outp ? create_cfl : anon_cfl)(outp ? argv[1] : "", BENCH_DIMS, dims);
+	bool outp = (NULL != out_file);
+	complex float* out = (outp ? create_cfl : anon_cfl)(out_file, BENCH_DIMS, dims);
 
 	num_init();
 
