@@ -28,16 +28,20 @@
 
 
 
-static const char usage_str[] = "<kspace> <coeff>|<proj_kspace>";
 static const char help_str[] = "Performs coil compression.";
 
 
-
-
-
-
 int main_cc(int argc, char* argv[argc])
 {
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "kspace"),
+		ARG_OUTFILE(true, &out_file, "coeff|proj_kspace"),
+	};
+
 	long calsize[3] = { 24, 24, 24 };
 	bool proj = true;
 	long P = -1;
@@ -56,7 +60,7 @@ int main_cc(int argc, char* argv[argc])
 		OPT_SELECT('E', enum cc_type, &cc_type, ECC, "type: ESPIRiT"),
 	};
 
-	cmdline(&argc, argv, 2, 2, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
@@ -68,7 +72,7 @@ int main_cc(int argc, char* argv[argc])
 
 	long in_dims[DIMS];
 
-	complex float* in_data = load_cfl(argv[1], DIMS, in_dims);
+	complex float* in_data = load_cfl(in_file, DIMS, in_dims);
 
 	assert(1 == in_dims[MAPS_DIM]);
 	long channels = in_dims[COIL_DIM];
@@ -81,7 +85,7 @@ int main_cc(int argc, char* argv[argc])
 	out_dims[MAPS_DIM] = channels;
 	out_dims[READ_DIM] = (SCC == cc_type) ? 1 : in_dims[READ_DIM];
 
-	complex float* out_data = (proj ? anon_cfl : create_cfl)(argv[2], DIMS, out_dims);
+	complex float* out_data = (proj ? anon_cfl : create_cfl)(out_file, DIMS, out_dims);
 
 
 	long caldims[DIMS];
@@ -123,7 +127,7 @@ int main_cc(int argc, char* argv[argc])
 		md_copy_dims(DIMS, trans_dims, in_dims);
 		trans_dims[COIL_DIM] = P;
 
-		complex float* trans_data = create_cfl(argv[2], DIMS, trans_dims);
+		complex float* trans_data = create_cfl(out_file, DIMS, trans_dims);
 
 		long fake_trans_dims[DIMS];
 		md_select_dims(DIMS, ~COIL_FLAG, fake_trans_dims, in_dims);
@@ -173,6 +177,7 @@ int main_cc(int argc, char* argv[argc])
 	}
 
 	printf("Done.\n");
+
 	return 0;
 }
 
