@@ -17,6 +17,7 @@
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
+#include "misc/opts.h"
 
 
 
@@ -25,26 +26,37 @@
 #endif
 
 
-static const char usage_str[] = "<input> <output";
-static const char help_str[] = "Copy phase from <input> to <output>.\n";
+static const char help_str[] = "Copy phase from <input> to <output>.";
 
 
 int main_cpyphs(int argc, char* argv[argc])
 {
-	mini_cmdline(&argc, argv, 2, usage_str, help_str);
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
+	const struct opt_s opts[] = {};
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	long dims[DIMS];
 	
-	complex float* in_data = load_cfl(argv[1], DIMS, dims);
-	complex float* out_data = create_cfl(argv[2], DIMS, dims);
+	complex float* in_data = load_cfl(in_file, DIMS, dims);
+	complex float* out_data = create_cfl(out_file, DIMS, dims);
 
 	md_zphsr(DIMS, dims, out_data, in_data);
 
 	unmap_cfl(DIMS, dims, in_data);
 	unmap_cfl(DIMS, dims, out_data);
-	exit(0);
+
+	return 0;
 }
 
 
