@@ -1,4 +1,3 @@
-
 /* 
  * Copyright 2013-2015 The Regents of the University of California.
  * All rights reserved. Use of this source code is governed by
@@ -16,11 +15,14 @@
 
 #define _GNU_SOURCE
 #include <math.h>
-#include <stdio.h>
 #include <string.h>
 #include <assert.h>
 #include <complex.h>
 
+#ifdef _WIN32
+#include "win/rand_r.h"
+#endif
+
 #include "num/multind.h"
 #include "misc/misc.h"
 
@@ -58,7 +60,7 @@ static void get_noise_amp (struct dfwavelet_plan_s* plan);
 
 
 
-struct dfwavelet_plan_s* prepare_dfwavelet_plan(int numdims, long* imSize, long* minSize, data_t* res,int use_gpu)
+struct dfwavelet_plan_s* prepare_dfwavelet_plan(int numdims, long* imSize, const long* minSize, data_t* res,int use_gpu)
 {
   struct dfwavelet_plan_s* plan = (struct dfwavelet_plan_s*) malloc(sizeof(struct dfwavelet_plan_s));
   plan->use_gpu = use_gpu;
@@ -819,7 +821,7 @@ void circshift_cpu(struct dfwavelet_plan_s* plan, data_t *data) {
     return;
   }
   // Copy data
-  data_t* dataCopy = malloc(sizeof(data_t)*plan->numPixel);
+  data_t* dataCopy = xmalloc(sizeof(data_t)*plan->numPixel);
   memcpy(dataCopy, data, plan->numPixel*sizeof(data_t));
   if (plan->numdims==2)
     {
@@ -876,7 +878,7 @@ void circunshift_cpu(struct dfwavelet_plan_s* plan, data_t *data) {
   }
 
   // Copy data
-  data_t* dataCopy = malloc(sizeof(data_t)*plan->numPixel);
+  data_t* dataCopy = xmalloc(sizeof(data_t)*plan->numPixel);
   memcpy(dataCopy, data, plan->numPixel*sizeof(data_t));
   if (plan->numdims==2)
     {
