@@ -106,7 +106,7 @@ int main_ecalib(int argc, char* argv[argc])
 	if (conf.weighting) {
 
 		conf.numsv      = -1.;
-		conf.threshold  =   0;
+		conf.threshold  = 0.;
 		conf.percentsv  = -1.;
 		conf.orthiter   = false;
 	}
@@ -168,12 +168,15 @@ int main_ecalib(int argc, char* argv[argc])
 
 
 	// FIXME: we should scale the data
-
-	(conf.usegpu ? num_init_gpu : num_init)();
+	bart_use_gpu = conf.usegpu;
+	num_init_gpu_support();
 
         if ((conf.var < 0.) && (conf.weighting || (conf.crop < 0.))) {
 
-		const char* toolbox = getenv("TOOLBOX_PATH");
+		const char* toolbox = getenv("BART_TOOLBOX_PATH");
+		// support old environment variable:
+		if (NULL == toolbox)
+			toolbox = getenv("TOOLBOX_PATH");
 
 		conf.var = estvar_calreg(toolbox, conf.kdims, cal_dims, cal_data);
 	}
