@@ -26,6 +26,7 @@
 #include "misc/mmio.h"
 #include "misc/opts.h"
 #include "misc/mri.h"
+#include "misc/debug.h"
 
 #include "num/init.h"
 #include "num/qform.h"
@@ -45,12 +46,22 @@
 
 
 
-static const char usage_str[] = "<trajectory> <data> [<qf>]";
 static const char help_str[] = "Estimate gradient delays from radial data.";
 
 
 int main_estdelay(int argc, char* argv[argc])
 {
+	const char* traj_file = NULL;
+	const char* data_file = NULL;
+	const char* qf_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &traj_file, "trajectory"),
+		ARG_INFILE(true, &data_file, "data"),
+		ARG_OUTFILE(false, &qf_file, "qf"),
+	};
+
 	bool do_ring = false;
 	struct ring_conf conf = ring_defaults;
 
@@ -62,7 +73,7 @@ int main_estdelay(int argc, char* argv[argc])
 		OPT_FLOAT('r', &conf.size, "r", "[RING] Central region size"),
 	};
 
-	cmdline(&argc, argv, 2, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
@@ -71,38 +82,33 @@ int main_estdelay(int argc, char* argv[argc])
 
 
 	long tdims[DIMS];
-	const complex float* traj = load_cfl(argv[1], DIMS, tdims);
+	const complex float* traj = load_cfl(traj_file, DIMS, tdims);
 
-	long tdims1[DIMS];
-	md_select_dims(DIMS, ~MD_BIT(1), tdims1, tdims);
-
-	complex float* traj1 = md_alloc(DIMS, tdims1, CFL_SIZE);
-	md_slice(DIMS, MD_BIT(1), (long[DIMS]){ 0 }, tdims, traj1, traj, CFL_SIZE);
 
 	int N = tdims[2];
 
 	float angles[N];
-	for (int i = 0; i < N; i++)
-		angles[i] = M_PI + atan2f(crealf(traj1[3 * i + 0]), crealf(traj1[3 * i + 1]));
 
+	traj_radial_angles(N, angles, tdims, traj);
 
-	if (do_ring) {
+	float dc_shift = traj_radial_dcshift(tdims, traj);
+	float scale = traj_radial_dk(tdims, traj);
 
-		assert(0 == tdims[1] % 2);
+	// Warn on unexpected shifts: != 0.5 for even number of samples, != 0 for odd number of sampled
+	if (1 == tdims[1] % 2) {
 
-		md_slice(DIMS, MD_BIT(1), (long[DIMS]){ [1] = tdims[1] / 2 }, tdims, traj1, traj, CFL_SIZE);
+		debug_printf(DP_WARN, "odd number of samples\n");
+		if (fabsf(dc_shift/scale - 0.0f) > 0.0001)
+			debug_printf(DP_WARN, "DC is shifted by: %f [sample], 1 sample = %f [1/FOV]\n", dc_shift, scale);
 
-		for (int i = 0; i < N; i++)
-			if (0. != cabsf(traj1[3 * i]))
-				error("Nominal trajectory must be centered for RING.\n");
+	} else if (fabsf(dc_shift/scale - 0.5f) > 0.0001) {
+
+		debug_printf(DP_WARN, "DC is shifted by: %f [sample], 1 sample = %f [1/FOV]\n", dc_shift, scale);
 	}
 
 
-	md_free(traj1);
-
-
 	long full_dims[DIMS];
-	const complex float* full_in = load_cfl(argv[2], DIMS, full_dims);
+	const complex float* full_in = load_cfl(data_file, DIMS, full_dims);
 
 	// Remove not needed dimensions
 	long dims[DIMS];
@@ -124,6 +130,7 @@ int main_estdelay(int argc, char* argv[argc])
 		// Block and Uecker, ISMRM 19:2816 (2001)
 
 		float delays[N];
+
 		radial_self_delays(N, delays, angles, dims, in);
 
 		/* We allow an arbitrary quadratic form to account for
@@ -133,6 +140,12 @@ int main_estdelay(int argc, char* argv[argc])
 
 		fit_quadratic_form(qf, N, angles, delays);
 
+		if (0 == tdims[1] % 2) {
+
+			qf[0] += 0.5;
+			qf[1] += 0.5;
+		}
+
 	} else {
 
 		/* RING method
@@ -142,18 +155,24 @@ int main_estdelay(int argc, char* argv[argc])
 		ring(&conf, qf, N, angles, dims, in);
 	}
 
+	qf[0] -= dc_shift / scale;
+	qf[1] -= dc_shift / scale;
+
+
 	bart_printf("%f:%f:%f\n", qf[0], qf[1], qf[2]);
 
-	if (NULL != argv[3]) {
+	if (NULL != qf_file) {
 
-		long qf_dims[1] = { 3 };
+		long qf_dims[DIMS];
+		md_singleton_dims(DIMS, qf_dims);
+		qf_dims[0] = 3;
 
-		complex float* oqf = create_cfl(argv[3], 1, qf_dims);
+		complex float* oqf = create_cfl(qf_file, DIMS, qf_dims);
 
 		for (int i = 0; i < 3; i++)
 			oqf[i] = qf[i];
 
-		unmap_cfl(1, qf_dims, oqf);
+		unmap_cfl(DIMS, qf_dims, oqf);
 	}
 
 	unmap_cfl(DIMS, full_dims, full_in);
