@@ -26,12 +26,18 @@
 #include "calib/estvar.h"
 
 
-static const char usage_str[] = "<kspace>";
 static const char help_str[] = "Estimate the noise variance assuming white Gaussian noise.";
 
 
 int main_estvar(int argc, char* argv[argc])
 {
+	const char* ksp_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &ksp_file, "kspace"),
+	};
+
 	long calsize_dims[3]  = { 24, 24, 24};
 	long kernel_dims[3]   = {  6,  6,  6};
 
@@ -43,14 +49,14 @@ int main_estvar(int argc, char* argv[argc])
 		OPT_VEC3('R', &calsize_dims, "", "()"),
 	};
 
-	cmdline(&argc, argv, 1, 1, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	int N = DIMS;
 	long kspace_dims[N];
 
-	complex float* kspace = load_cfl(argv[1], N, kspace_dims);
+	complex float* kspace = load_cfl(ksp_file, N, kspace_dims);
 
 	for (int idx = 0; idx < 3; idx++) {
 
@@ -58,7 +64,12 @@ int main_estvar(int argc, char* argv[argc])
 		calsize_dims[idx] = (kspace_dims[idx] == 1) ? 1 : calsize_dims[idx];
 	}
 
-	float variance = estvar_kspace(N, kernel_dims, calsize_dims, kspace_dims, kspace);
+	const char* toolbox = getenv("BART_TOOLBOX_PATH");
+	// support old environment variable:
+	if (NULL == toolbox)
+		toolbox = getenv("TOOLBOX_PATH");
+
+	float variance = estvar_kspace(toolbox, N, kernel_dims, calsize_dims, kspace_dims, kspace);
 
 	unmap_cfl(N, kspace_dims, kspace);
 
