@@ -17,36 +17,47 @@
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
+#include "misc/opts.h"
 
 #ifndef DIMS
 #define DIMS 16
 #endif
 
 
-static const char usage_str[] = "bitmask <input> <output>";
-static const char help_str[] = "Flip (reverse) dimensions specified by the {bitmask}.\n";
+static const char help_str[] = "Flip (reverse) dimensions specified by the {bitmask}.";
 
 
 
-int main_flip(int argc, char* argv[])
+int main_flip(int argc, char* argv[argc])
 {
-	mini_cmdline(&argc, argv, 3, usage_str, help_str);
+	unsigned long flags = 0;
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_ULONG(true, &flags, "bitmask"),
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
+	const struct opt_s opts[] = { };
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	int N = DIMS;
 	long dims[N];
-	complex float* idata = load_cfl(argv[2], N, dims);
-	complex float* odata = create_cfl(argv[3], N, dims);
-
-	unsigned long flags = atoi(argv[1]);
+	complex float* idata = load_cfl(in_file, N, dims);
+	complex float* odata = create_cfl(out_file, N, dims);
 
 	md_flip(N, dims, flags, odata, idata, sizeof(complex float));
 
 	unmap_cfl(N, dims, idata);
 	unmap_cfl(N, dims, odata);
 
-	exit(0);
+	return 0;
 }
 
 
