@@ -6,6 +6,7 @@ enum NETWORK_SELECT {
 	NETWORK_MNIST,
 	NETWORK_RESBLOCK,
 	NETWORK_VARNET,
+	NETWORK_TENSORFLOW,
 };
 
 extern struct network_s* get_default_network(enum NETWORK_SELECT net);
@@ -22,3 +23,6 @@ extern struct opt_s unet_reco_opts[];
 extern const int N_unet_segm_opts;
 extern struct opt_s unet_segm_opts[];
 
+extern const int N_tensorflow_opts;
+extern struct opt_s network_tensorflow_opts[];
+
