@@ -1,5 +1,5 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2016. Martin Uecker.
+ * Copyright 2016-2019. Martin Uecker.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  */
@@ -14,7 +14,7 @@ struct operator_p_s;
 
 #ifndef ITER_CONF_S
 #define ITER_CONF_S
-typedef struct iter_conf_s { TYPEID* TYPEID; } iter_conf;
+typedef struct iter_conf_s { TYPEID* TYPEID; float alpha; } iter_conf;
 #endif
 
 struct iter_monitor_s;
@@ -36,9 +36,11 @@ struct iter_conjgrad_conf {
 	unsigned int maxiter;
 	float l2lambda;
 	float tol;
+
+	long Bo;
+	long Bi;
 };
 
-extern DEF_TYPEID(iter_conjgrad_conf);
 
 
 struct iter_landweber_conf {
@@ -50,7 +52,6 @@ struct iter_landweber_conf {
 	float tol;
 };
 
-extern DEF_TYPEID(iter_landweber_conf);
 
 struct iter_ist_conf {
 
@@ -63,7 +64,6 @@ struct iter_ist_conf {
 	float tol;
 };
 
-extern DEF_TYPEID(iter_ist_conf);
 
 struct iter_fista_conf {
 
@@ -74,9 +74,9 @@ struct iter_fista_conf {
 	float continuation;
 	_Bool hogwild;
 	float tol;
+	int maxeigen_iter;
 };
 
-extern DEF_TYPEID(iter_fista_conf);
 
 
 struct iter_chambolle_pock_conf {
@@ -90,9 +90,9 @@ struct iter_chambolle_pock_conf {
 	float decay;
 	float tol;
 	_Bool fast;
+	int maxeigen_iter;
 };
 
-extern DEF_TYPEID(iter_chambolle_pock_conf);
 
 struct iter_admm_conf {
 
@@ -107,7 +107,7 @@ struct iter_admm_conf {
 	_Bool dynamic_tau;
 	_Bool relative_norm;
 	_Bool hogwild;
-	
+
 	double ABSTOL;
 	double RELTOL;
 
@@ -122,7 +122,6 @@ struct iter_admm_conf {
 	_Bool fast;
 };
 
-extern DEF_TYPEID(iter_admm_conf);
 
 
 struct iter_pocs_conf {
@@ -132,8 +131,6 @@ struct iter_pocs_conf {
 	unsigned int maxiter;
 };
 
-extern DEF_TYPEID(iter_pocs_conf);
-
 struct iter_niht_conf {
 
 	INTERFACE(iter_conf);
@@ -143,7 +140,6 @@ struct iter_niht_conf {
 	_Bool do_warmstart;
 };
 
-extern DEF_TYPEID(iter_niht_conf);
 
 extern const struct iter_conjgrad_conf iter_conjgrad_defaults;
 extern const struct iter_landweber_conf iter_landweber_defaults;
@@ -173,7 +169,6 @@ struct iter_call_s {
 	iter_conf* _conf;
 };
 
-extern DEF_TYPEID(iter_call_s);
 
 
 #endif
