@@ -27,7 +27,7 @@ extern void lad2(unsigned int N, const struct lad_conf* conf,
 		const long x_dims[N], complex float* x,
 		const long y_dims[N], const complex float* y);
 
-extern const struct operator_s* lad2_create(const struct lad_conf* conf,
+extern const struct operator_p_s* lad2_create(const struct lad_conf* conf,
 		italgo_fun2_t italgo, iter_conf* iconf,
 		const float* init,
 		const struct linop_s* model_op,
