@@ -92,7 +92,9 @@ const struct operator_p_s* lsqr2_create(const struct lsqr_conf* conf,
 	if (NULL == model_op) {
 
 		assert(0 < num_funs);
-		iov = linop_domain(prox_linops[0]);
+
+		iov = operator_p_domain(prox_funs[0]);
+		//iov = linop_domain(prox_linops[0]);
 		data->model_op = NULL;
 
 	} else {
@@ -136,7 +138,9 @@ const struct operator_p_s* lsqr2_create(const struct lsqr_conf* conf,
 	if (conf->it_gpu) {
 
 		debug_printf(DP_DEBUG1, "lsqr: add GPU wrapper\n");
-		itop_op = operator_p_gpu_wrapper(itop_op);
+		auto tmp = operator_p_gpu_wrapper(itop_op);
+		operator_p_free(itop_op);
+		itop_op = tmp;
 	}
 
 	const struct operator_p_s* lsqr_op;
