@@ -14,6 +14,7 @@
 #include <math.h>
 
 #include "num/multind.h"
+#include "num/multiplace.h"
 #include "num/flpmath.h"
 #include "num/ops_p.h"
 #include "num/ops.h"
@@ -22,11 +23,10 @@
 #include "num/gpuops.h"
 #endif
 
-#include "linops/linop.h"
-
 #include "iter/iter.h"
 
 #include "misc/misc.h"
+#include "misc/debug.h"
 
 #include "prox.h"
 
@@ -39,176 +39,132 @@
  */
 
 
-/**
- * Data for computing prox_normaleq_fun:
- * Proximal function for f(z) = 0.5 || y - A z ||_2^2.
- *
- * @param op operator that applies A^H A
- * @param cgconf conf file for conjugate gradient iter interface
- * @param adj A^H y
- * @param size size of z
- */
-struct prox_normaleq_data {
-
-	INTERFACE(operator_data_t);
-
-	const struct linop_s* op;
-	void* cgconf;
-	float* adj;
-
-	long size;
-};
-
-static DEF_TYPEID(prox_normaleq_data);
-
-
-
-/**
- * Proximal function for f(z) = 0.5 || y - A z ||_2^2.
- * Solution is (A^H A + (1/mu) I)z = A^H y + (1/mu)(x_plus_u)
- *
- * @param prox_data should be of type prox_normaleq_data
- * @param mu proximal penalty
- * @param z output
- * @param x_plus_u input
- */
-static void prox_normaleq_fun(const operator_data_t* prox_data, float mu, float* z, const float* x_plus_u)
-{
-	auto pdata = CAST_DOWN(prox_normaleq_data, prox_data);
-
-	if (0 == mu) {
-
-		md_copy(1, MD_DIMS(pdata->size), z, x_plus_u, FL_SIZE);
-
-	} else {
-
-		float rho = 1. / mu;
-
-		float* b = md_alloc_sameplace(1, MD_DIMS(pdata->size), FL_SIZE, x_plus_u);
-
-		md_copy(1, MD_DIMS(pdata->size), b, pdata->adj, FL_SIZE);
-		md_axpy(1, MD_DIMS(pdata->size), b, rho, x_plus_u);
-
-		if (NULL == pdata->op->norm_inv) {
-
-			struct iter_conjgrad_conf* cg_conf = pdata->cgconf;
-			cg_conf->l2lambda = rho;
-
-			iter_conjgrad(CAST_UP(cg_conf), pdata->op->normal, NULL, pdata->size, z, (float*)b, NULL);
-
-		} else {
-
-			linop_norm_inv_unchecked(pdata->op, rho, (complex float*)z, (const complex float*)b);
-		}
-
-		md_free(b);
-	}
-}
-
-static void prox_normaleq_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
-{
-	prox_normaleq_fun(_data, mu, (float*)dst, (const float*)src);
-}
-
-static void prox_normaleq_del(const operator_data_t* _data)
-{
-	auto pdata = CAST_DOWN(prox_normaleq_data, _data);
-
-	xfree(pdata->cgconf);
-	md_free(pdata->adj);
-	xfree(pdata);
-}
-
-const struct operator_p_s* prox_normaleq_create(const struct linop_s* op, const complex float* y)
-{
-	PTR_ALLOC(struct prox_normaleq_data, pdata);
-	SET_TYPEID(prox_normaleq_data, pdata);
-	PTR_ALLOC(struct iter_conjgrad_conf, cgconf);
-
-	*cgconf = iter_conjgrad_defaults;
-	cgconf->maxiter = 10;
-	cgconf->l2lambda = 0;
-
-	pdata->cgconf = PTR_PASS(cgconf);
-	pdata->op = op;
-
-	pdata->size = 2 * md_calc_size(linop_domain(op)->N, linop_domain(op)->dims);
-
-	pdata->adj = md_alloc_sameplace(1, &(pdata->size), FL_SIZE, y);
-
-	linop_adjoint_unchecked(op, (complex float*)pdata->adj, y);
-
-	return operator_p_create(linop_domain(op)->N, linop_domain(op)->dims,
-			linop_domain(op)->N, linop_domain(op)->dims,
-			CAST_UP(PTR_PASS(pdata)), prox_normaleq_apply, prox_normaleq_del);
-}
-
 
 /**
  * Data for computing prox_leastsquares_fun:
- * Proximal function for f(z) = lambda / 2 || y - z ||_2^2.
+ * Proximal function for f(z) = lambda / 2 || W * (y - z) ||_2^2.
  *
  * @param y
  * @param lambda regularization
  * @param size size of z
  */
-struct prox_leastsquares_data {
+struct prox_weighted_leastsquares_data {
 
 	INTERFACE(operator_data_t);
 
-	const float* y;
-	float lambda;
+	int N;
 
-	long size;
+	const long* dims;
+	struct multiplace_array_s* y;
+
+	const long* wdims;
+	struct multiplace_array_s* W;
+
+	float lambda;
 };
 
-static DEF_TYPEID(prox_leastsquares_data);
+static DEF_TYPEID(prox_weighted_leastsquares_data);
 
 
 /**
- * Proximal function for f(z) = lambda / 2 || y - z ||_2^2.
- * Solution is z =  (mu * lambda * y + x_plus_u) / (mu * lambda + 1)
+ * Proximal function for f(z) = lambda / 2 || W (y - z) ||_2^2.
+ * Solution is z =  (mu * lambda * y + x_plus_u) / (mu * lambda * |W|^2 + 1)
  *
- * @param prox_data should be of type prox_leastsquares_data
+ * @param prox_data should be of type prox_weighted_leastsquares_data
  * @param mu proximal penalty
  * @param z output
  * @param x_plus_u input
  */
-static void prox_leastsquares_fun(const operator_data_t* prox_data, float mu, float* z, const float* x_plus_u)
+static void prox_weighted_leastsquares_fun(const operator_data_t* prox_data, float mu, complex float* z, const complex float* x_plus_u)
 {
-	auto pdata = CAST_DOWN(prox_leastsquares_data, prox_data);
+	auto pdata = CAST_DOWN(prox_weighted_leastsquares_data, prox_data);
 
-	md_copy(1, MD_DIMS(pdata->size), z, x_plus_u, FL_SIZE);
+	int N = pdata->N;
+	const long* dims = pdata->dims;
+	const long* wdims = pdata->wdims;
+
+	const complex float* y = multiplace_read(pdata->y, z);
+	const complex float* W_sqr = multiplace_read(pdata->W, z);
+
+	if (z != x_plus_u)
+		md_copy(N, dims, z, x_plus_u, CFL_SIZE);
 
 	if (0 != mu) {
 
-		if (NULL != pdata->y)
-			md_axpy(1, MD_DIMS(pdata->size), z, pdata->lambda * mu, pdata->y);
+		if (NULL == W_sqr) {
 
-		md_smul(1, MD_DIMS(pdata->size), z, z, 1. / (mu * pdata->lambda + 1));
+			if (NULL != y)
+				md_zaxpy(N, dims, z, pdata->lambda * mu, multiplace_read(pdata->y, z));
+
+			md_zsmul(N, dims, z, z, 1. / (mu * pdata->lambda + 1));
+
+		} else {
+
+			complex float* tmp = md_alloc_sameplace(N, wdims, CFL_SIZE, z);
+			md_zsmul(N, wdims, tmp, W_sqr, mu * pdata->lambda);
+
+			if (NULL != y)
+				md_zfmac2(N, dims, MD_STRIDES(N, dims, CFL_SIZE), z, MD_STRIDES(N, dims, CFL_SIZE), y, MD_STRIDES(N, wdims, CFL_SIZE), tmp);
+
+			complex float* ones = md_alloc_sameplace(N, wdims, CFL_SIZE, z);
+			md_zfill(N, wdims, ones, 1);
+
+			md_zadd(N, wdims, tmp, tmp, ones);
+			md_zdiv(N, wdims, tmp, ones, tmp);
+
+			md_free(ones);
+
+			md_zmul2(N, dims, MD_STRIDES(N, dims, CFL_SIZE), z, MD_STRIDES(N, dims, CFL_SIZE), z, MD_STRIDES(N, wdims, CFL_SIZE), tmp);
+			md_free(tmp);
+		}
 	}
 }
 
-static void prox_leastsquares_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
+
+static void prox_weighted_leastsquares_del(const operator_data_t* _data)
 {
-	prox_leastsquares_fun(_data, mu, (float*)dst, (const float*)src);
+	auto data = CAST_DOWN(prox_weighted_leastsquares_data, _data);
+	multiplace_free(data->y);
+
+	xfree(data->dims);
+	xfree(data->wdims);
+
+	xfree(data);
 }
 
-static void prox_leastsquares_del(const operator_data_t* _data)
+const struct operator_p_s* prox_weighted_leastsquares_create(int N, const long dims[N], float lambda, const complex float* y, unsigned long flags, const complex float* W)
 {
-	xfree(CAST_DOWN(prox_leastsquares_data, _data));
-}
+	PTR_ALLOC(struct prox_weighted_leastsquares_data, pdata);
+	SET_TYPEID(prox_weighted_leastsquares_data, pdata);
 
-const struct operator_p_s* prox_leastsquares_create(unsigned int N, const long dims[N], float lambda, const complex float* y)
-{
-	PTR_ALLOC(struct prox_leastsquares_data, pdata);
-	SET_TYPEID(prox_leastsquares_data, pdata);
+	pdata->N = N;
+	pdata->dims = ARR_CLONE(long[N], dims);
 
-	pdata->y = (const float*)y;
+	pdata->y = (NULL == y) ? NULL : multiplace_move(N, dims, CFL_SIZE, y);
 	pdata->lambda = lambda;
-	pdata->size = md_calc_size(N, dims) * 2;
 
-	return operator_p_create(N, dims, N, dims, CAST_UP(PTR_PASS(pdata)), prox_leastsquares_apply, prox_leastsquares_del);
+	long wdims[N];
+	md_select_dims(N, flags, wdims, dims);
+	pdata->wdims = ARR_CLONE(long[N], wdims);
+
+	if (NULL != W) {
+
+		complex float* tmp = md_alloc_sameplace(N, wdims, CFL_SIZE, W);
+		md_zmulc(N, wdims, tmp, W, W);
+
+		pdata->W = multiplace_move_F(N, wdims, CFL_SIZE, tmp);
+
+	} else {
+
+		pdata->W = NULL;
+	}
+
+	return operator_p_create(N, dims, N, dims, CAST_UP(PTR_PASS(pdata)), prox_weighted_leastsquares_fun, prox_weighted_leastsquares_del);
+}
+
+const struct operator_p_s* prox_leastsquares_create(int N, const long dims[N], float lambda, const complex float* y)
+{
+	return prox_weighted_leastsquares_create(N, dims, lambda, y, 0, NULL);
 }
 
 
@@ -267,7 +223,7 @@ static void prox_l2norm_del(const operator_data_t* _data)
 	xfree(CAST_DOWN(prox_l2norm_data, _data));
 }
 
-const struct operator_p_s* prox_l2norm_create(unsigned int N, const long dims[N], float lambda)
+const struct operator_p_s* prox_l2norm_create(int N, const long dims[N], float lambda)
 {
 	PTR_ALLOC(struct prox_l2norm_data, pdata);
 	SET_TYPEID(prox_l2norm_data, pdata);
@@ -279,6 +235,7 @@ const struct operator_p_s* prox_l2norm_create(unsigned int N, const long dims[N]
 }
 
 
+
 /**
  * Data for computing prox_l2ball_fun:
  * Proximal function for f(z) = Ind{ || y - z ||_2 < eps }
@@ -369,7 +326,7 @@ static void prox_l2ball_del(const operator_data_t* _data)
 	xfree(data);
 }
 
-const struct operator_p_s* prox_l2ball_create(unsigned int N, const long dims[N], float eps, const complex float* y)
+const struct operator_p_s* prox_l2ball_create(int N, const long dims[N], float eps, const complex float* y)
 {
 	PTR_ALLOC(struct prox_l2ball_data, pdata);
 	SET_TYPEID(prox_l2ball_data, pdata);
@@ -484,7 +441,7 @@ static void prox_zero_del(const operator_data_t* _data)
 	xfree(CAST_DOWN(prox_zero_data, _data));
 }
 
-const struct operator_p_s* prox_zero_create(unsigned int N, const long dims[N])
+const struct operator_p_s* prox_zero_create(int N, const long dims[N])
 {
 	PTR_ALLOC(struct prox_zero_data, pdata);
 	SET_TYPEID(prox_zero_data, pdata);
@@ -497,67 +454,6 @@ const struct operator_p_s* prox_zero_create(unsigned int N, const long dims[N])
 
 
 
-/**
- * Data for computing prox_lineq_fun:
- * Proximal function for f(z) = 1{ A z = y }
- * Assumes AA^T = I
- * Solution is z = x - A^T A x + A^T y
- *
- * @param op linop A
- * @param adj A^H y
- * @param tmp tmp
- */
-struct prox_lineq_data {
-
-	INTERFACE(operator_data_t);
-
-	const struct linop_s* op;
-	complex float* adj;
-	complex float* tmp;
-};
-
-static DEF_TYPEID(prox_lineq_data);
-
-static void prox_lineq_apply(const operator_data_t* _data, float mu, complex float* dst, const complex float* src)
-{
-	UNUSED(mu);
-	auto pdata = CAST_DOWN(prox_lineq_data, _data);
-
-	const struct linop_s* op = pdata->op;
-	linop_normal(op, linop_domain(op)->N, linop_domain(op)->dims, pdata->tmp, src);
-
-	md_zsub(linop_domain(op)->N, linop_domain(op)->dims, dst, src, pdata->tmp);
-	md_zadd(linop_domain(op)->N, linop_domain(op)->dims, dst, dst, pdata->adj);
-}
-
-static void prox_lineq_del(const operator_data_t* _data)
-{
-	auto pdata = CAST_DOWN(prox_lineq_data, _data);
-
-	md_free(pdata->adj);
-	md_free(pdata->tmp);
-	xfree(pdata);
-}
-
-const struct operator_p_s* prox_lineq_create(const struct linop_s* op, const complex float* y)
-{
-	PTR_ALLOC(struct prox_lineq_data, pdata);
-	SET_TYPEID(prox_lineq_data, pdata);
-
-	unsigned int N = linop_domain(op)->N;
-	const long* dims = linop_domain(op)->dims;
-
-	pdata->op = op;
-
-	pdata->adj = md_alloc_sameplace(N, dims, CFL_SIZE, y);
-	linop_adjoint(op, N, dims, pdata->adj, N, linop_codomain(op)->dims, y);
-
-	pdata->tmp = md_alloc_sameplace(N, dims, CFL_SIZE, y);
-
-	return operator_p_create(N, dims, N, dims, CAST_UP(PTR_PASS(pdata)), prox_lineq_apply, prox_lineq_del);
-}
-
-
 /**
  * Data for computing prox_ineq_fun:
  * Proximal function for f(z) = 1{ z <= b }
@@ -629,7 +525,7 @@ static const struct operator_p_s* prox_ineq_create(unsigned int N, const long di
  * Proximal function for less than or equal to:
  * f(z) = 1{z <= b}
  */
-const struct operator_p_s* prox_lesseq_create(unsigned int N, const long dims[N], const complex float* b)
+const struct operator_p_s* prox_lesseq_create(int N, const long dims[N], const complex float* b)
 {
 	return prox_ineq_create(N, dims, b, 0., false);
 }
@@ -638,7 +534,7 @@ const struct operator_p_s* prox_lesseq_create(unsigned int N, const long dims[N]
  * Proximal function for greater than or equal to:
  * f(z) = 1{z >= b}
  */
-const struct operator_p_s* prox_greq_create(unsigned int N, const long dims[N], const complex float* b)
+const struct operator_p_s* prox_greq_create(int N, const long dims[N], const complex float* b)
 {
 	return prox_ineq_create(N, dims, b, 0., true);
 }
@@ -647,7 +543,7 @@ const struct operator_p_s* prox_greq_create(unsigned int N, const long dims[N],
  * Proximal function for nonnegative orthant
  * f(z) = 1{z >= 0}
  */
-const struct operator_p_s* prox_nonneg_create(unsigned int N, const long dims[N])
+const struct operator_p_s* prox_nonneg_create(int N, const long dims[N])
 {
 	return prox_ineq_create(N, dims, NULL, 0., true);
 }
@@ -656,7 +552,7 @@ const struct operator_p_s* prox_nonneg_create(unsigned int N, const long dims[N]
  * Proximal function for greater than or equal to a scalar:
  * f(z) = 1{z >= a}
  */
-const struct operator_p_s* prox_zsmax_create(unsigned int N, const long dims[N], float a)
+const struct operator_p_s* prox_zsmax_create(int N, const long dims[N], float a)
 {
 	return prox_ineq_create(N, dims, NULL, a, true);
 }
@@ -688,7 +584,7 @@ static void prox_rvc_del(const operator_data_t* _data)
 /*
  * Proximal function for real-value constraint
  */
-const struct operator_p_s* prox_rvc_create(unsigned int N, const long dims[N])
+const struct operator_p_s* prox_rvc_create(int N, const long dims[N])
 {
 	PTR_ALLOC(struct prox_rvc_data, pdata);
 	SET_TYPEID(prox_rvc_data, pdata);
@@ -699,118 +595,3 @@ const struct operator_p_s* prox_rvc_create(unsigned int N, const long dims[N])
 
 
 
-
-
-struct auto_norm_s {
-
-	INTERFACE(operator_data_t);
-
-	long flags;
-	const struct operator_p_s* op;
-};
-
-DEF_TYPEID(auto_norm_s);
-
-static void auto_norm_apply(const operator_data_t* _data, float mu, complex float* y, const complex float* x)
-{
-	auto data = CAST_DOWN(auto_norm_s, _data);
-
-	auto io = operator_p_domain(data->op);
-
-	unsigned int N = io->N;
-
-	long sdims[N];
-	md_select_dims(N, ~data->flags, sdims, io->dims);
-
-	long sstrs[N];
-	md_calc_strides(N, sstrs, sdims, CFL_SIZE);
-
-#if 0
-	complex float* scale = md_alloc_sameplace(N, sdims, CFL_SIZE, x);
-
-	md_zrss(N, io->dims, data->flags, scale, x);
-	md_zdiv2(N, io->dims, io->strs, y, io->strs, x, sstrs, scale);
-#else
-	long pos[N];
-	for (unsigned int i = 0; i < N; i++)
-		pos[i] = 0;
-
-	long xdims[N];
-	md_select_dims(N, data->flags, xdims, io->dims);
-
-	complex float* scale = md_alloc(N, sdims, CFL_SIZE);
-
-
-
-	do {
-		MD_ACCESS(N, sstrs, pos, scale) = md_znorm2(N, xdims, io->strs, &MD_ACCESS(N, io->strs, pos, x));
-
-		complex float val = MD_ACCESS(N, sstrs, pos, scale);
-
-		md_zsmul2(N, xdims, io->strs, &MD_ACCESS(N, io->strs, pos, y),
-				io->strs, &MD_ACCESS(N, io->strs, pos, x),
-				(0. == val) ? 0. : (1. / val));
-
-	} while (md_next(N, io->dims, ~data->flags, pos));
-#endif
-
-	operator_p_apply_unchecked(data->op, mu, y, y);	// FIXME: input == output
-#if 0
-	md_zmul2(N, io->dims, io->strs, y, io->strs, y, sstrs, scale);
-#else
-	for (unsigned int i = 0; i < N; i++)
-		pos[i] = 0;
-
-	do {
-		complex float val = MD_ACCESS(N, sstrs, pos, scale);
-
-		md_zsmul2(N, xdims, io->strs, &MD_ACCESS(N, io->strs, pos, y),
-				io->strs, &MD_ACCESS(N, io->strs, pos, y),
-				val);
-
-	} while (md_next(N, io->dims, ~data->flags, pos));
-#endif
-
-	md_free(scale);
-}
-
-static void auto_norm_del(const operator_data_t* _data)
-{
-	auto data = CAST_DOWN(auto_norm_s, _data);
-
-	operator_p_free(data->op);
-
-	xfree(data);
-}
-
-
-
-/* This functor normalizes data along given dimensions and undoes
- * the normalization after application of the operator.
- *
- */
-const struct operator_p_s* op_p_auto_normalize(const struct operator_p_s* op, long flags)
-{
-	PTR_ALLOC(struct auto_norm_s, data);
-	SET_TYPEID(auto_norm_s, data);
-
-	data->flags = flags;
-	data->op = operator_p_ref(op);
-
-	auto io_in = operator_p_domain(op);
-	auto io_out = operator_p_codomain(op);
-
-	unsigned int N = io_in->N;
-	long dims[N];
-	md_copy_dims(N, dims, io_in->dims);
-
-	long strs[N];
-	md_calc_strides(N, strs, dims, CFL_SIZE);
-
-	assert(N == io_out->N);
-	assert(md_check_compat(N, 0L, dims, io_out->dims));
-	assert(md_check_compat(N, 0L, strs, io_in->strs));
-	assert(md_check_compat(N, 0L, strs, io_out->strs));
-
-	return operator_p_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), auto_norm_apply, auto_norm_del);
-}
