@@ -95,6 +95,9 @@ extern struct linop_s* linop_chainN_F(int N, struct linop_s* x[N]);
 extern struct linop_s* linop_stack(int D, int E, const struct linop_s* a, const struct linop_s* b);
 extern struct linop_s* linop_stack_FF(int D, int E, const struct linop_s* a, const struct linop_s* b);
 
+extern struct linop_s* linop_stack_cod(int N, struct linop_s* lops[N], int stack_dim);
+extern struct linop_s* linop_stack_cod_F(int N, struct linop_s* lops[N], int stack_dim);
+
 
 struct iovec_s;
 extern const struct iovec_s* linop_domain(const struct linop_s* x);
@@ -105,12 +108,15 @@ extern const struct linop_s* linop_clone(const struct linop_s* x);
 extern const struct linop_s* linop_get_adjoint(const struct linop_s* x);
 extern const struct linop_s* linop_get_normal(const struct linop_s* x);
 
-extern struct linop_s* linop_loop(unsigned int D, const long dims[D], struct linop_s* op);
+extern struct linop_s* linop_loop(int D, const long dims[D], struct linop_s* op);
+extern struct linop_s* linop_loop_F(int D, const long dims[D], struct linop_s* op);
 extern struct linop_s* linop_copy_wrapper2(int DI, const long istrs[DI], int DO, const long ostrs[DO],  struct linop_s* op);
 extern struct linop_s* linop_copy_wrapper(unsigned int D, const long istrs[D], const long ostrs[D], struct linop_s* op);
 extern struct linop_s* linop_gpu_wrapper(struct linop_s* op);
+extern struct linop_s* linop_vptr_wrapper(struct vptr_hint_s* hint, struct linop_s* op);
 
-
+extern struct linop_s* linop_assign_gpu(const struct linop_s* op, int device);
+extern struct linop_s* linop_assign_gpu_F(const struct linop_s* op, int device);
 
 
 extern struct linop_s* linop_null_create2(unsigned int NO, const long odims[NO], const long ostrs[NO], unsigned int NI, const long idims[NI], const long istrs[NI]);
@@ -128,6 +134,7 @@ extern struct linop_s* linop_reshape_out_F(const struct linop_s* op, unsigned in
 
 
 extern struct linop_s* graph_optimize_linop(const struct linop_s* op);
+extern const struct operator_s* graph_optimize_operator_linop_F(const struct operator_s* op);
 
 void operator_linops_apply_joined_unchecked(unsigned int N, const struct operator_s* op[N], complex float* dst[N], const complex float* src);
 
