@@ -1,5 +1,6 @@
 /* Copyright 2014. The Regents of the University of California.
- * Copyright 2015-2021. Uecker Lab. University Medical Center Göttingen
+ * Copyright 2015-2022. Uecker Lab. University Medical Center Göttingen
+ * Copyright 2021-2023. Institute of Biomedical Imaging. TU Graz.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
@@ -9,6 +10,7 @@
 
 #include <complex.h>
 #include <assert.h>
+#include <math.h>
 
 #include "misc/misc.h"
 #include "misc/debug.h"
@@ -41,6 +43,7 @@ struct cdiag_s {
 	const long* dstrs;
 
 	struct multiplace_array_s* diag;
+	struct multiplace_array_s* normal;
 	bool rmul;
 };
 
@@ -67,14 +70,28 @@ static void cdiag_adjoint(const linop_data_t* _data, complex float* dst, const c
 
 static void cdiag_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
-	cdiag_apply(_data, dst, src);
-	cdiag_adjoint(_data, dst, dst);
+	const auto data = CAST_DOWN(cdiag_s, _data);
+
+	#pragma omp critical (linop_cdiag_normal)
+	if (NULL == data->normal) {
+
+		const complex float* diag = multiplace_read(data->diag, src);
+
+		complex float* tmp = md_alloc_sameplace(data->N, data->ddims, CFL_SIZE, dst);
+		(data->rmul ? md_zrmul : md_zmulc)(data->N, data->ddims, tmp, diag, diag);
+		data->normal = multiplace_move_F(data->N, data->ddims, CFL_SIZE, tmp);
+	}
+
+	const complex float* normal = multiplace_read(data->normal, src);
+
+	(data->rmul ? md_zrmul2 : md_zmul2)(data->N, data->dims, data->strs, dst, data->strs, src, data->dstrs, normal);
 }
 
 static void cdiag_free(const linop_data_t* _data)
 {
 	const auto data = CAST_DOWN(cdiag_s, _data);
 
+	multiplace_free(data->normal);
 	multiplace_free(data->diag);
 	xfree(data->ddims);
 	xfree(data->dims);
@@ -109,6 +126,7 @@ static struct linop_s* linop_gdiag_create(unsigned int N, const long dims[N], un
 	data->dstrs = *PTR_PASS(dstrs);
 
 	data->diag = (NULL == diag) ? NULL : multiplace_move(N, data->ddims, CFL_SIZE, diag);
+	data->normal = NULL;
 
 	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), cdiag_apply, cdiag_adjoint, cdiag_normal, NULL, cdiag_free);
 }
@@ -151,10 +169,42 @@ void linop_gdiag_set_diag(const struct linop_s* lop, int N, const long ddims[N],
 	assert(md_check_equal_dims(N, ddims, data->ddims, ~0));
 
 	multiplace_free(data->diag);
+	multiplace_free(data->normal);
 
+	data->normal = NULL;
 	data->diag = multiplace_move(N, data->ddims, CFL_SIZE, diag);
 }
 
+void linop_gdiag_set_diag_F(const struct linop_s* lop, int N, const long ddims[N], const complex float* diag)
+{
+	auto _data = linop_get_data(lop);
+	auto data = CAST_DOWN(cdiag_s, _data);
+
+	assert(data->N == (unsigned int)N);
+	assert(md_check_equal_dims(N, ddims, data->ddims, ~0));
+
+	multiplace_free(data->diag);
+	multiplace_free(data->normal);
+
+	data->normal = NULL;
+	data->diag = multiplace_move_F(N, data->ddims, CFL_SIZE, diag);
+}
+
+void linop_gdiag_set_diag_ref(const struct linop_s* lop, int N, const long ddims[N], const complex float* diag)
+{
+	auto _data = linop_get_data(lop);
+	auto data = CAST_DOWN(cdiag_s, _data);
+
+	assert(data->N == (unsigned int)N);
+	assert(md_check_equal_dims(N, ddims, data->ddims, ~0));
+
+	multiplace_free(data->diag);
+	multiplace_free(data->normal);
+
+	data->normal = NULL;
+	data->diag = multiplace_move_wrapper(N, data->ddims, CFL_SIZE, diag);
+}
+
 struct scale_s {
 
 	INTERFACE(linop_data_t);
@@ -317,32 +367,6 @@ struct linop_s* linop_zreal_create(unsigned int N, const long dims[N])
 
 
 
-struct identity_data_s {
-
-	INTERFACE(linop_data_t);
-
-	const struct iovec_s* domain;
-};
-
-static DEF_TYPEID(identity_data_s);
-
-static void identity_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
-{
-	const auto data = CAST_DOWN(identity_data_s, _data);
-	const struct iovec_s* domain = data->domain;
-
-	md_copy2(domain->N, domain->dims, domain->strs, dst, domain->strs, src, CFL_SIZE);
-}
-
-static void identity_free(const linop_data_t* _data)
-{
-	const auto data = CAST_DOWN(identity_data_s, _data);
-
-	iovec_free(data->domain);
-
-	xfree(data);
-}
-
 /**
  * Create an Identity linear operator: I x
  * @param N number of dimensions
@@ -350,12 +374,16 @@ static void identity_free(const linop_data_t* _data)
  */
 struct linop_s* linop_identity_create(unsigned int N, const long dims[N])
 {
-	PTR_ALLOC(struct identity_data_s, data);
-	SET_TYPEID(identity_data_s, data);
+	auto op = operator_identity_create(N, dims);
+	auto result = linop_from_ops(op, op, op, NULL);
+	operator_free(op);
 
-	data->domain = iovec_create(N, dims, CFL_SIZE);
+	return result;
+}
 
-	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), identity_apply, identity_apply, identity_apply, NULL, identity_free);
+bool linop_is_identity(const struct linop_s* lop)
+{
+	return check_simple_copy(lop->forward);
 }
 
 
@@ -848,8 +876,8 @@ struct permute_op_s {
 	int N;
 	const long* idims;
 	const long* odims;
-	const unsigned int* order;
-	const unsigned int* order_adj;
+	const int* order;
+	const int* order_adj;
 };
 
 static DEF_TYPEID(permute_op_s);
@@ -888,7 +916,7 @@ static void permute_free(const linop_data_t* _data)
 }
 
 
-struct linop_s* linop_permute_create(unsigned int N, const unsigned int order[N], const long idims[N])
+struct linop_s* linop_permute_create(unsigned int N, const int order[N], const long idims[N])
 {
 	long odims[N];
 	md_permute_dims(N, order, odims, idims);
@@ -904,8 +932,8 @@ struct linop_s* linop_permute_create(unsigned int N, const unsigned int order[N]
 
 	long* tidims = *TYPE_ALLOC(long[N]);
 	long* todims = *TYPE_ALLOC(long[N]);
-	unsigned int* torder = *TYPE_ALLOC(unsigned int[N]);
-	unsigned int* torder_adj = *TYPE_ALLOC(unsigned int[N]);
+	int* torder = *TYPE_ALLOC(int[N]);
+	int* torder_adj = *TYPE_ALLOC(int[N]);
 
 	for (unsigned int i = 0; i < N; i++) {
 
@@ -925,7 +953,7 @@ struct linop_s* linop_permute_create(unsigned int N, const unsigned int order[N]
 	return linop_create(N, odims, N, idims, CAST_UP(PTR_PASS(data)), permute_forward, permute_adjoint, permute_normal, NULL, permute_free);
 }
 
-extern struct linop_s* linop_permute_create(unsigned int N, const unsigned int order[__VLA(N)], const long idims[N]);
+extern struct linop_s* linop_permute_create(unsigned int N, const int order[N], const long idims[N]);
 
 struct transpose_op_s {
 
@@ -951,6 +979,18 @@ static void transpose_forward(const linop_data_t* _data, complex float* dst, con
 	md_transpose(data->N, data->a, data->b, odims, dst, data->dims, src, CFL_SIZE);
 }
 
+static void transpose_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	auto data = CAST_DOWN(transpose_op_s, _data);
+
+	long odims[data->N];
+	md_copy_dims(data->N, odims, data->dims);
+	odims[data->a] = data->dims[data->b];
+	odims[data->b] = data->dims[data->a];
+
+	md_transpose(data->N, data->a, data->b, data->dims, dst, odims, src, CFL_SIZE);
+}
+
 static void transpose_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
 	auto data = CAST_DOWN(transpose_op_s, _data);
@@ -990,7 +1030,8 @@ struct linop_s* linop_transpose_create(int N, int a, int b, const long dims[N])
 	odims[a] = idims[b];
 	odims[b] = idims[a];
 
-	return linop_create(N, odims, N, idims, CAST_UP(PTR_PASS(data)), transpose_forward, transpose_forward, transpose_normal, NULL, transpose_free);
+	return linop_create(N, odims, N, idims, CAST_UP(PTR_PASS(data)),
+			transpose_forward, transpose_adjoint, transpose_normal, NULL, transpose_free);
 }
 
 
@@ -999,6 +1040,56 @@ struct linop_s* linop_transpose_create(int N, int a, int b, const long dims[N])
 
 
 
+struct flip_op_s {
+
+	INTERFACE(linop_data_t);
+
+	int N;
+	unsigned long flags;
+	const long* dims;
+};
+
+static DEF_TYPEID(flip_op_s);
+
+static void flip_forward(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	auto data = CAST_DOWN(flip_op_s, _data);
+	md_flip(data->N, data->dims, data->flags, dst, src, CFL_SIZE);
+}
+
+static void flip_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	auto data = CAST_DOWN(flip_op_s, _data);
+
+	md_copy(data->N, data->dims, dst, src, CFL_SIZE);
+}
+
+static void flip_free(const linop_data_t* _data)
+{
+	auto data = CAST_DOWN(flip_op_s, _data);
+
+	xfree(data->dims);
+	xfree(data);
+}
+
+
+struct linop_s* linop_flip_create(int N, const long dims[N], unsigned long flags)
+{
+	PTR_ALLOC(struct flip_op_s, data);
+	SET_TYPEID(flip_op_s, data);
+
+	data->N = N;
+	data->flags = flags;
+
+	long* ndims = *TYPE_ALLOC(long[N]);
+	md_copy_dims(N, ndims, dims);
+	data->dims = ndims;
+
+	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), flip_forward, flip_forward, flip_normal, NULL, flip_free);
+}
+
+
+
 struct operator_matrix_s {
 
 	INTERFACE(linop_data_t);
@@ -1433,7 +1524,6 @@ struct fft_linop_s {
 	const struct operator_s* frw;
 	const struct operator_s* adj;
 
-	bool center;
 	float nscale;
 
 	int N;
@@ -1447,6 +1537,14 @@ static void fft_linop_apply(const linop_data_t* _data, complex float* out, const
 {
 	const auto data = CAST_DOWN(fft_linop_s, _data);
 
+#ifdef USE_CUDA
+	if (cuda_ondevice(out)) {
+
+		operator_apply(data->frw, data->N, data->dims, out, data->N, data->dims, in);
+		return;
+	}
+#endif
+
 	if (in != out)
 		md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
 
@@ -1457,6 +1555,14 @@ static void fft_linop_adjoint(const linop_data_t* _data, complex float* out, con
 {
 	const auto data = CAST_DOWN(fft_linop_s, _data);
 
+#ifdef USE_CUDA
+	if (cuda_ondevice(out)) {
+
+		operator_apply(data->adj, data->N, data->dims, out, data->N, data->dims, in);
+		return;
+	}
+#endif
+
 	if (in != out)
 		md_copy2(data->N, data->dims, data->strs, out, data->strs, in, CFL_SIZE);
 
@@ -1480,19 +1586,22 @@ static void fft_linop_normal(const linop_data_t* _data, complex float* out, cons
 {
 	const auto data = CAST_DOWN(fft_linop_s, _data);
 
-	if (data->center)
-		md_copy(data->N, data->dims, out, in, CFL_SIZE);
-	else
-		md_zsmul(data->N, data->dims, out, in, data->nscale);
+	md_zsmul(data->N, data->dims, out, in, data->nscale);
 }
 
-
-static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned int flags, bool forward, bool center, bool measure)
+/**
+ * Uncentered forward Fourier transform linear operator
+ *
+ * @param N number of dimensions
+ * @param dims dimensions of input
+ * @param flags bitmask of the dimensions to apply the Fourier transform
+ */
+struct linop_s* linop_fft_create(int N, const long dims[N], unsigned long flags)
 {
 	const struct operator_s* plan = NULL;
 	const struct operator_s* iplan = NULL;
 
-	if (measure) {
+	if (use_fftw_wisdom) {
 
 		plan = fft_measure_create(N, dims, flags, true, false);
 		iplan = fft_measure_create(N, dims, flags, true, true);
@@ -1514,8 +1623,6 @@ static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned
 	data->adj = iplan;
 	data->N = N;
 
-	data->center = center;
-
 	data->dims = *TYPE_ALLOC(long[N]);
 	md_copy_dims(N, data->dims, dims);
 
@@ -1526,76 +1633,71 @@ static struct linop_s* linop_fft_create_priv(int N, const long dims[N], unsigned
 	md_select_dims(N, flags, fft_dims, dims);
 	data->nscale = (float)md_calc_size(N, fft_dims);
 
-	lop_fun_t apply = forward ? fft_linop_apply : fft_linop_adjoint;
-	lop_fun_t adjoint = forward ? fft_linop_adjoint : fft_linop_apply;
+	return linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), fft_linop_apply, fft_linop_adjoint, fft_linop_normal, NULL, fft_linop_free);
+}
 
-	struct linop_s* lop =  linop_create(N, dims, N, dims, CAST_UP(PTR_PASS(data)), apply, adjoint, fft_linop_normal, NULL, fft_linop_free);
+/**
+ * Uncentered backward Fourier transform linear operator
+ *
+ * @param N number of dimensions
+ * @param dims dimensions of input
+ * @param flags bitmask of the dimensions to apply the Fourier transform
+ */
+struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned long flags)
+{
+	struct linop_s* lop_fft = linop_fft_create(N, dims, flags);
+	struct linop_s* lop_ifft = (struct linop_s*)linop_get_adjoint(lop_fft);
 
-	if (center) {
+	linop_free(lop_fft);
+	return lop_ifft;
+}
 
-		complex float* fftmod_mat = md_alloc(N, fft_dims, CFL_SIZE);
-		complex float* fftmodk_mat = md_alloc(N, fft_dims, CFL_SIZE);
 
-		// we need fftmodk only because we want to apply scaling only once
+static struct linop_s* linop_fft_create_priv(	int N, const long dims[N], unsigned long flags,
+						bool forward, unsigned long center_flags, unsigned long unitary_flags,
+						unsigned long pre_flags, const complex float* pre_diag,
+						unsigned long post_flags, const complex float* post_diag)
+{
+	flags = flags & md_nontriv_dims(N, dims);
+	center_flags = center_flags & flags;
+	unitary_flags = unitary_flags & flags;
 
-		complex float one[1] = { 1. };
-		md_fill(N, fft_dims, fftmod_mat, one, CFL_SIZE);
+	struct linop_s* lop = (forward ? linop_fft_create : linop_ifft_create)(N, dims, flags);
 
-		if (forward)
-			fftmod(N, fft_dims, flags, fftmodk_mat, fftmod_mat);
-		else
-			ifftmod(N, fft_dims, flags, fftmodk_mat, fftmod_mat);
+	if (0 != (center_flags | unitary_flags)) {
 
-		fftscale(N, fft_dims, flags, fftmod_mat, fftmodk_mat);
+		long fft_mod_dims[N];
+		md_select_dims(N, center_flags, fft_mod_dims, dims);
 
-		struct linop_s* mod = linop_cdiag_create(N, dims, flags, fftmod_mat);
-		struct linop_s* modk = linop_cdiag_create(N, dims, flags, fftmodk_mat);
+		complex float* fftmod_a = md_alloc(N, fft_mod_dims, CFL_SIZE);
 
-		struct linop_s* tmp = linop_chain(mod, lop);
+		long fft_scale_dims[N];
+		md_select_dims(N, unitary_flags, fft_scale_dims, dims);
+		md_zfill(N, fft_mod_dims, fftmod_a, 1. / sqrtf(sqrtf(md_calc_size(N, fft_scale_dims))));
 
-		linop_free(lop);
-		linop_free(mod);
+		(forward ? fftmod : ifftmod)(N, fft_mod_dims, center_flags & flags, fftmod_a, fftmod_a);
 
-		lop = linop_chain(tmp, modk);
+		struct linop_s* lop_fftmod = linop_cdiag_create(N, dims, center_flags, fftmod_a);
+		md_free(fftmod_a);
 
-		linop_free(tmp);
-		linop_free(modk);
+		lop = linop_chain_FF(lop, linop_clone(lop_fftmod));
+		lop = linop_chain_FF(lop_fftmod, lop);
+	}
 
-		md_free(fftmod_mat);
-		md_free(fftmodk_mat);
+	if (NULL != post_diag) {
+
+		lop = linop_chain_FF(lop, linop_cdiag_create(N, dims, post_flags, post_diag));
+	}
+
+	if (NULL != pre_diag) {
+
+		lop = linop_chain_FF(linop_cdiag_create(N, dims, pre_flags, pre_diag), lop);
 	}
 
 	return lop;
 }
 
 
-/**
- * Uncentered forward Fourier transform linear operator
- *
- * @param N number of dimensions
- * @param dims dimensions of input
- * @param flags bitmask of the dimensions to apply the Fourier transform
- * @param gpu use gpu
- */
-struct linop_s* linop_fft_create(int N, const long dims[N], unsigned int flags)
-{
-	return linop_fft_create_priv(N, dims, flags, true, false, false);
-}
-
-
-/**
- * Uncentered inverse Fourier transform linear operator
- *
- * @param N number of dimensions
- * @param dims dimensions of input
- * @param flags bitmask of the dimensions to apply the Fourier transform
- */
-struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned int flags)
-{
-	return linop_fft_create_priv(N, dims, flags, false, false, false);
-}
-
-
 /**
  * Centered forward Fourier transform linear operator
  *
@@ -1605,7 +1707,7 @@ struct linop_s* linop_ifft_create(int N, const long dims[N], unsigned int flags)
  */
 struct linop_s* linop_fftc_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, true, true, false);
+	return linop_fft_create_priv(N, dims, flags, true, flags, flags, 0, NULL, 0, NULL);
 }
 
 
@@ -1618,24 +1720,49 @@ struct linop_s* linop_fftc_create(int N, const long dims[N], unsigned int flags)
  */
 struct linop_s* linop_ifftc_create(int N, const long dims[N], unsigned int flags)
 {
-	return linop_fft_create_priv(N, dims, flags, false, true, false);
+	return linop_fft_create_priv(N, dims, flags, false, flags, flags, 0, NULL, 0, NULL);
 }
 
-
 /**
- * Uncentered forward Fourier transform linear operator
- * fft_measure_create used
+ * Centered forward Fourier transform linear operator chained with cdiag operator from both sides
  *
  * @param N number of dimensions
  * @param dims dimensions of input
  * @param flags bitmask of the dimensions to apply the Fourier transform
- * @param gpu use gpu
+ * @param center_flags bitmask for centerd fft
+ * @param unitary_flags bitmask for unitary scaling
+ * @param pre_flags bitmask of the dimensions of the pre-chained diag operator
+ * @param pre_diag diagonal of the pre-chained diag operator
+ * @param post_flags bitmask of the dimensions of the post-chained diag operator
+ * @param post_diag diagonal of the post-chained diag operator
  */
-struct linop_s* linop_fft_create_measure(int N, const long dims[N], unsigned int flags)
+struct linop_s* linop_fft_generic_create(int N, const long dims[N], unsigned long flags, unsigned long center_flags, unsigned long unitary_flags,
+					   unsigned long pre_flag, const complex float* pre_diag, unsigned long post_flag, const complex float* post_diag)
 {
-	return linop_fft_create_priv(N, dims, flags, true, false, true);
+	return linop_fft_create_priv(N, dims, flags, true, center_flags, unitary_flags, pre_flag, pre_diag, post_flag, post_diag);
 }
 
+
+/**
+ * Centered inverse Fourier transform linear operator chained with cdiag operator from both sides
+ *
+ * @param N number of dimensions
+ * @param dims dimensions of input
+ * @param flags bitmask of the dimensions to apply the Fourier transform
+ * @param center_flags bitmask for centerd fft
+ * @param unitary_flags bitmask for unitary scaling
+ * @param pre_flags bitmask of the dimensions of the pre-chained diag operator
+ * @param pre_diag diagonal of the pre-chained diag operator
+ * @param post_flags bitmask of the dimensions of the post-chained diag operator
+ * @param post_diag diagonal of the post-chained diag operator
+ */
+struct linop_s* linop_ifft_generic_create(int N, const long dims[N], unsigned long flags, unsigned long center_flags, unsigned long unitary_flags,
+					    unsigned long pre_flag, const complex float* pre_diag, unsigned long post_flag, const complex float* post_diag)
+{
+	return linop_fft_create_priv(N, dims, flags, false, center_flags, unitary_flags, pre_flag, pre_diag, post_flag, post_diag);
+}
+
+
 struct linop_cdf97_s {
 
 	INTERFACE(linop_data_t);
