@@ -1,4 +1,6 @@
 /* Copyright 2013-2015. The Regents of the University of California.
+ * Copyright 2016-2021. Uecker Lab. University Center Göttingen.
+ * Copyright 2022-2023. Institute of Biomedical Imaging. TU Graz.
  * Copyright 2016. Martin Uecker.
  * Copyright 2018. Damien Nguyen.
  * All rights reserved. Use of this source code is governed by
@@ -21,12 +23,18 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <time.h>
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(_WIN32) && !defined(__EMSCRIPTEN__)
 #include <execinfo.h>
 #endif
 
-#include "num/multind.h"
+#ifdef USE_DWARF
+#include <elfutils/libdwfl.h>
+#endif //USE_DWARF
 
+#include "num/multind.h"
+#include "num/mpi_ops.h"
+
+#include "misc/io.h"
 #include "misc/mmio.h"
 #include "misc/cppmap.h"
 #include "misc/misc.h"
@@ -48,6 +56,8 @@ double timestamp(void)
 
 void dump_cfl(const char* name, int D, const long dimensions[D], const complex float* src)
 {
+	io_reserve_output(name);
+
 	complex float* out = create_cfl(name, D, dimensions);
 
 	md_copy(D, dimensions, out, src, sizeof(complex float));
@@ -55,6 +65,17 @@ void dump_cfl(const char* name, int D, const long dimensions[D], const complex f
 	unmap_cfl(D, dimensions, out);
 }
 
+void dump_multi_cfl(const char* name, int N, int D[N], const long* dimensions[N], const _Complex float* x[N])
+{
+	complex float* args[N];
+	create_multi_cfl(name, N, D, dimensions, args);
+
+	for (int i = 0; i < N; i++)
+		md_copy(D[i], dimensions[i], args[i], x[i], sizeof(complex float));
+
+	unmap_multi_cfl(N, D, dimensions, args);
+}
+
 
 int debug_level = -1;
 bool debug_logging = false;
@@ -94,10 +115,18 @@ void debug_vprintf(int level, const char* fmt, va_list ap)
 {
 	if (-1 == debug_level) {
 
-		char* str = getenv("DEBUG_LEVEL");
+		char* str = getenv("BART_DEBUG_LEVEL");
+
+		// support old environment variable:
+		if (NULL == str)
+			str = getenv("DEBUG_LEVEL");
+
 		debug_level = (NULL != str) ? atoi(str) : DP_INFO;
 	}
 
+	if (0 < mpi_get_rank())
+		debug_level = DP_WARN;
+
 	if (level <= debug_level) {
 
 		FILE* ofp = (level < DP_INFO) ? stderr : stdout;
@@ -109,18 +138,23 @@ void debug_vprintf(int level, const char* fmt, va_list ap)
 
 			fprintf(ofp, "[%s] [%s] - ", dt_str, get_level_str(level));
 
-		}
-		else {
+		} else {
+
 			if (level < DP_INFO) {
-				fprintf(ofp, "%s%s: ", (level < DP_INFO ? RED : ""), get_level_str(level));
+
+				char rank[16] = { '\0' };
+
+				if (1 < mpi_get_num_procs())
+					sprintf(rank, " [Rank %d]", mpi_get_rank());
+
+				fprintf(ofp, "%s%s%s: ", (level < DP_INFO ? RED : ""), get_level_str(level), rank);
 			}
 		}
 
 		vfprintf(ofp, fmt, ap);
 
-		if ((!debug_logging) && (level < DP_INFO)) {
+		if ((!debug_logging) && (level < DP_INFO))
 			fprintf(ofp, RESET);
-		}
 
 		fflush(ofp);
 	}
@@ -138,7 +172,7 @@ void debug_printf(int level, const char* fmt, ...)
 
 void debug_vprintf_trace(const char* func_name,
 			 const char* file,
-			 unsigned int line,
+			 int line,
 			 int level,
 			 const char* fmt,
 			 va_list ap)
@@ -146,6 +180,7 @@ void debug_vprintf_trace(const char* func_name,
 {
 #ifndef USE_LOG_BACKEND
 	UNUSED(func_name); UNUSED(file); UNUSED(line);
+
 	debug_vprintf(level, fmt, ap);
 #else
 	char tmp[1024] = { 0 };
@@ -162,7 +197,7 @@ void debug_vprintf_trace(const char* func_name,
 
 void debug_printf_trace(const char* func_name,
 			const char* file,
-			unsigned int line,
+			int line,
 			int level,
 			const char* fmt,
 			...)
@@ -176,7 +211,7 @@ void debug_printf_trace(const char* func_name,
 
 void debug_backtrace(size_t n)
 {
-#ifndef __CYGWIN__
+#if !defined(__CYGWIN__) && !defined(_WIN32) && !defined(__EMSCRIPTEN__)
 	void* ptrs[n + 1];
 	size_t l = backtrace(ptrs, n + 1);
 
@@ -188,6 +223,99 @@ void debug_backtrace(size_t n)
 #endif
 }
 
+#ifdef USE_DWARF
+
+enum {backtrace_size = 50};
+
+static void debug_good_backtrace_file(FILE * stream, int skip)
+{
+#if !defined(__CYGWIN__) && !defined(_WIN32)
+	char* debuginfo_path = NULL;
+
+	Dwfl_Callbacks callbacks = {
+
+		.find_elf = dwfl_linux_proc_find_elf,
+		.find_debuginfo = dwfl_standard_find_debuginfo,
+		.debuginfo_path = &debuginfo_path,
+	};
+
+	Dwfl* dwfl = dwfl_begin(&callbacks);
+
+	if (NULL == dwfl)
+		goto err;
+
+	if (0 != dwfl_linux_proc_report(dwfl, getpid()))
+		goto err;
+
+	if (0 != dwfl_report_end(dwfl, NULL, NULL))
+		goto err;
+
+	void* stack[backtrace_size + 1];
+
+	int stack_size = backtrace(stack, backtrace_size + 1);
+
+	for (int i = skip; i < stack_size; ++i) {
+
+		Dwarf_Addr addr = (Dwarf_Addr)stack[i];
+
+		Dwfl_Module* module = dwfl_addrmodule(dwfl, addr);
+		const char* name = dwfl_module_addrname(module, addr);
+
+		Dwfl_Line* dwfl_line;
+		int line = -1;
+		const char* file = NULL;
+
+		if ((dwfl_line = dwfl_module_getsrc(module, addr))) {
+
+			Dwarf_Addr addr2;
+			file = dwfl_lineinfo(dwfl_line, &addr2, &line, NULL, NULL, NULL);
+		}
+
+		fprintf(stream, "%d: %p %s", i - skip, stack[i], name);
+
+		if (file)
+			fprintf(stream, " at %s:%d", file, line);
+
+		fprintf(stream, "\n");
+	}
+
+	dwfl_end(dwfl);
+	return;
+err:
+	debug_printf(DP_WARN, "Backtrace failed\n.");
+#else
+	debug_printf(DP_WARN, "no backtrace on cygwin.");
+#endif
+}
+
+const char* debug_good_backtrace_string(int skip)
+{
+	FILE* fp = tmpfile(); 
+	debug_good_backtrace_file(fp, skip);	
+	rewind(fp);
+
+	char tmp[1024];
+	const char* ret = ptr_printf("");
+
+	while (NULL != fgets(tmp, 1023, fp)) {
+
+		const char* tmp2 = ptr_printf("%s%s", ret, tmp);
+		xfree(ret);
+		ret = tmp2;
+	}
+
+	fclose(fp);
+
+	return ret;
+}
+
+void debug_good_backtrace(int skip)
+{
+	if (mpi_is_main_proc())
+		debug_good_backtrace_file(stderr, skip);
+}
+
+#endif // USE_DWARF
 
 
 void debug_trace(const char* fmt, ...)
@@ -218,3 +346,4 @@ extern void __cyg_profile_func_exit(void *this_fn, void *call_site)
 	debug_trace("LEAVE %p\n", this_fn);
 }
 #endif
+
