@@ -2,7 +2,7 @@
  * Copyright 2015-2018. Martin Uecker.
  * Copyright 2017. University of Oxford.
  * Copyright 2017-2018. Damien Nguyen
- * Copyright 2019. Uecker Lab, University Medical Center Göttingen.
+ * Copyright 2019-2022. Uecker Lab, University Medical Center Göttingen.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
@@ -15,6 +15,7 @@
 
 #define _GNU_SOURCE
 #include <stdlib.h>
+#include <stddef.h>
 #include <assert.h>
 #include <complex.h>
 #include <stdbool.h>
@@ -23,6 +24,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <math.h>
+#include <limits.h>
 
 #ifdef BART_WITH_PYTHON
 #include <Python.h>
@@ -35,7 +37,7 @@
 
 
 #ifndef isnanf
-#define isnanf(X) isnan(X)  
+#define isnanf(X) isnan(X)
 #endif
 
 
@@ -109,10 +111,14 @@ void error(const char* fmt, ...)
 
 	va_end(ap);
 
+#ifdef USE_DWARF
+	debug_good_backtrace(1); // Skip 1, to not include debug_good_backtrace()
+#endif
+
 	if (error_jumper.initialized)
 		longjmp(error_jumper.buf, 1);
 
-	exit(EXIT_FAILURE);
+	abort();
 }
 
 
@@ -204,7 +210,7 @@ void debug_print_dims(int dblevel, int D, const long dims[D])
 
 void debug_print_dims_trace(const char* func_name,
 			    const char* file,
-			    unsigned int line,
+			    int line,
 			    int dblevel,
 			    int D,
 			    const long dims[D])
@@ -227,7 +233,7 @@ int parse_cfl(complex float res[1], const char* str)
 	char* tail;
 	float re = strtof(str, &tail);
 	float im = 0.;
-	
+
 	if (str == tail)
 		return -1;
 
@@ -252,9 +258,32 @@ ok:
 }
 
 
+int parse_long(long res[1], const char* str)
+{
+	char* tail;
+	long l = strtol(str, &tail, 10);
+
+	if ('\0' != tail[0])
+		return -1;
+	res[0] = l;
+	return 0;
+}
 
 
-void quicksort(int N, int ord[N], const void* data, quicksort_cmp_t cmp)
+int parse_int(int res[1], const char* str)
+{
+	long val;
+	if (0 != parse_long(&val, str))
+		return -1;
+
+	if (val < INT_MIN || val > INT_MAX)
+		error("Value %ld too large for int!\n", val);
+
+	res[0] = (int) val;
+	return 0;
+}
+
+void quicksort(int N, int ord[N], quicksort_cmp_t cmp)
 {
 	if (N < 2)
 		return;
@@ -265,13 +294,13 @@ void quicksort(int N, int ord[N], const void* data, quicksort_cmp_t cmp)
 
 	while (l <= h) {
 
-		if (cmp(data, ord[l], pivot) < 0) {
+		if (cmp(ord[l], pivot) < 0) {
 
 			l++;
 			continue;
 		}
 
-		if (cmp(data, ord[h], pivot) > 0) {
+		if (cmp(ord[h], pivot) > 0) {
 
 			h--;
 			continue;
@@ -286,16 +315,16 @@ void quicksort(int N, int ord[N], const void* data, quicksort_cmp_t cmp)
 	}
 
 	if (h + 1 > 0)
-		quicksort(h + 1, ord, data, cmp);
+		quicksort(h + 1, ord, cmp);
 
 	if (N > l)
-		quicksort(N - l, ord + l, data, cmp);
+		quicksort(N - l, ord + l, cmp);
 }
 
 
 
 /**
- * Quickselect adapted from §8.5 in Numerical Recipes in C, 
+ * Quickselect adapted from §8.5 in Numerical Recipes in C,
  * The Art of Scientific Computing
  * Second Edition, William H. Press, 1992.
  * Sort descending of an array of floats, stop at k largest element.
@@ -306,49 +335,62 @@ void quicksort(int N, int ord[N], const void* data, quicksort_cmp_t cmp)
  *
  * @returns the k-th largest float in the array
  *
- * @note In-place sort. The input array contents are not preserved in their original order. 
+ * @note In-place sort. The input array contents are not preserved in their original order.
  */
-float quickselect(float *arr, unsigned int n, unsigned int k) {
-	unsigned long i,ir,j,l,mid;
+float quickselect(float *arr, int n, int k)
+{
+	long i, ir, j, l, mid;
 	float a;
-   
-	l=0;
-	ir=n-1;
+
+	l = 0;
+	ir = n - 1;
+
 	for(;;) {
-		if (ir <= l+1) { 
-			if (ir == l+1 && arr[ir] > arr[l]) {
-				SWAP(arr[l],arr[ir], float);
-			}
+
+		if (ir <= l + 1) {
+
+			if ((ir == l + 1) && (arr[ir] > arr[l]))
+				SWAP(arr[l], arr[ir]);
+
 			return arr[k];
 		}
-		else {
-			mid=(l+ir) >> 1; 
-			SWAP(arr[mid],arr[l+1], float);
-			if (arr[l] < arr[ir]) {
-				SWAP(arr[l],arr[ir], float);
-			}
-			if (arr[l+1] < arr[ir]) {
-				SWAP(arr[l+1],arr[ir], float);
-			}
-			if (arr[l] < arr[l+1]) {
-				SWAP(arr[l],arr[l+1], float);
-			}
-			i=l+1; 
-			j=ir;
-			a=arr[l+1];
 
-			for (;;) { 
-				do i++; while (arr[i] > a); 
-				do j--; while (arr[j] < a); 
-				if (j < i) break; 
-				SWAP(arr[i],arr[j], float);
-			} 
-			arr[l+1]=arr[j]; 
-			arr[j]=a;
-      
-			if (j >= k) ir=j-1; 
-			if (j <= k) l=i;
+		mid = (l + ir) / 2;
+
+		SWAP(arr[mid], arr[l + 1]);
+
+		if (arr[l] < arr[ir])
+			SWAP(arr[l], arr[ir]);
+
+		if (arr[l + 1] < arr[ir])
+			SWAP(arr[l + 1], arr[ir]);
+
+		if (arr[l] < arr[l + 1])
+			SWAP(arr[l], arr[l + 1]);
+
+		i = l + 1;
+		j = ir;
+		a = arr[l + 1];
+
+		for (;;) {
+
+			do i++; while (arr[i] > a);
+			do j--; while (arr[j] < a);
+
+			if (j < i)
+				break;
+
+			SWAP(arr[i], arr[j]);
 		}
+
+		arr[l + 1] = arr[j];
+		arr[j] = a;
+
+		if (j >= k)
+			ir = j - 1;
+
+		if (j <= k)
+			l = i;
 	}
 }
 
@@ -359,48 +401,62 @@ float quickselect(float *arr, unsigned int n, unsigned int k) {
  * Possibly faster for application to complex arrays.
  *
  */
-float quickselect_complex(complex float *arr, unsigned int n, unsigned int k) {
-	unsigned long i,ir,j,l,mid;
+float quickselect_complex(complex float* arr, int n, int k)
+{
+	long i, ir, j, l, mid;
 	float a;
 	complex float ca;
-   
-	l=0;
-	ir=n-1;
+
+	l = 0;
+	ir = n - 1;
+
 	for(;;) {
-		if (ir <= l+1) { 
-			if (ir == l+1 && cabsf(arr[ir]) > cabsf(arr[l])) {
-				SWAP(arr[l],arr[ir], complex float);
-			}
+
+		if (ir <= l + 1) {
+
+			if ((ir == l + 1) && (cabsf(arr[ir]) > cabsf(arr[l])))
+				SWAP(arr[l], arr[ir]);
+
 			return cabsf(arr[k]);
 		}
-		else {
-			mid=(l+ir) >> 1; 
-			SWAP(arr[mid],arr[l+1], complex float);
-			if (cabsf(arr[l]) < cabsf(arr[ir])) {
-				SWAP(arr[l],arr[ir], complex float);
-			}
-			if (cabsf(arr[l+1]) < cabsf(arr[ir])) {
-				SWAP(arr[l+1],arr[ir], complex float);
-			}
-			if (cabsf(arr[l]) < cabsf(arr[l+1])) {
-				SWAP(arr[l],arr[l+1], complex float);
-			}
-			i=l+1; 
-			j=ir;
-			a=cabsf(arr[l+1]);
-			ca = arr[l+1];
-			for (;;) { 
-				do i++; while (cabsf(arr[i]) > a); 
-				do j--; while (cabsf(arr[j]) < a); 
-				if (j < i) break; 
-				SWAP(arr[i],arr[j], complex float);
-			} 
-			arr[l+1]=arr[j]; 
-			arr[j]=ca;
-      
-			if (j >= k) ir=j-1; 
-			if (j <= k) l=i;
+
+		mid = (l + ir) / 2;
+
+		SWAP(arr[mid], arr[l + 1]);
+
+		if (cabsf(arr[l]) < cabsf(arr[ir]))
+			SWAP(arr[l], arr[ir]);
+
+		if (cabsf(arr[l + 1]) < cabsf(arr[ir]))
+			SWAP(arr[l + 1], arr[ir]);
+
+		if (cabsf(arr[l]) < cabsf(arr[l + 1]))
+			SWAP(arr[l], arr[l + 1]);
+
+		i = l + 1;
+		j = ir;
+		a = cabsf(arr[l + 1]);
+		ca = arr[l + 1];
+
+		for (;;) {
+
+			do i++; while (cabsf(arr[i]) > a);
+			do j--; while (cabsf(arr[j]) < a);
+
+			if (j < i)
+				break;
+
+			SWAP(arr[i], arr[j]);
 		}
+
+		arr[l + 1] = arr[j];
+		arr[j] = ca;
+
+		if (j >= k)
+			ir = j - 1;
+
+		if (j <= k)
+			l = i;
 	}
 }
 
@@ -461,7 +517,7 @@ static const char* quote(const char* str)
 
 const char* command_line = NULL;
 
-void save_command_line(int argc, char* argv[])
+void* save_command_line(int argc, char* argv[static argc])
 {
 	size_t len = 0;
 	const char* qargv[argc];
@@ -480,82 +536,63 @@ void save_command_line(int argc, char* argv[])
 
 		strcpy((*buf) + pos, qargv[i]);
 		pos += strlen(qargv[i]);
-		xfree(qargv[i]);
+
 		(*buf)[pos++] = ' ';
 	}
 
+	// separate loop to not confuse analyzer
+	for (int i = 0; i < argc; i++)
+		xfree(qargv[i]);
+
 	(*buf)[pos] = '\0';
 
-	XFREE(command_line);
+#pragma omp critical (bart_opts_commandline)
+	{
+		if (NULL == command_line) {
 
-	command_line = (*buf);
-}
+			command_line = *buf;
 
+		} else {
 
-
-void mini_cmdline(int* argcp, char* argv[], int expected_args, const char* usage_str, const char* help_str)
-{
-	mini_cmdline_bool(argcp, argv, '\0', expected_args, usage_str, help_str);
-}
-
-
-bool mini_cmdline_bool(int* argcp, char* argv[], char flag_char, int expected_args, const char* usage_str, const char* help_str)
-{
-	bool flag = false;
-	struct opt_s opts[1] = { { flag_char, false, opt_set, &flag, NULL } };
-
-	char* help = strdup(help_str);
-
-	int hlen = strlen(help);
-
-	if ((hlen > 1) && ('\n' == help[hlen - 1]))
-		help[hlen - 1] = '\0';
-
-	int min_args = expected_args;
-	int max_args = expected_args;
-
-	if (expected_args < 0) {
-
-		min_args = -expected_args;
-		max_args = 1000;
+			assert(0 == strcmp(*buf, command_line));
+			xfree(*buf);
+		}
 	}
 
-	cmdline(argcp, argv, min_args, max_args, usage_str, help, 1, opts);
-
-	xfree(help);
-
-	return flag;
+	// FIXME: workaround analyzer detecting a leak
+	return (void*)buf;
 }
 
 
-void print_long(unsigned int D, const long arr[D])
+
+void print_long(int D, const long arr[D])
 {
-	for (unsigned int i = 0; i < D; i++)
+	for (int i = 0; i < D; i++)
 		printf("arr[%i] = %ld\n", i, arr[i]);
 }
 
-void print_float(unsigned int D, const float arr[D])
+void print_float(int D, const float arr[D])
 {
-	for (unsigned int i = 0; i < D; i++)
+	for (int i = 0; i < D; i++)
 		printf("arr[%i] = %f\n", i, arr[i]);
 }
 
-void print_int(unsigned int D, const int arr[D])
+void print_int(int D, const int arr[D])
 {
-	for (unsigned int i = 0; i < D; i++)
+	for (int i = 0; i < D; i++)
 		printf("arr[%i] = %i\n", i, arr[i]);
 }
 
-void print_complex(unsigned int D, const complex float arr[D])
+void print_complex(int D, const complex float arr[D])
 {
-	for (unsigned int i = 0; i < D; i++)
+	for (int i = 0; i < D; i++)
 		printf("arr[%i]: real = %f, imag = %f\n", i, crealf(arr[i]), cimagf(arr[i]));
 }
 
 
-unsigned int bitcount(unsigned long flags)
+int bitcount(unsigned long flags)
 {
-	unsigned int N = 0;
+	int N = 0;
 
 	for (; flags > 0; N++)
 		flags &= (flags - 1);
@@ -577,3 +614,114 @@ bool safe_isfinite(float x)
 }
 
 
+static bool long_mul_overflow_p(long a, long b)
+{
+	bool of = false;
+
+	of |= (a > 0) && (b > 0) && (a > LONG_MAX / b);
+	of |= (a > 0) && (b < 0) && (b < LONG_MIN / a);
+	of |= (a < 0) && (b > 0) && (a < LONG_MIN / b);
+	of |= (a < 0) && (b < 0) && (b < LONG_MAX / a);
+
+	return of;
+}
+
+long io_calc_size(int D, const long dims[D?:1], size_t size)
+{
+	if (0 == D)
+		return size;
+
+	long a = io_calc_size(D - 1, dims + 1, size);
+	long b = dims[0];
+
+	if ((a < 0) || (b < 0))
+		return -1;
+
+	if (long_mul_overflow_p(a, b))
+		return -1;
+
+	return a * b;
+}
+
+const char* ptr_vprintf(const char* fmt, va_list ap)
+{
+	va_list ap1;
+	va_copy(ap1, ap);
+
+	ssize_t len = vsnprintf(NULL, 0, fmt, ap1);
+
+	va_end(ap1);
+
+	if (len < 0)
+		abort();
+
+	PTR_ALLOC(char[len + 1], result);
+
+	vsprintf(*result, fmt, ap);
+
+	return *PTR_PASS(result);
+}
+
+const char* ptr_printf(const char* fmt, ...)
+{
+	va_list ap;
+	va_start(ap, fmt);
+
+	auto result = ptr_vprintf(fmt, ap);
+
+	va_end(ap);
+
+	return result;
+}
+
+const char* ptr_print_dims(int D, const long dims[D])
+{
+	const char* result = ptr_printf("[");
+
+	for (int i = 0; i < D; i++) {
+
+		const char* tmp = ptr_printf("%s%3ld ", result, dims[i]);
+
+		xfree(result);
+
+		result = tmp;
+	}
+
+	const char* tmp = ptr_printf("%s]", result);
+
+	xfree(result);
+
+	return tmp;
+}
+
+
+static const char* spec = "xyzcmnopqsfrtuvw";
+
+char* construct_filename(int D, const long loopdims[D], const long pos[D], const char* prefix, const char* ext)
+{
+	// Prepare output filename
+
+	int len = 0;
+
+	len += snprintf(NULL, 0, "%s", prefix);
+
+	for (int i = 0; i < D; i++)
+		if (1 != loopdims[i])
+			len += snprintf(NULL, 0, "_%c%04ld", spec[i], pos[i]);
+
+	len += snprintf(NULL, 0, ".%s", ext);
+	len++;
+
+	char* name = xmalloc(len);
+	int off = 0;
+
+	off += snprintf(name + off, len - off, "%s", prefix);
+
+	for (int i = 0; i < D; i++)
+		if (1 != loopdims[i])
+			off += snprintf(name + off, len - off, "_%c%04ld", spec[i], pos[i]);
+
+	off += snprintf(name + off, len - off, ".%s", ext);
+
+	return name;
+}
