@@ -65,6 +65,11 @@ extern __attribute__((noreturn)) void error(const char* str, ...);
 #endif
 
 
+#ifdef USE_DWARF
+#undef assert
+#define assert(expr) do { if (!(expr)) error("Assertion '" #expr "' failed in %s:%d\n",  __FILE__, __LINE__); } while (0)
+#endif
+
 extern int error_catcher(int fun(int argc, char* argv[__VLA(argc)]), int argc, char* argv[__VLA(argc)]);
 
 extern int bart_printf(const char* fmt, ...);
@@ -101,7 +106,7 @@ extern void print_complex(int D, const _Complex float arr[__VLA(D)]);
 extern int bitcount(unsigned long flags);
 
 extern const char* command_line;
-extern void save_command_line(int argc, char* argv[__VLA(argc)]);
+extern void* save_command_line(int argc, char* argv[__VLA(argc)]);
 
 extern _Bool safe_isnanf(float x);
 extern _Bool safe_isfinite(float x);
@@ -112,6 +117,13 @@ extern const char* ptr_printf(const char* fmt, ...);
 extern const char* ptr_vprintf(const char* fmt, va_list ap);
 extern const char* ptr_print_dims(int D, const long dims[__VLA(D)]);
 
+extern char* construct_filename(int D, const long loopdims[__VLA(D)], const long pos[__VLA(D)], const char* prefix, const char* ext);
+
+
+#define DEG2RAD(d) ((d) * M_PI / 180.)
+#define RAD2DEG(r) ((r) / M_PI * 180.)
+
+
 #include "misc/cppwrap.h"
 
 #endif // __MISC_H
