@@ -1,11 +1,8 @@
 /* Copyright 2013-2015. The Regents of the University of California.
  * Copyright 2016-2021. Uecker Lab. University Center Göttingen.
+ * Copyright 2023. Institute of Biomedical Imaging. TU Graz.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
- *
- * Authors:
- * 2012-2021 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2015 Jonathan Tamir <jtamir@eecs.berkeley.edu>
  */
 
 #define _GNU_SOURCE
@@ -22,6 +19,14 @@
 #include <unistd.h>
 #include <stdarg.h>
 
+#ifdef USE_MPI
+#include <mpi.h>
+#endif
+
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
 #ifdef _WIN32
 #include "win/mman.h"
 #include "win/open_patch.h"
@@ -29,9 +34,11 @@
 #include <sys/mman.h>
 #endif
 
+#include "num/mpi_ops.h"
 #include "num/multind.h"
 
 #include "misc/misc.h"
+#include "misc/list.h"
 #include "misc/io.h"
 #include "misc/debug.h"
 #include "misc/memcfl.h"
@@ -47,6 +54,33 @@
 #  include <Python.h>
 #endif /* BART_WITH_PYTHON */
 
+#ifndef DIMS
+#define DIMS 16
+#endif
+
+bool mpi_shared_files = false;
+
+struct cfl_loop_desc_s {
+
+	int omp_threads;
+	unsigned long flags;
+	long loop_dims[DIMS];
+	long offs_dims[DIMS];
+};
+
+static struct cfl_loop_desc_s cfl_loop_desc = {
+
+	.omp_threads = 1,
+	.flags = 0UL,
+	.loop_dims =  { [0 ... DIMS - 1] = 1 },
+	.offs_dims =  { [0 ... DIMS - 1] = 0 },
+};
+
+#define MAX_WORKER 128
+#define THREAD_BATCH_LVL 1
+
+static long cfl_loop_index[MAX_WORKER] = { [ 0 ... MAX_WORKER - 1 ] = 0 };
+static list_t unmap_addrs = NULL;
 
 
 static void io_error(const char* fmt, ...)
@@ -58,10 +92,10 @@ static void io_error(const char* fmt, ...)
 	debug_vprintf_trace("error", __FILE__, __LINE__, DP_ERROR, fmt, ap);
 	va_end(ap);
 #else
+	perror(" ");
 	vfprintf(stderr, fmt, ap);
 	va_end(ap);
 	fflush(stderr);
-	perror(" ");
 #endif
 #else
 	if (NULL == PyErr_Occurred()) {
@@ -81,6 +115,294 @@ static void io_error(const char* fmt, ...)
 
 #define err_assert(x)	({ if (!(x)) { debug_printf(DP_ERROR, "%s", #x); exit(EXIT_FAILURE); } })
 
+bool cfl_loop_omp(void)
+{
+#ifdef _OPENMP
+	return (1 < cfl_loop_desc.omp_threads);
+#endif
+	return false;
+}
+
+long cfl_loop_worker_id(void)
+{
+	if (!cfl_loop_desc_active())
+		return 0;
+
+	if (1 < mpi_get_num_procs()) {
+		
+		int procno = mpi_get_rank();
+
+		if(MAX_WORKER <= procno)
+			error("Maximum supported number of MPI workers (%d) exceeded!\n", MAX_WORKER);
+
+		return procno;
+	}
+
+#ifdef _OPENMP
+	if (1 < cfl_loop_desc.omp_threads) {
+
+		int threadno = omp_get_ancestor_thread_num(THREAD_BATCH_LVL);
+
+		if ((THREAD_BATCH_LVL < omp_get_level()) || (1 < omp_get_team_size(0)))
+			debug_printf(DP_WARN, "File accessed in OMP region! Cannot guarantee thread safty!\n");
+
+		if (MAX_WORKER <= threadno)
+			error("Maximum supported number of MPI workers (%d) exceeded!\n", MAX_WORKER);
+
+		return MAX(0, threadno);
+	}
+#endif
+
+	return 0;
+}
+
+long cfl_loop_num_workers(void)
+{
+	if (!cfl_loop_desc_active())
+		return 1;
+
+	if (1 < mpi_get_num_procs()) {
+
+		int nprocs = mpi_get_num_procs();
+
+		if (MAX_WORKER < nprocs)
+			error("Maximum supported number of MPI workers (%d) exceeded!\n", MAX_WORKER);
+
+		return nprocs;
+	}
+
+	if (1 < cfl_loop_desc.omp_threads) {
+		
+		int omp_threads = cfl_loop_desc.omp_threads;
+		
+		if (MAX_WORKER < omp_threads)
+			error("Maximum supported number of OMP workers exceeded!\n");
+
+		return omp_threads;
+	}
+
+	return 1;
+}
+
+
+
+void init_cfl_loop_desc(int D, const long loop_dims[__VLA(D)], long start_dims[__VLA(D)], unsigned long flags, int omp_threads, int index)
+{
+	if (MAX_WORKER < omp_threads)
+		error("Maximum supported number of OMP workers exceeded!\n");
+
+	cfl_loop_desc.omp_threads = omp_threads;
+	cfl_loop_desc.flags = flags;
+
+	md_copy_dims(D, cfl_loop_desc.loop_dims, loop_dims);
+	md_copy_dims(D, cfl_loop_desc.offs_dims, start_dims);
+
+	set_cfl_loop_index(index);
+
+#pragma omp critical(unmap_addrs)
+	if (NULL == unmap_addrs)
+		unmap_addrs = list_create();
+
+}
+
+long cfl_loop_desc_total(void)
+{
+	return md_calc_size(DIMS, cfl_loop_desc.loop_dims);
+}
+
+void set_cfl_loop_index(long index)
+{
+	if (!cfl_loop_desc_active())
+		return;
+
+	int worker_id = cfl_loop_worker_id();
+
+	if (MAX_WORKER < worker_id)
+		error("Worker id exceeds maximum supported workers!\n");
+
+	cfl_loop_index[worker_id] = index;
+
+}
+
+bool cfl_loop_desc_active(void)
+{
+	return cfl_loop_desc.flags > 0;
+}
+
+
+struct cfl_file_desc_s {
+	
+	void* file_addr;
+	void* data_addr;
+
+	int D;
+	long* file_dims; 
+	long* data_dims;
+	long* pos;
+
+	bool writeback;
+};
+
+static bool cmp_addr(const void* _item, const void* _ref)
+{
+	const struct cfl_file_desc_s* item = _item;
+	const complex float* ref = _ref;
+
+	return (item->data_addr == ref);
+}
+
+static void work_buffer_get_pos(int D, const long dims[D], long pos[D], bool output, long index)
+{
+	md_set_dims(D, pos, 0);
+	md_unravel_index(MIN(D, DIMS), pos, cfl_loop_desc.flags, cfl_loop_desc.loop_dims, index);
+
+	if (!output) {
+
+		for (int i = 0; i < MIN(D, DIMS); i++) {
+
+			if (1 == dims[i])
+				pos[i] = 0;
+			else
+				pos[i] += cfl_loop_desc.offs_dims[i];
+
+			if (pos[i] >= dims[i])
+				error("Position in cfl loop out of range!\n");
+		}
+	}
+}
+
+
+/**
+ * Creates working buffer containing slices of the original file
+ * dims contain file_dims on entry and slice dims on return
+**/
+static void* create_worker_buffer(int D, long dims[D], void* addr, bool output)
+{
+	if (!cfl_loop_desc_active())
+		return addr;
+
+	if (output)
+		assert(md_check_equal_dims(MIN(D, DIMS), dims, cfl_loop_desc.loop_dims, cfl_loop_desc.flags));
+
+	long slc_dims[D];
+	md_select_dims(D, ~cfl_loop_desc.flags, slc_dims, dims);
+
+	long slc_strs[D];
+	long tot_strs[D];
+
+	md_calc_strides(D, tot_strs, dims, sizeof(complex float));
+	md_calc_strides(D, slc_strs, slc_dims, sizeof(complex float));
+
+	long pos[D];
+	work_buffer_get_pos(D, dims, pos, output, cfl_loop_index[cfl_loop_worker_id()]);
+
+	void* buf = NULL;
+
+	if (md_check_equal_dims(D, tot_strs, slc_strs, ~cfl_loop_desc.flags)) {
+
+		buf = &(MD_ACCESS(D, tot_strs, pos, (complex float*)addr)); 
+
+		if (output && (1 == mpi_get_num_procs()))
+			output = false;
+	} else {
+
+		buf = md_alloc(D, slc_dims, sizeof(complex float));
+		md_slice(D, cfl_loop_desc.flags, pos, dims, buf, addr, sizeof(complex float));
+	}
+
+	if (!mpi_shared_files && (1 < mpi_get_num_procs())) {
+
+		for (int i = 1; i < mpi_get_num_procs(); i++) {
+
+			complex float* src = NULL;
+			
+			if (mpi_is_main_proc()) {
+
+				long tpos[D];
+				work_buffer_get_pos(D, dims, tpos, output, cfl_loop_index[cfl_loop_worker_id()] + i);
+				src = &(MD_ACCESS(D, tot_strs, tpos, (complex float*)addr)); 
+			}
+
+			mpi_copy2(D, slc_dims, slc_strs, buf, tot_strs, src, sizeof(complex float), 0, i);
+		}
+	}
+
+	PTR_ALLOC(struct cfl_file_desc_s, desc);
+
+	desc->D = D;
+	desc->file_dims = ARR_CLONE(long[D], dims);
+	desc->data_dims = ARR_CLONE(long[D], slc_dims);
+	desc->pos = ARR_CLONE(long[D], pos);
+
+	desc->file_addr = addr;
+	desc->data_addr = buf;
+
+	desc->writeback = output;
+
+#pragma omp critical(unmap_addrs)
+	list_append(unmap_addrs, PTR_PASS(desc));
+
+	md_copy_dims(D, dims, slc_dims);
+
+	return buf;
+}
+
+/**
+ * Check if addr contains a working buffer and return underlying pointer to file
+ * dims contain slice dims on entry and file dims on return
+**/
+static void* free_worker_buffer(int D, long dims[D], void* addr)
+{
+	struct cfl_file_desc_s* desc = NULL;
+
+	if (NULL == unmap_addrs)
+		return addr;
+
+#pragma omp critical(unmap_addrs)
+	desc = list_get_first_item(unmap_addrs, addr, cmp_addr, true);
+
+	if (NULL == desc)
+		return addr;
+
+	assert(D == desc->D);
+
+	if (desc->writeback) {
+
+		if (1 < mpi_get_num_procs()) {
+
+			for (int i = 0; i < mpi_get_num_procs(); i++) {
+
+				complex float* dst = NULL;
+
+				if (mpi_is_main_proc()) {
+
+					long tpos[D];
+					work_buffer_get_pos(D, dims, tpos, true, cfl_loop_index[cfl_loop_worker_id()] + i);
+					dst = &(MD_ACCESS(D, MD_STRIDES(D, desc->file_dims, sizeof(complex float)), tpos, (complex float*)desc->file_addr));
+				}
+
+				mpi_copy2(D, desc->data_dims, MD_STRIDES(D, desc->file_dims, sizeof(complex float)), dst, MD_STRIDES(D, desc->data_dims, sizeof(complex float)), addr, sizeof(complex float), i, 0);
+			}
+		} else {
+
+			md_copy_block(desc->D, desc->pos, desc->file_dims, desc->file_addr, desc->data_dims, desc->data_addr, sizeof(complex float));
+		}
+	}
+
+	if (   (desc->file_addr > desc->data_addr)
+	    || (desc->data_addr > desc->file_addr + (sizeof(complex float) * md_calc_size(desc->D, desc->file_dims))))
+		md_free(desc->data_addr);
+
+	addr = desc->file_addr;
+	md_copy_dims(D, dims, desc->file_dims);
+
+	xfree(desc->file_dims);
+	xfree(desc->data_dims);
+	xfree(desc->pos);
+	xfree(desc);
+
+	return addr;
+}
 
 
 static complex float* load_zra_internal(int fd, const char* name, int D, long dims[D])
@@ -186,7 +508,7 @@ static complex float* create_zra_internal(int ofd, const char* name, int D, cons
 complex float* create_zra(const char* name, int D, const long dims[D])
 {
 	int ofd;
-	if (-1 == (ofd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)))
+	if (-1 == (ofd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
 		io_error("Creating ra file %s\n", name);
 
 	return create_zra_internal(ofd, name, D, dims);
@@ -199,7 +521,7 @@ complex float* create_zshm(const char* name, int D, const long dims[D])
 		error("shm file name does not start with a slash.\n");
 
 	int ofd;
-	if (-1 == (ofd = shm_open(name, O_RDWR /* |O_CREAT */, S_IRUSR|S_IWUSR)))
+	if (-1 == (ofd = shm_open(name, O_RDWR /* |O_CREAT */, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
 		io_error("Creating shm file %s\n", name);
 
 	return create_zra_internal(ofd, name, D, dims);
@@ -211,7 +533,7 @@ float* create_coo(const char* name, int D, const long dims[D])
 {
 	int ofd;
 
-	if (-1 == (ofd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)))
+	if (-1 == (ofd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
 		io_error("Creating coo file %s\n", name);
 
 	if (-1 == write_coo(ofd, D, dims))
@@ -288,52 +610,86 @@ static complex float* create_pipe(int pfd, int D, const long dimensions[D])
 
 complex float* create_cfl(const char* name, int D, const long dimensions[D])
 {
-	io_unlink_if_opened(name);
-	io_register_output(name);
+	long dims[D];
+	md_copy_dims(D, dims, dimensions);
+	
+	if (cfl_loop_desc_active()) {
 
-	enum file_types_e type = file_type(name);
+		if (!md_check_equal_dims(MIN(DIMS, D), dimensions, MD_SINGLETON_DIMS(DIMS), cfl_loop_desc.flags))
+			io_error("Loop over altered dimensions!\n");
+		
+		for (int i = 0; i < MIN(D, DIMS); ++i)
+			dims[i] = (MD_IS_SET(cfl_loop_desc.flags, i)) ? cfl_loop_desc.loop_dims[i] : dimensions[i];
 
-	switch (type) {
+	} else {
 
-	case FILE_TYPE_PIPE:
-		return create_pipe(1, D, dimensions);
-
-	case FILE_TYPE_RA:
-		return create_zra(name, D, dimensions);
-
-	case FILE_TYPE_COO:
-		return create_zcoo(name, D, dimensions);
-
-	case FILE_TYPE_SHM:
-		return create_zshm(name, D, dimensions);
-
-	case FILE_TYPE_MEM:
-		return memcfl_create(name, D, dimensions);
-
-	default:
-		; // handled in this function
+		io_unlink_if_opened(name);
 	}
 
+	io_register_output(name);
 
-	char name_bdy[1024];
-	if (1024 <= snprintf(name_bdy, 1024, "%s.cfl", name))
-		error("Creating cfl file %s\n", name);
+	complex float* addr = NULL;
 
-	char name_hdr[1024];
-	if (1024 <= snprintf(name_hdr, 1024, "%s.hdr", name))
-		error("Creating cfl file %s\n", name);
+#pragma omp critical (bart_file_access)
+	if (mpi_is_main_proc()) {
 
-	int ofd;
-	if (-1 == (ofd = open(name_hdr, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR)))
-		io_error("Creating cfl file %s\n", name);
+		enum file_types_e type = file_type(name);
 
-	if (-1 == write_cfl_header(ofd, NULL, D, dimensions))
-		error("Creating cfl file %s\n", name);
+		switch (type) {
 
-	if (-1 == close(ofd))
-		io_error("Creating cfl file %s\n", name);
+		case FILE_TYPE_PIPE:
+			addr = create_pipe(1, D, dims);
+			break;
 
-	return shared_cfl(D, dimensions, name_bdy);
+		case FILE_TYPE_RA:
+			addr = create_zra(name, D, dims);
+			break;
+
+		case FILE_TYPE_COO:
+			addr = create_zcoo(name, D, dims);
+			break;
+
+		case FILE_TYPE_SHM:
+			addr = create_zshm(name, D, dims);
+			break;
+
+		case FILE_TYPE_MEM:
+			addr = memcfl_create(name, D, dims);
+			break;
+
+		case FILE_TYPE_CFL:;
+	
+			char name_bdy[1024];
+			if (1024 <= snprintf(name_bdy, 1024, "%s.cfl", name))
+				error("Creating cfl file %s\n", name);
+
+			char name_hdr[1024];
+			if (1024 <= snprintf(name_hdr, 1024, "%s.hdr", name))
+				error("Creating cfl file %s\n", name);
+
+			int ofd;
+			if (-1 == (ofd = open(name_hdr, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
+				io_error("Creating cfl file %s\n", name);
+
+			if (-1 == write_cfl_header(ofd, NULL, D, dims))
+				error("Creating cfl file %s\n", name);
+
+			if (-1 == close(ofd))
+				io_error("Creating cfl file %s\n", name);
+
+			addr = shared_cfl(D, dims, name_bdy);
+
+			break;
+		
+		default:
+			error("Unknown filetype!\n");
+		}
+	} else {
+
+		addr = anon_cfl(NULL, D, dims);
+	}
+
+	return create_worker_buffer(D, dims, addr, true);
 }
 
 
@@ -393,80 +749,101 @@ static complex float* load_cfl_internal(const char* name, int D, long dimensions
 
 	io_register_input(name);
 
-	char* filename = NULL;
 	enum file_types_e type = file_type(name);
 
-	switch (type) {
+	complex float* addr = NULL;
 
-	case FILE_TYPE_PIPE:
+	if (mpi_is_main_proc() || mpi_shared_files) {
 
-		;
+		switch (type) {
 
-		static bool once_r = false;
+		case FILE_TYPE_PIPE:
 
-		if (once_r)
-			error("reading two inputs from pipe is not supported\n");
+			;
 
-		once_r = true;
+			assert(1 == mpi_get_num_procs());
 
-		// read header from stdin
+			static bool once_r = false;
 
-		if (-1 == read_cfl_header(0, &filename, D, dimensions))
-			error("Reading input\n");
+			if (once_r)
+				error("reading two inputs from pipe is not supported\n");
 
-		if (NULL == filename)
-			error("No data.\n");
+			once_r = true;
 
-		goto skip;
+			// read header from stdin
+			char* filename = NULL;
 
-	case FILE_TYPE_RA:
-		return load_zra(name, D, dimensions);
+			if (-1 == read_cfl_header(0, &filename, D, dimensions))
+				error("Reading input\n");
 
-	case FILE_TYPE_COO:
-		return load_zcoo(name, D, dimensions);
+			if (NULL == filename)
+				error("No data.\n");
 
-	case FILE_TYPE_SHM:
-		return load_zshm(name, D, dimensions);
+			addr = (priv ? private_cfl : shared_cfl)(D, dimensions, filename);
 
-	case FILE_TYPE_MEM:
-		return memcfl_load(name, D, dimensions);
+			if (0 != unlink(filename))
+				error("Error unlinking temporary file %s\n", filename);
 
-	default:
-		; // handled in this function
+			free(filename);
+
+			break;
+
+		case FILE_TYPE_RA:
+			addr = load_zra(name, D, dimensions);
+			break;
+
+		case FILE_TYPE_COO:
+			addr = load_zcoo(name, D, dimensions);
+			break;
+
+		case FILE_TYPE_SHM:
+			addr = load_zshm(name, D, dimensions);
+			break;
+
+		case FILE_TYPE_MEM:
+			addr = memcfl_load(name, D, dimensions);
+			break;
+		
+		case FILE_TYPE_CFL: ;
+
+			char name_bdy[1024];
+
+			if (1024 <= snprintf(name_bdy, 1024, "%s.cfl", name))
+				error("Loading cfl file %s\n", name);
+
+			char name_hdr[1024];
+
+			if (1024 <= snprintf(name_hdr, 1024, "%s.hdr", name))
+				error("Loading cfl file %s\n", name);
+
+			int ofd;
+
+			if (-1 == (ofd = open(name_hdr, O_RDONLY)))
+				io_error("Loading cfl file %s\n", name);
+
+			if (-1 == read_cfl_header(ofd, &filename, D, dimensions))
+				error("Loading cfl file %s\n", name);
+
+			if (-1 == close(ofd))
+				io_error("Loading cfl file %s\n", name);
+
+			addr = (priv ? private_cfl : shared_cfl)(D, dimensions, name_bdy);
+		}
 	}
 
+	if (1 < mpi_get_num_procs() && !mpi_shared_files) {
 
-	char name_bdy[1024];
+		mpi_sync_val(dimensions, sizeof(long)*D);
 
-	if (1024 <= snprintf(name_bdy, 1024, "%s.cfl", name))
-		error("Loading cfl file %s\n", name);
+		if (!mpi_is_main_proc())
+			addr = anon_cfl(NULL, D, dimensions);
 
-	char name_hdr[1024];
+		// sliced sync is performed in create_work_buffer
+		if (!cfl_loop_desc_active())
+			mpi_sync_val(addr, io_calc_size(D, dimensions, sizeof(complex float*)));
+	}
 
-	if (1024 <= snprintf(name_hdr, 1024, "%s.hdr", name))
-		error("Loading cfl file %s\n", name);
-
-	int ofd;
-
-	if (-1 == (ofd = open(name_hdr, O_RDONLY)))
-		io_error("Loading cfl file %s\n", name);
-
-	if (-1 == read_cfl_header(ofd, &filename, D, dimensions))
-		error("Loading cfl file %s\n", name);
-
-	if (-1 == close(ofd))
-		io_error("Loading cfl file %s\n", name);
-
-skip: ;
-	complex float* ret = (priv ? private_cfl : shared_cfl)(D, dimensions, filename ?: name_bdy);
-
-	if (FILE_TYPE_PIPE == type)
-		if (0 != unlink(filename))
-			error("Error unlinking temporary file %s\n", filename);
-
-	free(filename);
-
-	return ret;
+	return create_worker_buffer(D, dimensions, addr, false);
 }
 
 
@@ -494,7 +871,7 @@ complex float* shared_cfl(int D, const long dims[D], const char* name)
 
 	err_assert(T > 0);
 
-        if (-1 == (fd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR)))
+        if (-1 == (fd = open(name, O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
 		io_error("shared cfl %s\n", name);
 
 //	if (-1 == (fstat(fd, &st)))
@@ -592,14 +969,19 @@ complex float* private_cfl(int D, const long dims[D], const char* name)
 }
 
 
-void unmap_cfl(int D, const long dims[D?:1], const complex float* x)
+void unmap_cfl(int D, const long dims[D], const complex float* x)
 {
 	if (memcfl_unmap(x))
 		return;
 
+	long tdims[D?:1];
+	md_copy_dims(D, tdims, dims);
+
+	x = free_worker_buffer(D, tdims, (void*)x);
+
 	long T;
 
-	if (-1 == (T = io_calc_size(D, dims, sizeof(complex float))))
+	if (-1 == (T = io_calc_size(D, tdims, sizeof(complex float))))
 		error("unmap cfl\n");
 
 #ifdef _WIN32
@@ -623,6 +1005,9 @@ void create_multi_cfl(const char* name, int N, int D[N], const long* dimensions[
 {
 	io_register_output(name);
 
+	if (cfl_loop_desc_active())
+		error("multi cfl not supported for bart loop!");
+
 #ifdef MEMONLY_CFL
 	error("multi cfl not supported with MEMONLY_CFL");
 #else
@@ -649,7 +1034,7 @@ void create_multi_cfl(const char* name, int N, int D[N], const long* dimensions[
 		error("Creating cfl file %s\n", name);
 
 	int ofd;
-	if (-1 == (ofd = open(name_hdr, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR)))
+	if (-1 == (ofd = open(name_hdr, O_RDWR|O_CREAT|O_TRUNC, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH)))
 		io_error("Creating cfl file %s\n", name);
 
 	long num_ele = 0;
@@ -668,10 +1053,13 @@ void create_multi_cfl(const char* name, int N, int D[N], const long* dimensions[
 #endif /* MEMONLY_CFL */
 }
 
-static int load_multi_cfl_internal(const char* name, int N_max, int D_max, int D[__VLA(N_max)], long dimensions[__VLA(N_max)][D_max], _Complex float* args[__VLA(N_max)], bool priv)
+static int load_multi_cfl_internal(const char* name, int N_max, int D_max, int D[N_max], long dimensions[N_max][D_max], _Complex float* args[N_max], bool priv)
 {
 	io_register_input(name);
 
+	if (cfl_loop_desc_active())
+		error("multi cfl not supported for bart loop!");
+
 #ifdef MEMONLY_CFL
 	error("multi cfl not supported with MEMONLY_CFL");
 #else
