@@ -199,14 +199,14 @@ int (poissondisc_mc)(int D, int T, int N, int I, float vardens, const float delt
 			if (accept) {
 
 				long off[D];
-				for (int i = 0; i < D; i++)
-					off[i] = MIN(MAX(0, pos[i] - (patchdims[i] + 1) / 2), griddims[i] - patchdims[i]);
+				for (int ii = 0; ii < D; ii++)
+					off[ii] = MIN(MAX(0, pos[ii] - (patchdims[ii] + 1) / 2), griddims[ii] - patchdims[ii]);
 
 				md_copy_block(D, off, patchdims, patch, griddims, grid, sizeof(int));
 
-				for (int i = 0; i < md_calc_size(D, patchdims); i++)
-					if (-1 != patch[i])
-						accept &= distance_check(D, T, N, vardens, delta, points, kind, p, patch[i]);
+				for (int ii = 0; ii < md_calc_size(D, patchdims); ii++)
+					if (-1 != patch[ii])
+						accept &= distance_check(D, T, N, vardens, delta, points, kind, p, patch[ii]);
 			}
 
 #endif
@@ -258,7 +258,7 @@ out:
 	md_free(grid);
 	md_free(patch);
 #endif
-	free(active);
+	XFREE(active);
 	return p;
 }
 
@@ -271,7 +271,7 @@ extern int poissondisc(int D, int N, int I, float vardens, float delta, float po
 	memset(*kind, 0, I * sizeof(int));
 	const float dd[1][1] = { { delta } };
 	int P = poissondisc_mc(D, 1, N, I, vardens, dd, points, *kind);
-	free(kind);
+	XFREE(kind);
 	return P;
 }
 
@@ -327,7 +327,7 @@ extern void mc_poisson_rmatrix(int D, int T, float rmatrix[T][T], const float de
 		table[i].index = i;
 		table[i].x = delta[i];
 	}
-	
+
 	qsort(&table, T, sizeof(struct sort_label), sort_cmp);
 
 	int mc[T][T];
@@ -336,11 +336,11 @@ extern void mc_poisson_rmatrix(int D, int T, float rmatrix[T][T], const float de
 	int i;
 
 	for (i = 0; (i < T) && (ind < T); i++) {
-	
+
 		float val = table[ind].x;
 		int j = 0;
 
-		while ((table[ind].x == val) && (ind < T))
+		while ((ind < T) && (table[ind].x == val))
 			mc[i][j++] = table[ind++].index;
 
 		nc[i] = j;
