@@ -1,6 +1,7 @@
 
+#ifndef NO_PNG
 extern int png_write_rgb24(const char* name, int w, int h, long inum, const unsigned char* buf);
 extern int png_write_rgb32(const char* name, int w, int h, long inum, const unsigned char* buf);
 extern int png_write_bgr24(const char* name, int w, int h, long inum, const unsigned char* buf);
 extern int png_write_bgr32(const char* name, int w, int h, long inum, const unsigned char* buf);
-
+#endif
