@@ -8,5 +8,5 @@ extern void bloch_forw_alpha(const struct linop_s* op, complex float* dst, const
 extern void bloch_back_alpha(const struct linop_s* op, complex float* dst, const complex float* src);
 
 extern struct nlop_s* nlop_bloch_create(int N, const long der_dims[N], const long map_dims[N], const long out_dims[N], const long in_dims[N],
-		const complex float* b1, const struct moba_conf_s* _data, bool use_gpu);
+		const complex float* b1, const complex float* b0, const struct moba_conf_s* _data, bool use_gpu);
 
