@@ -32,12 +32,11 @@
 struct meco_s meco_create(const long dims[DIMS], const long y_dims[DIMS], const long x_dims[DIMS], const complex float* mask, const complex float* TE, const complex float* psf, enum meco_model sel_model, bool real_pd, enum fat_spec fat_spec, const float* scale_fB0, bool use_gpu, const struct noir_model_conf_s* conf)
 {
 	struct meco_s ret;
+	struct noir_s nlinv = noir_create(dims, mask, psf, conf);
 
 	if (MECO_PI == sel_model) {
 
-		struct noir_s nlinv = noir_create(dims, mask, psf, conf);
-
-		ret.nlop = nlinv.nlop;
+		ret.nlop = nlop_flatten_F(nlinv.nlop);
 		ret.linop = nlinv.linop;
 
 	} else {
@@ -45,8 +44,6 @@ struct meco_s meco_create(const long dims[DIMS], const long y_dims[DIMS], const
 		// chain model
 		struct nlop_s* meco = nlop_meco_create(DIMS, y_dims, x_dims, TE, sel_model, real_pd, fat_spec, scale_fB0, use_gpu);
 		
-		struct noir_s nlinv = noir_create3(dims, mask, psf, conf);
-
 		const struct nlop_s* b = nlinv.nlop;
 		nlinv.nlop = nlop_chain2(meco, 0, b, 0);
 		nlop_free(b);
