@@ -3,7 +3,7 @@
  * a BSD-style license which can be found in the LICENSE file.
  */
  
-
+#include <complex.h>
 
 #include "misc/mri.h"
 
@@ -33,9 +33,7 @@ struct noir_s {
 	struct noir_op_s* noir_op;
 };
 
-extern struct noir_s noir_create2(const long dims[DIMS], const complex float* mask, const complex float* psf, const struct noir_model_conf_s* conf);
 extern struct noir_s noir_create(const long dims[DIMS], const complex float* mask, const complex float* psf, const struct noir_model_conf_s* conf);
-extern struct noir_s noir_create3(const long dims[DIMS], const complex float* mask, const complex float* psf, const struct noir_model_conf_s* conf);
 
 
 struct nlop_data_s;
