@@ -61,6 +61,9 @@ struct nufft_data {
 
 	const struct linop_s* cfft_op;   ///< Pcycle FFT operator
 	unsigned int cycle;
+
+	struct linop_s* lop_nufft_psf;
+	struct linop_s* lop_fftuc_psf;
 };
 
 
