@@ -4,13 +4,17 @@
  * a BSD-style license which can be found in the LICENSE file.
  *
  * 2014-2020 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2018-2019 Sebastian Rosenzweig <sebastian.rosenzweig@med.uni-goettingen.de>
+ * 2018-2020 Sebastian Rosenzweig <sebastian.rosenzweig@med.uni-goettingen.de>
+ * 2019-2020 Zhengguo Tan <zhengguo.tan@med.uni-goettingen.de>
  */
 
 #include <stdbool.h>
 #include <math.h>
+#include <assert.h>
 
 #include "misc/mri.h"
+#include "misc/misc.h"
+#include "misc/version.h"
 #ifdef SSAFARY_PAPER
 #include "misc/debug.h"
 #endif
@@ -29,23 +33,11 @@ const struct traj_conf traj_defaults = {
 	.d3d = false,
 	.transverse = false,
 	.asym_traj = false,
+	.mems_traj = false,
 	.accel = 1,
 	.tiny_gold = 0,
-};
-
-const struct traj_conf rmfreq_defaults = {
-
-	.radial = true,
-	.golden = false,
-	.aligned = false,
-	.full_circle = false,
-	.half_circle_gold = false,
-	.golden_partition = false,
-	.d3d = false,
-	.transverse = false,
-	.asym_traj = false,
-	.accel = 1,
-	.tiny_gold = 0,
+	.rational = false,
+	.double_base = false,
 };
 
 
@@ -82,25 +74,95 @@ void gradient_delay(float d[3], float coeff[2][3], float phi, float psi)
 	}
 }
 
-void calc_base_angles(double base_angle[DIMS], int Y, int mb, int turns, struct traj_conf conf)
+int recover_gen_fib_ind(int Y, int inc)
 {
-	/* Golden-ratio sampling
+	int step = 0;
+
+	while (inc > gen_fibonacci(1, step - 1))
+		step++;
+
+	// Assumption: Fib Index < 40
+	for (int i = 1; i <= 40; i++)
+		if (Y == gen_fibonacci(i, step))
+			return i;
+
+	return -1;
+}
+
+static void fib_next(int f[2])
+{
+	int t = f[0];
+	f[0] = f[1];
+	f[1] += t;
+}
+
+int gen_fibonacci(int n, int ind)
+{
+	int fib[2] = { 1, n };
+
+	for (int i = 0; i < ind - 1; i++)
+		fib_next(fib);
+
+	return (0 == ind) ? fib[0] : fib[1];
+}
+
+static int raga_find_index(int Y, int n)
+{
+	int i = 0;
+
+	while (Y > gen_fibonacci(n, i))
+		i++;
+
+	return i;
+}
+
+static int raga_increment(int Y, int n)
+{
+	int i = raga_find_index(Y, n);
+
+	return gen_fibonacci(1, i - 1);
+}
+
+static float rational_angle(int Y, int n)
+{
+	int inc = raga_increment(Y, n);
+
+	return M_PI / (float)Y * (float)inc;
+}
+
+
+void calc_base_angles(double base_angle[DIMS], int Y, int E, int mb, int turns, struct traj_conf conf)
+{
+	/*
 	 * Winkelmann S, Schaeffter T, Koehler T, Eggers H, Doessel O.
 	 * An optimal radial profile order based on the Golden Ratio
 	 * for time-resolved MRI. IEEE TMI 26:68--76 (2007)
-	 */
-	double golden_ratio = (sqrtf(5.) + 1.) / 2;
-
-	/* Tiny golden angle
+	 *
 	 * Wundrak S, Paul J, Ulrici J, Hell E, Geibel MA, Bernhardt P, Rottbauer W, Rasche V.
 	 * Golden ratio sparse MRI using tiny golden angles.
 	 * Magn Reson Med 75:2372-2378 (2016)
 	 */
+
+	double golden_ratio = (sqrt(5.) + 1.) / 2;
+
+	if (use_compat_to_version("v0.5.00"))
+		golden_ratio = (sqrtf(5.) + 1.) / 2;
+
 	double golden_angle = M_PI / (golden_ratio + conf.tiny_gold - 1.);
 
 	// For numerical stability
-	if (1 == conf.tiny_gold)
-		golden_angle = M_PI * (2. - (3. - sqrtf(5.))) / 2.;
+	if (1 == conf.tiny_gold) {
+
+		golden_angle = M_PI * (2. - (3. - sqrt(5.))) / 2.;
+
+		if (use_compat_to_version("v0.5.00"))
+			golden_angle = M_PI * (2. - (3. - sqrtf(5.))) / 2.;
+	}
+
+	if (conf.rational)
+		golden_angle = rational_angle(Y / (conf.double_base ? 1 : 2), conf.tiny_gold);
+
+	golden_angle = golden_angle * (conf.double_base ? 2. : 1.);
 
 	double angle_atom = M_PI / Y;
 
@@ -119,9 +181,22 @@ void calc_base_angles(double base_angle[DIMS], int Y, int mb, int turns, struct
 	if (turns > 1)
 		angle_t = angle_atom / turns * (conf.full_circle ? 2 : 1);
 
+	/* radial multi-echo multi-spoke sampling
+	 *
+	 * Tan Z, Voit D, Kollmeier JM, Uecker M, Frahm J.
+	 * Dynamic water/fat separation and B0 inhomogeneity mapping -- joint
+	 * estimation using undersampled  triple-echo multi-spoke radial FLASH.
+	 * Magn Reson Med 82:1000-1011 (2019)
+	 */
+	double angle_e = 0.;
 
-	// Golden Angle
-	if (conf.golden) {
+	if (conf.mems_traj) {
+
+		angle_s = angle_s * 1.;
+		angle_e = angle_s / E;
+		angle_t = golden_angle;
+
+	} else if (conf.golden) {
 
 		if (conf.aligned) {
 
@@ -155,10 +230,21 @@ void calc_base_angles(double base_angle[DIMS], int Y, int mb, int turns, struct
 			debug_printf(DP_INFO, "Trajectory generation to reproduce SSA-FARY Paper!\n");
 		}
 #endif
+		if (use_compat_to_version("v0.4.00")) {
+
+			// since the traj rewrite (commit d4e6e2e3a2313) we do not apply
+			// full circle to golden angle anymore. However, this is needed for
+			// reproducing the RING paper
+			angle_s *= (conf.full_circle ? 2. : 1.);
+			angle_m *= (conf.full_circle ? 2. : 1.);
+			angle_t *= (conf.full_circle ? 2. : 1.);
+		}
+
 	}
 
 	base_angle[PHS2_DIM] = angle_s;
 	base_angle[SLICE_DIM] = angle_m;
+	base_angle[TE_DIM] = angle_e;
 	base_angle[TIME_DIM] = angle_t;
 }
 
