@@ -3,7 +3,7 @@
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
- * 2018 Martin Uecker <martin.uecker@med.uni-goettingen.de>
+ * 2018-2019 Martin Uecker <martin.uecker@med.uni-goettingen.de>
  * 2018 Sebastian Rosenzweig <sebastian.rosenzweig@med.uni-goettingen.de>
  * 2019-2020 Zhengguo Tan <zhengguo.tan@med.uni-goettingen.de>
  */
@@ -20,6 +20,8 @@ struct traj_conf {
 	_Bool transverse;
 	_Bool asym_traj;
 	_Bool mems_traj;
+	_Bool rational;
+	_Bool double_base;
 	int accel;
 	int tiny_gold;
 };
@@ -35,5 +37,6 @@ extern void euler(float dir[3], float phi, float psi);
 extern void gradient_delay(float d[3], float coeff[2][3], float phi, float psi);
 extern void calc_base_angles(double base_angle[DIMS], int Y, int E, int mb, int turns, struct traj_conf conf);
 extern bool zpartition_skip(long partitions, long z_usamp[2], long partition, long frame);
-
+extern int gen_fibonacci(int n, int ind);
+extern int recover_gen_fib_ind(int Y, int inc);
 
