@@ -18,6 +18,7 @@
 
 #include "misc/misc.h"
 #include "misc/mmio.h"
+#include "misc/opts.h"
 #include "misc/utils.h"
 
 
@@ -29,33 +30,46 @@
 
 
 
-static const char usage_str[] = "flags <input> <output>";
-static const char help_str[] = "Normalize along selected dimensions.\n";
+static const char help_str[] = "Normalize along selected dimensions.";
 
 
 
-int main_normalize(int argc, char* argv[])
+int main_normalize(int argc, char* argv[argc])
 {
+	unsigned long flags = 0;
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_ULONG(true, &flags, "flags"),
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	bool l1 = false;
 
-	l1 = mini_cmdline_bool(&argc, argv, 'b', 3, usage_str, help_str);
+	const struct opt_s opts[] = {
+
+		OPT_SET('b', &l1, "l1"),
+	};
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	int N = DIMS;
 	long dims[N];
-	complex float* data = load_cfl(argv[2], N, dims);
+	complex float* data = load_cfl(in_file, N, dims);
 
-	int flags = atoi(argv[1]);
+	complex float* out = create_cfl(out_file, N, dims);
 
-	assert(flags >= 0);
-
-	complex float* out = create_cfl(argv[3], N, dims);
 	md_copy(N, dims, out, data, CFL_SIZE);
 
 	(l1 ? normalizel1 : normalize)(N, flags, dims, out);
 
 	unmap_cfl(N, dims, out);
+
 	return 0;
 }
 
