@@ -62,64 +62,25 @@ static void cublas_error(const char* file, int line, cublasStatus_t code)
 	error("cuBLAS Error: %s in %s:%d \n", err_str, file, line);
 }
 
-#define CUBLAS_ERROR(x)	({ CUDA_ASYNC_ERROR_NOTE("before cuBLAS call"); cublasStatus_t errval = (x); if (CUBLAS_STATUS_SUCCESS != errval) cublas_error(__FILE__, __LINE__, errval); CUDA_ASYNC_ERROR_NOTE("after cuBLAS call"); })
-#define CUBLAS_CALL(x)	({ CUDA_ASYNC_ERROR_NOTE("before cuBLAS call"); cublas_set_gpulock(); cublasStatus_t errval = (x); cublas_unset_gpulock(); if (CUBLAS_STATUS_SUCCESS != errval) cublas_error(__FILE__, __LINE__, errval); CUDA_ASYNC_ERROR_NOTE("after cuBLAS call"); })
-
-
-static cublasHandle_t handle[MAX_CUDA_DEVICES];
-static int num_devices_initialized = 0;
-
-void cublas_init(void)
-{
-	if (0 != num_devices_initialized)
-		error("Cannot reinitialize cuBLAS, deinit it first!");
-
-	int old_device = cuda_get_device();
-
-	for (int device = 0; device < cuda_num_devices(); ++device) {
-
-		cuda_set_device(device);
-		CUBLAS_ERROR(cublasCreate(&handle[device]));
-	}
-
-	cuda_set_device(old_device);
-
-	num_devices_initialized = cuda_num_devices();	
-}
-
-void cublas_deinit(void)
-{
-	if (cuda_num_devices() != num_devices_initialized)
-		error("Cannot deinitialize cuBLAS, number of devices has changed from initialization!");
-
-	for (int device = 0; device < cuda_num_devices(); ++device)
-		CUBLAS_ERROR(cublasDestroy(handle[device]));
-
-	num_devices_initialized = 0;	
-}
+#define CUBLAS_ERROR(x)	({ cublasStatus_t errval = (x); if (CUBLAS_STATUS_SUCCESS != errval) cublas_error(__FILE__, __LINE__, errval); })
+#define CUBLAS_CALL(x)		({ CUDA_ASYNC_ERROR_NOTE("before cuBLAS call"); cublas_set_gpulock(); cublasStatus_t errval = (x); cublas_unset_gpulock(); if (CUBLAS_STATUS_SUCCESS != errval) cublas_error(__FILE__, __LINE__, errval); CUDA_ASYNC_ERROR_NOTE("after cuBLAS call"); })
 
+static cublasHandle_t handle_host[CUDA_MAX_STREAMS + 1];
+static cublasHandle_t handle_device[CUDA_MAX_STREAMS + 1];
 
 #ifdef _OPENMP
+//FIXME: Following cuBLAS documentation, cuBLAS calls should be threadsafe
+//	 However, tests/test-pics-multigpu fails with too many (16) threads
 #include <omp.h>
-static bool gpulock_init = false;
-static omp_lock_t gpulock[MAX_CUDA_DEVICES];
+static omp_lock_t gpulock[CUDA_MAX_STREAMS + 1];;
 static void cublas_set_gpulock(void)
 {
-	#pragma omp critical (init_cublas_gpulock)
-	if (!gpulock_init) {
-
-		for (int i = 0; i < MAX_CUDA_DEVICES; i++)
-			omp_init_lock(&gpulock[i]);
-
-		gpulock_init = true;		
-	}
-
-	omp_set_lock(&gpulock[cuda_get_device()]);
+	omp_set_lock(&(gpulock[cuda_get_stream_id()]));
 }
 
 static void cublas_unset_gpulock(void)
 {
-	omp_unset_lock(&gpulock[cuda_get_device()]);
+	omp_unset_lock(&(gpulock[cuda_get_stream_id()]));
 }
 #else
 static void cublas_set_gpulock(void)
@@ -133,28 +94,48 @@ static void cublas_unset_gpulock(void)
 }
 #endif
 
+void cublas_init(void)
+{
+	for (int i = 0; i < CUDA_MAX_STREAMS + 1; i++) {
+
+		CUBLAS_ERROR(cublasCreate(&(handle_host[i])));
+		CUBLAS_ERROR(cublasCreate(&(handle_device[i])));
+
+		CUBLAS_ERROR(cublasSetPointerMode(handle_host[i], CUBLAS_POINTER_MODE_HOST));
+		CUBLAS_ERROR(cublasSetPointerMode(handle_device[i], CUBLAS_POINTER_MODE_DEVICE));
+
+		CUBLAS_ERROR(cublasSetStream(handle_host[i], cuda_get_stream_by_id(i)));
+		CUBLAS_ERROR(cublasSetStream(handle_device[i], cuda_get_stream_by_id(i)));
+
+#ifdef _OPENMP
+		omp_init_lock(&(gpulock[i]));
+#endif
+	}
+}
+
+void cublas_deinit(void)
+{
+	for (int i = 0; i < CUDA_MAX_STREAMS + 1; i++) {
+
+		CUBLAS_ERROR(cublasDestroy(handle_device[i]));
+		CUBLAS_ERROR(cublasDestroy(handle_host[i]));
+
+#ifdef _OPENMP
+		omp_destroy_lock(&(gpulock[i]));
+#endif
+	}
+}
+
+
+
 static cublasHandle_t get_handle_device(void)
 {
-	if (cuda_num_devices() != num_devices_initialized)
-		error("cuBLAS not initialized correctly!");
-
-	cublasHandle_t result = handle[cuda_get_device()];
-	CUBLAS_ERROR(cublasSetPointerMode(result, CUBLAS_POINTER_MODE_DEVICE));
-	CUBLAS_ERROR(cublasSetStream(result, cuda_get_stream()));
-
-	return result;
+	return handle_device[cuda_get_stream_id()];
 }
 
 static cublasHandle_t get_handle_host(void)
 {
-	if (cuda_num_devices() != num_devices_initialized)
-		error("cuBLAS not initialized correctly!");
-
-	cublasHandle_t result = handle[cuda_get_device()];
-	CUBLAS_ERROR(cublasSetPointerMode(result, CUBLAS_POINTER_MODE_HOST));
-	CUBLAS_ERROR(cublasSetStream(result, cuda_get_stream()));
-
-	return result;
+	return handle_host[cuda_get_stream_id()];
 }
 
 static cublasOperation_t cublas_trans(char trans)
@@ -192,6 +173,26 @@ double cuda_sdot(long size, const float* src1, const float* src2)
 	return result;
 }
 
+complex double cuda_zdot(long size, const complex float* src1, const complex float* src2)
+{
+//FIXME: opposite convetion of conjugate vs blas
+	complex double result = 0;
+
+	while (size > 0) {
+
+		complex float tmp;
+		CUBLAS_CALL(cublasCdotc(get_handle_host(), MIN(size, INT_MAX / 4), (const cuComplex*)src2, 1, (const cuComplex*)src1, 1, (cuComplex*)(&tmp)));
+
+		result += tmp;
+		
+		src1 += INT_MAX / 4;
+		src2 += INT_MAX / 4;
+		size -= INT_MAX / 4;
+	}
+
+	return result;
+}
+
 
 double cuda_norm(long size, const float* src1)
 {
@@ -201,9 +202,9 @@ double cuda_norm(long size, const float* src1)
 	// git rev: ab28a9a953a80d243511640b23501f964a585349
 //	printf("cublas: %f\n", cublasSnrm2(size, src1, 1));
 //	printf("GPU norm (sdot: %f)\n", sqrt(cuda_sdot(size, src1, src1)));
-#ifdef GPU_ASSERTS
-	assert(cuda_ondevice_num(src1, cuda_get_device_internal()));
-#endif
+//#ifdef GPU_ASSERTS
+//	assert(cuda_ondevice_num(src1, cuda_get_device_internal()));
+//#endif
 	return sqrt(cuda_sdot(size, src1, src1));
 #else
 	return cublasSnrm2(size, src1, 1);
@@ -257,27 +258,58 @@ void cuda_swap(long size, float* a, float* b)
 
 #endif
 
+#ifdef _OPENMP
+#include <omp.h>
 
+static bool cpulock_init = false;
+static omp_nest_lock_t cpulock;
 
-
-static void openblas_set_threads(void)
+static void blas_cpu_set_lock(void)
 {
-#ifndef USE_OPENBLAS
-	return;
-#else
+	if (!cpulock_init) {
+	
+		#pragma omp critical(cpulock_init)
+		{
+			if (!cpulock_init) {
 
-#ifndef _OPENMP
-	return;
-#else
-	if (1 != openblas_get_parallel())
-		return; //pthread version of openblas
+				#ifdef USE_OPENBLAS
+				//set num threads to ne if openblas is built with pthreads
+				if (1 == openblas_get_parallel())
+					openblas_set_num_threads(1);
+				#endif
 
-	#pragma omp critical
-	openblas_set_num_threads(omp_in_parallel() ? 1 : omp_get_max_threads());
-#endif
+				omp_init_nest_lock(&cpulock);
+				cpulock_init = true;
+			}
+		}
+	}
+
+	if (!omp_in_parallel())
+		return;
+
+#ifndef BLAS_THREADSAFE
+	omp_set_nest_lock(&cpulock);
 #endif
 }
 
+static void blas_cpu_unset_lock(void)
+{
+	if (!omp_in_parallel())
+		return;
+
+#ifndef BLAS_THREADSAFE
+	omp_unset_nest_lock(&cpulock);
+#endif
+}
+
+#define BLAS_CALL(x)	({ blas_cpu_set_lock(); (x); blas_cpu_unset_lock(); })
+
+#else
+
+#define BLAS_CALL(x)	x
+
+#endif
+
 
 
 
@@ -292,11 +324,10 @@ void blas2_cgemm(char transa, char transb, long M, long N, long K, const complex
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
+	BLAS_CALL(cblas_cgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
 		    ('T' == transb) ? CblasTrans : (('C' == transb) ? CblasConjTrans : CblasNoTrans),
-		    M, N, K, (void*)alpha, (void*)A, lda, (void*)B, ldb, (void*)beta, (void*)C, ldc);
+		    M, N, K, (void*)alpha, (void*)A, lda, (void*)B, ldb, (void*)beta, (void*)C, ldc));
 }
 
 
@@ -313,11 +344,9 @@ void blas_cgemm(char transa, char transb, long M, long N,  long K, const complex
 	}
 #endif
 
-	openblas_set_threads();
-
-	cblas_cgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
+	BLAS_CALL(cblas_cgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
 		    ('T' == transb) ? CblasTrans : (('C' == transb) ? CblasConjTrans : CblasNoTrans),
-		    M, N, K, (void*)(&alpha), (void*)A, lda, (void*)B, ldb, (void*)(&beta), (void*)C, ldc);
+		    M, N, K, (void*)(&alpha), (void*)A, lda, (void*)B, ldb, (void*)(&beta), (void*)C, ldc));
 }
 
 
@@ -333,10 +362,9 @@ void blas2_cgemv(char trans, long M, long N, const complex float* alpha, long ld
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cgemv(CblasColMajor, ('T' == trans) ? CblasTrans : (('C' == trans) ? CblasConjTrans : CblasNoTrans),
-		    M, N, (void*)alpha, (void*)A, lda, (void*)x, incx, (void*)beta, (void*)y, incy);
+	BLAS_CALL(cblas_cgemv(CblasColMajor, ('T' == trans) ? CblasTrans : (('C' == trans) ? CblasConjTrans : CblasNoTrans),
+		    M, N, (void*)alpha, (void*)A, lda, (void*)x, incx, (void*)beta, (void*)y, incy));
 }
 
 
@@ -352,10 +380,9 @@ void blas_cgemv(char trans, long M, long N, complex float alpha, long lda, const
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cgemv(CblasColMajor, ('T' == trans) ? CblasTrans : (('C' == trans) ? CblasConjTrans : CblasNoTrans),
-		    M, N, (void*)&alpha, (void*)A, lda, (void*)x, incx, (void*)&beta, (void*)y, incy);
+	BLAS_CALL(cblas_cgemv(CblasColMajor, ('T' == trans) ? CblasTrans : (('C' == trans) ? CblasConjTrans : CblasNoTrans),
+		    M, N, (void*)&alpha, (void*)A, lda, (void*)x, incx, (void*)&beta, (void*)y, incy));
 }
 
 
@@ -371,9 +398,8 @@ void blas2_cgeru(long M, long N, const complex float* alpha, long incx, const co
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cgeru(CblasColMajor, M, N, alpha, x, incx, y, incy, A, lda);
+	BLAS_CALL(cblas_cgeru(CblasColMajor, M, N, alpha, x, incx, y, incy, A, lda));
 }
 
 
@@ -389,9 +415,8 @@ void blas_cgeru(long M, long N, complex float alpha, long incx, const complex fl
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cgeru(CblasColMajor, M, N, &alpha, x, incx, y, incy, (float*)A, lda);
+	BLAS_CALL(cblas_cgeru(CblasColMajor, M, N, &alpha, x, incx, y, incy, (float*)A, lda));
 }
 
 
@@ -407,9 +432,7 @@ void blas2_caxpy(long N, const complex float* alpha, long incx, const complex fl
 	}
 #endif
 
-	openblas_set_threads();
-
-	cblas_caxpy(N, alpha, x, incx, y, incy);
+	BLAS_CALL(cblas_caxpy(N, alpha, x, incx, y, incy));
 }
 
 
@@ -425,9 +448,7 @@ void blas_caxpy(long N, const complex float alpha, long incx, const complex floa
 	}
 #endif
 
-	openblas_set_threads();
-
-	cblas_caxpy(N, &alpha, x, incx, y, incy);
+	BLAS_CALL(cblas_caxpy(N, &alpha, x, incx, y, incy));
 }
 
 
@@ -442,9 +463,8 @@ void blas2_cscal(long N, const complex float* alpha, long incx, complex float* x
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cscal(N, alpha, x, incx);
+	BLAS_CALL(cblas_cscal(N, alpha, x, incx));
 }
 
 
@@ -459,9 +479,8 @@ void blas_cscal(long N, const complex float alpha, long incx, complex float* x)
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cscal(N, &alpha, x, incx);
+	BLAS_CALL(cblas_cscal(N, &alpha, x, incx));
 }
 
 
@@ -476,9 +495,8 @@ void blas2_cdotu(complex float* result, long N, long incx, const complex float*
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_cdotu_sub(N, x, incx, y, incy, (void*)result);
+	BLAS_CALL(cblas_cdotu_sub(N, x, incx, y, incy, (void*)result));
 }
 
 
@@ -493,11 +511,10 @@ void blas2_sgemm(char transa, char transb, long M, long N, long K, const float*
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
+	BLAS_CALL(cblas_sgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
 		    ('T' == transb) ? CblasTrans : (('C' == transb) ? CblasConjTrans : CblasNoTrans),
-		    M, N, K, *alpha, A, lda, B, ldb, *beta, C, ldc);
+		    M, N, K, *alpha, A, lda, B, ldb, *beta, C, ldc));
 }
 
 
@@ -512,11 +529,10 @@ void blas_sgemm(char transa, char transb, long M, long N,  long K, const float a
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
+	BLAS_CALL(cblas_sgemm(CblasColMajor, ('T' == transa) ? CblasTrans : (('C' == transa) ? CblasConjTrans : CblasNoTrans),
 		    ('T' == transb) ? CblasTrans : (('C' == transb) ? CblasConjTrans : CblasNoTrans),
-		    M, N, K, alpha, A, lda, B, ldb, beta, C, ldc);
+		    M, N, K, alpha, A, lda, B, ldb, beta, C, ldc));
 }
 
 
@@ -527,15 +543,14 @@ void blas2_sgemv(char trans, long M, long N, const float* alpha, long lda, const
 	if (cuda_ondevice(A)) {
 
 		CUBLAS_CALL(cublasSgemv(get_handle_device(), cublas_trans(trans), M, N, alpha,
-			    (const float*)A, lda, x, incx, beta, y, incy));
+			    A, lda, x, incx, beta, y, incy));
 
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sgemv(CblasColMajor, ('T' == trans) ? CblasTrans : CblasNoTrans, M, N, *alpha,
-		    (const float*)A, lda, x, incx, *beta, y, incy);
+	BLAS_CALL(cblas_sgemv(CblasColMajor, ('T' == trans) ? CblasTrans : CblasNoTrans, M, N, *alpha,
+			A, lda, x, incx, *beta, y, incy));
 }
 
 
@@ -546,15 +561,14 @@ void blas_sgemv(char trans, long M, long N, const float alpha, long lda, const f
 	if (cuda_ondevice(A)) {
 
 		CUBLAS_CALL(cublasSgemv(get_handle_host(), cublas_trans(trans), M, N, &alpha,
-			    (const float*)A, lda, x, incx, &beta, y, incy));
+			A, lda, x, incx, &beta, y, incy));
 
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sgemv(CblasColMajor, ('T' == trans) ? CblasTrans : CblasNoTrans, M, N, alpha,
-		    (const float*)A, lda, x, incx, beta, y, incy);
+	BLAS_CALL(cblas_sgemv(CblasColMajor, ('T' == trans) ? CblasTrans : CblasNoTrans, M, N, alpha,
+			A, lda, x, incx, beta, y, incy));
 }
 
 
@@ -564,14 +578,12 @@ void blas2_sger(long M, long N, const float* alpha, long incx, const float* x, l
 #ifdef USE_CUDA
 	if (cuda_ondevice(A)) {
 
-		CUBLAS_CALL(cublasSger(get_handle_device(), M, N, alpha, x, incx, y, incy, (float*)A, lda));
-
+		CUBLAS_CALL(cublasSger(get_handle_device(), M, N, alpha, x, incx, y, incy, A, lda));
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sger(CblasColMajor, M, N, *alpha, x, incx, y, incy, (float*)A, lda);
+	BLAS_CALL(cblas_sger(CblasColMajor, M, N, *alpha, x, incx, y, incy, A, lda));
 }
 
 
@@ -582,14 +594,12 @@ void blas_sger(long M, long N, const float alpha, long incx, const float* x, lon
 
 	if (cuda_ondevice(A)) {
 
-		CUBLAS_CALL(cublasSger(get_handle_host(), M, N, &alpha, x, incx, y, incy, (float*)A, lda));
-
+		CUBLAS_CALL(cublasSger(get_handle_host(), M, N, &alpha, x, incx, y, incy, A, lda));
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sger(CblasColMajor, M, N, alpha, x, incx, y, incy, (float*)A, lda);
+	BLAS_CALL(cblas_sger(CblasColMajor, M, N, alpha, x, incx, y, incy, A, lda));
 }
 
 
@@ -604,9 +614,8 @@ void blas2_saxpy(long N, const float* alpha, long incx, const float* x, long inc
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_saxpy(N, *alpha, x, incx, y, incy);
+	BLAS_CALL(cblas_saxpy(N, *alpha, x, incx, y, incy));
 }
 
 
@@ -621,9 +630,8 @@ void blas_saxpy(long N, const float alpha, long incx, const float* x, long incy,
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_saxpy(N, alpha, x, incx, y, incy);
+	BLAS_CALL(cblas_saxpy(N, alpha, x, incx, y, incy));
 }
 
 
@@ -639,9 +647,8 @@ void blas2_sscal(long N, const float* alpha, long incx, float* x)
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sscal(N, *alpha, x, incx);
+	BLAS_CALL(cblas_sscal(N, *alpha, x, incx));
 }
 
 
@@ -656,9 +663,8 @@ void blas_sscal(long N, float alpha, long incx, float* x)
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	cblas_sscal(N, alpha, x, incx);
+	BLAS_CALL(cblas_sscal(N, alpha, x, incx));
 }
 
 
@@ -673,9 +679,8 @@ void blas2_sdot(float* result, long N, long incx, const float* x, long incy, con
 		return;
 	}
 #endif
-	openblas_set_threads();
 
-	*result = cblas_sdot(N, x, incx, y, incy);
+	BLAS_CALL(*result = cblas_sdot(N, x, incx, y, incy));
 }
 
 
@@ -856,7 +861,7 @@ void blas_csyrk(char uplo, char trans, long N, long K, const complex float alpha
 	assert('U' == uplo);
 	assert(('T' == trans) || ('N' == trans));
 
-	cblas_csyrk(CblasColMajor, CblasUpper, ('T' == trans) ? CblasTrans : CblasNoTrans, N, K, (void*)&alpha, (void*)A, lda, (void*)&beta, (void*)C, ldc);
+	cblas_csyrk(CblasColMajor, CblasUpper, ('T' == trans) ? CblasTrans : CblasNoTrans, N, K, &alpha, (void*)A, lda, (void*)&beta, (void*)C, ldc);
 }
 
 
