@@ -8,11 +8,15 @@
 #include <assert.h>
 #include <complex.h>
 #include <stdbool.h>
+#include <limits.h>
 
 #include "misc/misc.h"
 
 #include "num/blas.h"
 #include "num/multind.h"
+#ifdef USE_CUDA
+#include "num/gpuops.h"
+#endif
 
 #include "blas_md_wrapper.h"
 
@@ -47,7 +51,7 @@ static bool check_blas_strides(int N, const long str[N], long size)
  * @param istr1 must be of the form {0, 1, dim[1] + x} or {0, dim[2] + x, 1}
  * @param iptr1
  **/
-void blas_zfmac_cgemm(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zfmac_cgemm(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -91,7 +95,7 @@ void blas_zfmac_cgemm(unsigned int N, const long dims[N], const long ostr[N], co
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_zfmac_cgemv(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zfmac_cgemv(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -131,7 +135,7 @@ void blas_zfmac_cgemv(unsigned int N, const long dims[N], const long ostr[N], co
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_zfmac_cgeru(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zfmac_cgeru(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -165,7 +169,7 @@ void blas_zfmac_cgeru(unsigned int N, const long dims[N], const long ostr[N], co
  * @param istr2 must be of the form {0}
  * @param iptr2
  **/
-void blas_zfmac_caxpy(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zfmac_caxpy(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -181,7 +185,26 @@ void blas_zfmac_caxpy(unsigned int N, const long dims[N], const long ostr[N], co
 	long incx = istr1[0] / size;
 	long incy = ostr[0] / size;
 
-	blas2_caxpy(dims[0], iptr2, incx, iptr1, incy, optr);
+#ifdef USE_CUDA
+	if (cuda_ondevice(optr)) {
+
+		blas2_caxpy(dims[0], iptr2, incx, iptr1, incy, optr);
+
+		return;
+	}
+#endif
+
+	complex float val = *iptr2;
+
+	if ((1 == incx) && (1 == incy)) {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i] += iptr1[i] * val;
+	} else {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i * incy] += iptr1[i * incx] * val;
+	}
 }
 
 
@@ -196,7 +219,7 @@ void blas_zfmac_caxpy(unsigned int N, const long dims[N], const long ostr[N], co
  * @param istr2 must be of the form {1+x}
  * @param iptr2
  **/
-void blas_zfmac_cdotu(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zfmac_cdotu(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -215,8 +238,17 @@ void blas_zfmac_cdotu(unsigned int N, const long dims[N], const long ostr[N], co
 
 	complex float* tmp = md_alloc_sameplace(1, MAKE_ARRAY(1l), size, optr);
 
-	blas2_cdotu(tmp, dims[0], incx, iptr1, incy, iptr2);
-	blas_caxpy(1, 1., size, tmp, size, optr);
+	long S = dims[0];
+	
+	while (S > 0) {
+
+		blas2_cdotu(tmp, MIN(S, INT_MAX / 4), incx, iptr1, incy, iptr2);
+		blas_caxpy(1, 1., 1, tmp, 1, optr);
+
+		iptr1 += (INT_MAX / 4) * incx;
+		iptr2 += (INT_MAX / 4) * incy;
+		S -= (INT_MAX / 4);
+	}
 
 	md_free(tmp);
 }
@@ -240,7 +272,7 @@ void blas_zfmac_cdotu(unsigned int N, const long dims[N], const long ostr[N], co
  * @param istr1 must be of the form {0, 1, dim[1]} or {0, dim[2], 1}
  * @param iptr1
  **/
-void blas_fmac_sgemm(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_fmac_sgemm(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -283,7 +315,7 @@ void blas_fmac_sgemm(unsigned int N, const long dims[N], const long ostr[N], flo
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_fmac_sgemv(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_fmac_sgemv(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -323,7 +355,7 @@ void blas_fmac_sgemv(unsigned int N, const long dims[N], const long ostr[N], flo
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_fmac_sger(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_fmac_sger(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -358,7 +390,7 @@ void blas_fmac_sger(unsigned int N, const long dims[N], const long ostr[N], floa
  * @param istr2 must be of the form {0}
  * @param iptr2
  **/
-void blas_fmac_saxpy(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_fmac_saxpy(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -374,7 +406,26 @@ void blas_fmac_saxpy(unsigned int N, const long dims[N], const long ostr[N], flo
 	long incx = istr1[0] / size;
 	long incy = ostr[0] / size;
 
-	blas2_saxpy(dims[0], iptr2, incx, iptr1, incy, optr);
+#ifdef USE_CUDA
+	if (cuda_ondevice(optr)) {
+
+		blas2_saxpy(dims[0], iptr2, incx, iptr1, incy, optr);
+
+		return;
+	}
+#endif
+
+	float val = *iptr2;
+
+	if ((1 == incx) && (1 == incy)) {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i] += iptr1[i] * val;
+	} else {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i * incy] += iptr1[i * incx] * val;
+	}
 }
 
 
@@ -389,7 +440,7 @@ void blas_fmac_saxpy(unsigned int N, const long dims[N], const long ostr[N], flo
  * @param istr2 must be of the form {1+x}
  * @param iptr2
  **/
-void blas_fmac_sdot(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_fmac_sdot(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -408,8 +459,16 @@ void blas_fmac_sdot(unsigned int N, const long dims[N], const long ostr[N], floa
 
 	float* tmp = md_alloc_sameplace(1, MAKE_ARRAY(1l), size, optr);
 
-	blas2_sdot(tmp, dims[0], incx, iptr1, incy, iptr2);
-	blas_saxpy(1, 1., size, tmp, size, optr);
+	long S = dims[0];
+	while (S > 0) {
+
+		blas2_sdot(tmp, MIN(S, INT_MAX / 4), incx, iptr1, incy, iptr2);
+		blas_saxpy(1, 1., 1, tmp, 1, optr);
+
+		iptr1 += (INT_MAX / 4) * incx;
+		iptr2 += (INT_MAX / 4) * incy;
+		S -= (INT_MAX / 4);
+	}
 
 	md_free(tmp);
 }
@@ -432,7 +491,7 @@ void blas_fmac_sdot(unsigned int N, const long dims[N], const long ostr[N], floa
  * @param istr1 must be of the form {0, 0}
  * @param iptr1
  **/
-void blas_zmul_cmatcopy(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zmul_cmatcopy(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -467,7 +526,7 @@ void blas_zmul_cmatcopy(unsigned int N, const long dims[N], const long ostr[N],
  * @param iptr
  * @param val
  **/
-void blas_zsmul_cmatcopy(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr[N], const complex float* iptr, complex float val)
+void blas_zsmul_cmatcopy(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr[N], const complex float* iptr, complex float val)
 {
 	long size = 8;
 
@@ -501,7 +560,7 @@ void blas_zsmul_cmatcopy(unsigned int N, const long dims[N], const long ostr[N],
  * @param istr1 must be of the form {1+x, 0} or {0, 1+x}
  * @param iptr1
  **/
-void blas_zmul_cdgmm(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zmul_cdgmm(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -537,7 +596,7 @@ void blas_zmul_cdgmm(unsigned int N, const long dims[N], const long ostr[N], com
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_zmul_cgeru(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zmul_cgeru(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
@@ -557,17 +616,39 @@ void blas_zmul_cgeru(unsigned int N, const long dims[N], const long ostr[N], com
  * @param istr1 must be of the form {0}
  * @param iptr1
  **/
-void blas_zmul_cscal(unsigned int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
+void blas_zmul_cscal(int N, const long dims[N], const long ostr[N], complex float* optr, const long istr1[N], const complex float* iptr1, const long istr2[N], const complex float* iptr2)
 {
 	long size = 8;
 
-	assert(((optr != iptr1) || (ostr[0] == istr1[0])) && (0 == ostr[0] % size) && (0 == istr2[0]));
+	assert(((optr != iptr1) || (ostr[0] == istr1[0])) && (0 == ostr[0] % size) && (0 == istr1[0] % size) && (0 == istr2[0]));
 	assert(1 == N);
 
-	if (optr != iptr1)
-		md_copy2(N, dims, ostr, optr, istr1, iptr1, size);
+#ifdef USE_CUDA
+	if (cuda_ondevice(optr)) {
 
-	blas2_cscal(dims[0], iptr2, ostr[0] / size, optr);
+		if (optr != iptr1)
+			md_copy2(N, dims, ostr, optr, istr1, iptr1, size);
+
+		blas2_cscal(dims[0], iptr2, ostr[0] / size, optr);
+
+	return;
+	}
+#endif
+
+	complex float val = *iptr2;
+
+	if ((size == ostr[0]) && (size == istr1[0])) {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i] = iptr1[i] * val;
+	} else {
+
+		long ostride = ostr[0] / size;
+		long istride = istr1[0] / size;
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i * ostride] = iptr1[i * istride] * val;
+	}
 }
 
 
@@ -589,7 +670,7 @@ void blas_zmul_cscal(unsigned int N, const long dims[N], const long ostr[N], com
  * @param istr1 must be of the form {0, 0}
  * @param iptr1
  **/
-void blas_mul_smatcopy(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_mul_smatcopy(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -624,7 +705,7 @@ void blas_mul_smatcopy(unsigned int N, const long dims[N], const long ostr[N], f
  * @param iptr
  * @param val
  **/
-void blas_smul_smatcopy(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr[N], const float* iptr, float val)
+void blas_smul_smatcopy(int N, const long dims[N], const long ostr[N], float* optr, const long istr[N], const float* iptr, float val)
 {
 	long size = 4;
 
@@ -658,7 +739,7 @@ void blas_smul_smatcopy(unsigned int N, const long dims[N], const long ostr[N],
  * @param istr1 must be of the form {1+x, 0} or {0, 1+x}
  * @param iptr1
  **/
-void blas_mul_sdgmm(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_mul_sdgmm(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -693,7 +774,7 @@ void blas_mul_sdgmm(unsigned int N, const long dims[N], const long ostr[N], floa
  * @param istr1 must be of the form {0, 1+x}
  * @param iptr1
  **/
-void blas_mul_sger(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_mul_sger(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
@@ -713,15 +794,37 @@ void blas_mul_sger(unsigned int N, const long dims[N], const long ostr[N], float
  * @param istr1 must be of the form {0}
  * @param iptr1
  **/
-void blas_mul_sscal(unsigned int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
+void blas_mul_sscal(int N, const long dims[N], const long ostr[N], float* optr, const long istr1[N], const float* iptr1, const long istr2[N], const float* iptr2)
 {
 	long size = 4;
 
-	assert((ostr[0] == istr1[0]) && (0 == ostr[0] % size) && (0 == istr2[0]));
+	assert(((optr != iptr1) || (ostr[0] == istr1[0])) && (0 == ostr[0] % size) && (0 == istr1[0] % size) && (0 == istr2[0]));
 	assert(1 == N);
 
-	if (optr != iptr1)
-		md_copy2(N, dims, ostr, optr, istr1, iptr1, size);
+#ifdef USE_CUDA
+	if (cuda_ondevice(optr)) {
 
-	blas2_sscal(dims[0], iptr2, istr1[0] / size, optr);
+		if (optr != iptr1)
+			md_copy2(N, dims, ostr, optr, istr1, iptr1, size);
+
+		blas2_sscal(dims[0], iptr2, ostr[0] / size, optr);
+
+	return;
+	}
+#endif
+
+	float val = *iptr2;
+
+	if ((size == ostr[0]) && (size == istr1[0])) {
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i] = iptr1[i] * val;
+	} else {
+
+		long ostride = ostr[0] / size;
+		long istride = istr1[0] / size;
+
+		for (long i = 0; i < dims[0]; i++)
+			optr[i * ostride] = iptr1[i * istride] * val;
+	}
 }
