@@ -26,6 +26,7 @@
 #include "num/rand.h"
 #include "num/init.h"
 #include "num/vecops_strided.h"
+#include "num/vptr.h"
 
 #include "misc/nested.h"
 #include "misc/misc.h"
@@ -245,6 +246,9 @@ bool simple_zconvcorr(	unsigned int N, const long dims[N],
 	if (!use_simple_convcorr)
 		return false;
 
+	if (is_vptr(optr) || is_vptr(iptr1) || is_vptr(iptr2))
+		return false;
+
 	if (simple_zconvcorr_fwd(N, dims, ostrs, optr, istrs1, iptr1, istrs2, iptr2))
 		return true;
 
@@ -582,13 +586,13 @@ static bool check_trivial_cf(	int N,
 		return false;
 
 	// check contiguous memory
-	if ((unsigned int)N > md_calc_blockdim(N, odims, ostrs, size))
+	if (N > md_calc_blockdim(N, odims, ostrs, size))
 		return false;
 
-	if ((unsigned int)N > md_calc_blockdim(N, idims, istrs, size))
+	if (N > md_calc_blockdim(N, idims, istrs, size))
 		return false;
 
-	if ((unsigned int)N > md_calc_blockdim(N, kdims, kstrs, size))
+	if (N > md_calc_blockdim(N, kdims, kstrs, size))
 		return false;
 
 	return true;
