@@ -13,21 +13,21 @@
 
 struct iovec_s {
 	
-	unsigned int N;
+	int N;
 	const long* dims;
 	const long* strs;
 	size_t size;
 };
 
 
-extern const struct iovec_s* iovec_create(unsigned int N, const long dims[__VLA(N)], size_t size);
-extern const struct iovec_s* iovec_create2(unsigned int N, const long dims[__VLA(N)], const long strs[__VLA(N)], size_t size);
+extern const struct iovec_s* iovec_create(int N, const long dims[__VLA(N)], size_t size);
+extern const struct iovec_s* iovec_create2(int N, const long dims[__VLA(N)], const long strs[__VLA(N)], size_t size);
 extern void iovec_free(const struct iovec_s* x);
-extern bool iovec_check(const struct iovec_s* iov, unsigned int N, const long dims[__VLA(N)], const long strs[__VLA(N)]);
+extern bool iovec_check(const struct iovec_s* iov, int N, const long dims[__VLA(N)], const long strs[__VLA(N)]);
 
 
 // in-place initialization and deconstruction
-extern void iovec_init2(struct iovec_s* n, unsigned int N, const long dims[__VLA(N)], const long strs[__VLA(N)], size_t size);
+extern void iovec_init2(struct iovec_s* n, int N, const long dims[__VLA(N)], const long strs[__VLA(N)], size_t size);
 extern void iovec_destroy(const struct iovec_s* x);
 
 extern void debug_print_iovec(int level, const struct iovec_s* vec);
