@@ -1,11 +1,13 @@
 /* Copyright 2015. The Regents of the University of California.
- * Copyright 2016-2019. Martin Uecker.
+ * Copyright 2016-2020. Uecker Lab. University Medical Center Göttingen.
+ * Copyright 2023. Institute of Biomedical Imaging. TU Graz.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors:
- * 2012-2019 Martin Uecker <martin.uecker@med.uni-goettingen.de>
- * 2013 Dara Bahri <dbahri123@gmail.com>
+ * 2019-2020 Sebastian Rosenzweig
+ * 2012-2020 Martin Uecker
+ * 2013	     Dara Bahri
  *
  *
  * Simple linear algebra functions.
@@ -56,6 +58,26 @@ void mat_zero(int A, int B, complex float m[A][B])
 			m[a][b] = 0.;
 }
 
+void vec_zero(int N, complex float x[N])
+{
+	for (int i = 0; i < N; i++)
+		x[i] = 0.;
+}
+
+void vec_copy(int N, complex float x[N], const complex float y[N])
+{
+	for (int i = 0; i < N; i++)
+		x[i] = y[i];
+}
+
+void vecf_copy(int N, float x[N], const float y[N])
+{
+	for (int i = 0; i < N; i++)
+		x[i] = y[i];
+}
+
+
+
 void mat_gaussian(int A, int B, complex float x[A][B])
 {
 	for (int i = 0; i < A; i++)
@@ -81,14 +103,14 @@ complex float vec_mean(long D, const complex float src[D])
 	return val / D;
 }
 
-void (mat_add)(int A, int B, complex float x[A][B], const complex float y[A][B], const complex float z[A][B])
+void mat_add(int A, int B, complex float x[A][B], const complex float y[A][B], const complex float z[A][B])
 {
 	for (int i = 0; i < A; i++)
 		for (int j = 0; j < B; j++)
 			x[i][j] = y[i][j] + z[i][j];
 }
 
-void (mat_muladd)(int A, int B, int C, complex float x[MVLA(A)][C], const complex float y[MVLA(A)][B], const complex float z[MVLA(B)][C])
+void mat_muladd(int A, int B, int C, complex float x[MVLA(A)][C], const complex float y[MVLA(A)][B], const complex float z[MVLA(B)][C])
 {
 #ifdef MAT_USE_LAPACK
 	complex float tmp[A][C];
@@ -110,7 +132,7 @@ void (mat_muladd)(int A, int B, int C, complex float x[MVLA(A)][C], const comple
 }
 
 
-void (mat_mul)(int A, int B, int C, complex float x[A][C], const complex float y[A][B], const complex float z[B][C])
+void mat_mul(int A, int B, int C, complex float x[A][C], const complex float y[A][B], const complex float z[B][C])
 {
 #ifdef MAT_USE_LAPACK
 	blas_matrix_multiply(C, A, B, x, z, y);
@@ -130,7 +152,41 @@ void (mat_mul)(int A, int B, int C, complex float x[A][C], const complex float y
 }
 
 
-bool (mat_inverse)(unsigned int N, complex float out[N][N], const complex float in[N][N])
+complex float mat_det(int N, const complex float mat[N][N])
+{
+	assert(N <= 2);
+
+	if (0 == N)
+		return 0.;
+
+	if (1 == N)
+		return mat[0][0];
+
+	if (2 == N)
+		return mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
+
+	return 0.;
+}
+
+
+void matf_mul(int A, int B, int C, float x[A][C], const float y[A][B], const float z[B][C])
+{
+	for (int i = 0; i < A; i++) {
+		for (int j = 0; j < C; j++) {
+
+			fl_acu_t tmp = 0.;
+
+			for (int k = 0; k < B; k++)
+				tmp += y[i][k] * z[k][j];
+
+			x[i][j] = tmp;
+		}
+	}
+}
+
+
+
+bool mat_inverse(int N, complex float out[N][N], const complex float in[N][N])
 {
 #ifdef MAT_USE_LAPACK
 //	return blas_matrix_inverse(N, out, in);
@@ -147,17 +203,17 @@ bool (mat_inverse)(unsigned int N, complex float out[N][N], const complex float
 	complex float tmp2[N][2 * N];
 	mat_transpose(2 * N, N, tmp2, tmp);
 
-	for (unsigned int i = 0; i < N; i++) {
+	for (int i = 0; i < N; i++) {
 
 		complex float diag = tmp2[i][i];
 
 		if (0. == diag)
 			return false;
 
-		for (unsigned int j = 0; j < 2 * N; j++)
+		for (int j = 0; j < 2 * N; j++)
 			tmp2[i][j] /= diag;
 
-		for (unsigned int j = 0; j < N; j++) {
+		for (int j = 0; j < N; j++) {
 
 			if (i != j)
 				vec_saxpy(2 * N, tmp2[j], -tmp2[j][i], tmp2[i]);
@@ -172,8 +228,14 @@ bool (mat_inverse)(unsigned int N, complex float out[N][N], const complex float
 }
 
 
+// Moore-Penrose pseudo inverse
+void mat_pinv(int A, int B, complex float out[B][A], const complex float in[A][B])
+{
+	((B <= A) ? mat_pinv_left : mat_pinv_right)(A, B, out, in);
+}
 
-void mat_pinv(unsigned int A, unsigned int B, complex float out[B][A], const complex float in[A][B])
+
+void mat_pinv_left(int A, int B, complex float out[B][A], const complex float in[A][B])
 {
 	if (A == B) {
 
@@ -196,41 +258,79 @@ void mat_pinv(unsigned int A, unsigned int B, complex float out[B][A], const com
 }
 
 
-void (mat_kron)(unsigned int A, unsigned int B, unsigned int C, unsigned int D, 
-		complex float out[A * C][B * D], const complex float in1[A][B], const complex float in2[C][D])
+void mat_pinv_right(int A, int B, complex float out[B][A], const complex float in[A][B])
 {
-	for (unsigned int a = 0; a < A; a++)
-		for (unsigned int b = 0; b < B; b++)
-			for (unsigned int c = 0; c < C; c++)
-				for (unsigned int d = 0; d < D; d++)
+	if (A == B) {
+
+		mat_inverse(A, out, in);
+		return;
+	}
+
+	assert(A < B);
+
+	complex float adj[B][A];
+	mat_adjoint(A, B, adj, in);
+
+	complex float prod[A][A];
+	mat_mul(A, B, A, prod, in, adj);
+
+	complex float inv[A][A];
+	mat_inverse(A, inv, prod);
+
+	mat_mul(B, A, A, out, adj, inv);
+}
+
+
+void mat_kron(int A, int B, int C, int D,
+	      complex float out[A * C][B * D], const complex float in1[A][B], const complex float in2[C][D])
+{
+	for (int a = 0; a < A; a++)
+		for (int b = 0; b < B; b++)
+			for (int c = 0; c < C; c++)
+				for (int d = 0; d < D; d++)
 					out[a + c * A][b + d * B] = in1[a][b] * in2[c][d];
 }
 
 
-void (mat_vecmul)(unsigned int A, unsigned int B, complex float out[A], const complex float mat[A][B], const complex float in[B])
+void mat_vecmul(int A, int B, complex float out[A], const complex float mat[A][B], const complex float in[B])
 {
-	for (unsigned int a = 0; a < A; a++) {
+	for (int a = 0; a < A; a++) {
 
 		cfl_acu_t tmp = 0.;
 
-		for (unsigned int b = 0; b < B; b++)
+		for (int b = 0; b < B; b++)
 			tmp += mat[a][b] * in[b];
 
 		out[a] = tmp;
 	}
 }
 
-void (mat_vec)(unsigned int A, unsigned int B, complex float out[A * B], const complex float in[A][B])
+
+void matf_vecmul(int A, int B, float out[A], const float mat[A][B], const float in[B])
 {
-	for (unsigned int a = 0; a < A; a++)
-		for (unsigned int b = 0; b < B; b++)
+	for (int a = 0; a < A; a++) {
+
+		fl_acu_t tmp = 0.;
+
+		for (int b = 0; b < B; b++)
+			tmp += mat[a][b] * in[b];
+
+		out[a] = tmp;
+	}
+}
+
+
+void mat_vec(int A, int B, complex float out[A * B], const complex float in[A][B])
+{
+	for (int a = 0; a < A; a++)
+		for (int b = 0; b < B; b++)
 			out[a * B + b] = in[a][b];
 }
 
-void (vec_mat)(unsigned int A, unsigned int B, complex float out[A][B], const complex float in[A * B])
+void vec_mat(int A, int B, complex float out[A][B], const complex float in[A * B])
 {
-	for (unsigned int a = 0; a < A; a++)
-		for (unsigned int b = 0; b < B; b++)
+	for (int a = 0; a < A; a++)
+		for (int b = 0; b < B; b++)
 			out[a][b] = in[a * B + b];
 }
 
@@ -263,7 +363,7 @@ void vec_saxpy(int N, complex float x[N], complex float alpha, const complex flo
 		x[k] += alpha * y[k];
 }
 
-void (gram_matrix)(int N, complex float cov[N][N], int L, const complex float data[N][L])
+void gram_matrix(int N, complex float cov[N][N], int L, const complex float data[N][L])
 {
 #pragma omp parallel for
 	for (int i = 0; i < N; i++) {
@@ -277,7 +377,7 @@ void (gram_matrix)(int N, complex float cov[N][N], int L, const complex float da
 	}
 }
 
-void (pack_tri_matrix)(int N, complex float cov[N * (N + 1) / 2], const complex float m[N][N])
+void pack_tri_matrix(int N, complex float cov[N * (N + 1) / 2], const complex float m[N][N])
 {
 	int l = 0;
 
@@ -286,7 +386,7 @@ void (pack_tri_matrix)(int N, complex float cov[N * (N + 1) / 2], const complex
 			cov[l++] = m[i][j];
 }
 
-void (unpack_tri_matrix)(int N, complex float m[N][N], const complex float cov[N * (N + 1) / 2])
+void unpack_tri_matrix(int N, complex float m[N][N], const complex float cov[N * (N + 1) / 2])
 {
 	int l = 0;
 
@@ -295,7 +395,7 @@ void (unpack_tri_matrix)(int N, complex float m[N][N], const complex float cov[N
 			m[i][j] = cov[l++];
 }
 
-void (gram_matrix2)(int N, complex float cov[N * (N + 1) / 2], int L, const complex float data[N][L])
+void gram_matrix2(int N, complex float cov[N * (N + 1) / 2], int L, const complex float data[N][L])
 {
 #if 0
 	int l = 0;
@@ -344,6 +444,13 @@ void (mat_transpose)(int A, int B, complex float dst[B][A], const complex float
 			dst[i][j] = src[j][i];	// swap
 }
 
+void matf_transpose(int A, int B, float dst[B][A], const float src[A][B])
+{
+	for (int i = 0; i < B; i++)
+		for (int j = 0; j < A; j++)
+			dst[i][j] = src[j][i];	// swap
+}
+
 void (mat_adjoint)(int A, int B, complex float dst[B][A], const complex float src[A][B])
 {
 	for (int i = 0; i < B; i++)
@@ -358,6 +465,14 @@ void (mat_copy)(int A, int B, complex float dst[A][B], const complex float src[A
 			dst[i][j] = src[i][j];
 }
 
+
+void matf_copy(int N, int M, float out[N][M], const float in[N][M])
+{
+	for (int i = 0; i < N; i++)
+		for (int j = 0; j < M; j++)
+			out[i][j] = in[i][j];
+}
+
 void (mat_conj)(int A, int B, complex float dst[A][B], const complex float src[A][B])
 {
 	for (int i = 0; i < A; i++)
