@@ -1,5 +1,6 @@
 /* Copyright 2015. The Regents of the University of California.
  * Copyright 2016-2019. Martin Uecker.
+ * Copyright 2023. Institute of Biomedical Imaging. TU Graz.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
@@ -27,6 +28,7 @@
 #include "num/iovec.h"
 #include "num/flpmath.h"
 #include "num/ops_graph.h"
+#include "num/vptr.h"
 
 #include "misc/misc.h"
 #include "misc/types.h"
@@ -388,7 +390,8 @@ static void identity_apply(const operator_data_t* _data, unsigned int N, void* a
 {
 	const auto d = CAST_DOWN(identity_s, _data);
 	assert(2 == N);
-	md_copy2(d->domain->N, d->domain->dims, d->codomain->strs, args[0], d->domain->strs, args[1], d->domain->size);
+	if (args[0] != args[1])
+		md_copy2(d->domain->N, d->domain->dims, d->codomain->strs, args[0], d->domain->strs, args[1], d->domain->size);
 }
 
 
@@ -533,6 +536,7 @@ const struct operator_s* get_in_reshape(const struct operator_s* op) {
 	return NULL;
 }
 
+
 struct zero_s {
 
 	INTERFACE(operator_data_t);
@@ -619,7 +623,7 @@ void operator_generic_apply_unchecked(const struct operator_s* op, unsigned int
 	debug_trace("LEAVE %p\n", op->apply);
 }
 
-void operator_generic_apply_parallel_unchecked(int D, const struct operator_s* op[__VLA(D)], unsigned int N, void* args[__VLA(D)][N], int num_threads)
+void operator_generic_apply_parallel_unchecked(int D, const struct operator_s* op[D], unsigned int N, void* args[D][N], int num_threads)
 {
 #ifdef _OPENMP
 	if (0 < num_threads) {
@@ -631,7 +635,6 @@ void operator_generic_apply_parallel_unchecked(int D, const struct operator_s* o
 		{
 			for (int i = omp_get_thread_num(); i < D; i += omp_get_num_threads())
 				operator_generic_apply_unchecked(op[i], N, args[i]);
-	
 		}
 
 		omp_set_num_threads(max_threads);
@@ -671,7 +674,7 @@ void operator_apply_parallel_unchecked(int D, const struct operator_s* op[D], co
 	operator_generic_apply_parallel_unchecked(D, op, 2, args, num_threads);
 }
 
-void operator_apply2(const struct operator_s* op, unsigned int ON, const long odims[ON], const long ostrs[ON], complex float* dst, const long IN, const long idims[IN], const long istrs[ON], const complex float* src)
+void operator_apply2(const struct operator_s* op, unsigned int ON, const long odims[ON], const long ostrs[ON], complex float* dst, const long IN, const long idims[IN], const long istrs[IN], const complex float* src)
 {
 	assert(2 == op->N);
 	assert(iovec_check(op->domain[1], IN, idims, istrs));
@@ -855,7 +858,6 @@ struct op_loop_s {
 	const struct operator_s* op;
 
 	unsigned int parallel;
-	bool gpu;
 };
 
 static DEF_TYPEID(op_loop_s);
@@ -882,16 +884,6 @@ static void op_loop_fun(const operator_data_t* _data, unsigned int N, void* args
 	const auto data = CAST_DOWN(op_loop_s, _data);
 	assert(N == data->N);
 
-	if (data->gpu) {
-#if defined(USE_CUDA) && defined(_OPENMP)
-		int nr_cuda_devices = cuda_num_devices();
-		omp_set_num_threads(nr_cuda_devices * 2);
-//		fft_set_num_threads(1);
-#else
-		error("Both OpenMP and CUDA are necessary for op_loop_fun. At least one was not found.\n");
-#endif
-	}
-
 	extern bool num_auto_parallelize;
 	bool ap_save = num_auto_parallelize;
 	num_auto_parallelize = false;
@@ -902,7 +894,6 @@ static void op_loop_fun(const operator_data_t* _data, unsigned int N, void* args
 	};
 
 	md_parallel_nary(N, data->D, data->dims0, data->parallel, data->strs, args, op_loop_nary);
-
 	num_auto_parallelize = ap_save;
 }
 
@@ -919,10 +910,10 @@ static void merge_dims(unsigned int D, long odims[D], const long idims1[D], cons
 	}
 }
 
-const struct operator_s* (operator_loop_parallel2)(unsigned int N, const unsigned int D,
+const struct operator_s* operator_loop_parallel2(unsigned int N, unsigned int D,
 				const long dims[D], const long (*strs)[D],
 				const struct operator_s* op,
-				unsigned int flags, bool gpu)
+				unsigned int flags)
 
 {
 	assert(N == operator_nr_args(op));
@@ -937,13 +928,13 @@ const struct operator_s* (operator_loop_parallel2)(unsigned int N, const unsigne
 
 	// TODO: we should have a flag and ignore args with flag
 
-	for (unsigned int i = 0; i < N; i++) {
+	for (int i = 0; i < (int)N; i++) {
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
 
-		assert(D == io->N);
+		assert((int)D == io->N);
 
-		for (unsigned int j = 0; j < D; j++) {
+		for (int j = 0; j < (int)D; j++) {
 
 			assert((0 == io->strs[j]) || (io->strs[j] == strs[i][j]));
 			assert((1 == io->dims[j]) == (0 == io->strs[j]));
@@ -972,7 +963,6 @@ const struct operator_s* (operator_loop_parallel2)(unsigned int N, const unsigne
 	data->strs = */*PTR_PASS*/(strs2);
 
 	data->parallel = flags;
-	data->gpu = gpu;
 
 	const struct operator_s* rop = operator_generic_create2(N, op->io_flags, D2, *dims2, *strs2, CAST_UP(PTR_PASS(data)), op_loop_fun, op_loop_del, NULL);
 
@@ -986,10 +976,10 @@ const struct operator_s* (operator_loop2)(unsigned int N, const unsigned int D,
 				const long dims[D], const long (*strs)[D],
 				const struct operator_s* op)
 {
-	return operator_loop_parallel2(N, D, dims, strs, op, 0u, false);
+	return operator_loop_parallel2(N, D, dims, strs, op, 0u);
 }
 
-const struct operator_s* operator_loop_parallel(unsigned int D, const long dims[D], const struct operator_s* op, unsigned int parallel, bool gpu)
+const struct operator_s* operator_loop_parallel(unsigned int D, const long dims[D], const struct operator_s* op, unsigned int parallel)
 {
 	unsigned int N = operator_nr_args(op);
 	long strs[N][D];
@@ -1002,14 +992,15 @@ const struct operator_s* operator_loop_parallel(unsigned int D, const long dims[
 		md_calc_strides(D, strs[i], tdims, operator_arg_domain(op, i)->size);
 	}
 
-	return operator_loop_parallel2(N, D, dims, strs, op, parallel, gpu);
+	return operator_loop_parallel2(N, D, dims, strs, op, parallel);
 }
 
 const struct operator_s* operator_loop(unsigned int D, const long dims[D], const struct operator_s* op)
 {
-	return operator_loop_parallel(D, dims, op, 0u, false);
+	return operator_loop_parallel(D, dims, op, 0u);
 }
 
+enum COPY_LOCATION { CL_SAMEPLACE, CL_DEVICE, CL_CPU };
 
 struct copy_data_s {
 
@@ -1017,10 +1008,13 @@ struct copy_data_s {
 
 	const struct operator_s* op;
 
-	const void* ref;
-
-	unsigned int N;
+	int N;
 	const long** strs;
+	
+	enum COPY_LOCATION* loc;
+
+	_Bool copy_output;
+	// select if operator reads data from output
 };
 
 static DEF_TYPEID(copy_data_s);
@@ -1030,21 +1024,46 @@ static void copy_fun(const operator_data_t* _data, unsigned int N, void* args[N]
 	const auto data = CAST_DOWN(copy_data_s, _data);
 	const struct operator_s* op = data->op;
 	void* ptr[N];
-	bool allocated[N];
 
 	assert(N == operator_nr_args(op));
 
+
 	for (unsigned int i = 0; i < N; i++) {
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
 
-		allocated[i] = (!md_check_equal_dims(io->N, io->strs, data->strs[i], ~0) || ((NULL != data->ref) && !md_is_sameplace(data->ref, args[i])));
+		ptr[i] = NULL;
 
-		if (allocated[i]) {
+		if(NULL == args[i])
+			continue;
 
-			ptr[i] = md_alloc_sameplace(io->N, io->dims, io->size, NULL == data->ref ? args[i] : data->ref);
+		bool allocate = !md_check_equal_dims(io->N, io->strs, data->strs[i], md_nontriv_dims(io->N, io->dims));
 
-			if (!op->io_flags[i])
+#ifdef USE_CUDA
+		switch (data->loc[i]) {
+		case CL_CPU:
+			if (allocate || cuda_ondevice(args[i]))
+				ptr[i] = md_alloc(io->N, io->dims, io->size);
+			break;
+		case CL_SAMEPLACE:
+			if (allocate)
+				ptr[i] = md_alloc_sameplace(io->N, io->dims, io->size, args[i]);
+			break;
+		case CL_DEVICE:
+
+			if (allocate || !cuda_ondevice(args[i]))
+				ptr[i] = md_alloc_gpu(io->N, io->dims, io->size);
+			break;
+		default: assert(0);
+		}
+#else
+		if (allocate)
+			ptr[i] = md_alloc(io->N, io->dims, io->size);
+#endif
+
+		if (NULL != ptr[i]) {
+
+			if (data->copy_output || !op->io_flags[i])
 				md_copy2(io->N, io->dims, io->strs, ptr[i], data->strs[i], args[i], io->size);
 		} else {
 
@@ -1054,15 +1073,18 @@ static void copy_fun(const operator_data_t* _data, unsigned int N, void* args[N]
 
 	operator_generic_apply_unchecked(op, N, ptr);
 
-	for (unsigned int i = 0; i < N; i++) {
+
+	for (int i = 0; i < (int)N; i++) {
+
+		if (args[i] == ptr[i])
+			continue;
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
 
-		if (op->io_flags[i] && allocated[i])
+		if (op->io_flags[i])
 			md_copy2(io->N, io->dims, data->strs[i], args[i], io->strs, ptr[i], io->size);
 
-		if (allocated[i])
-			md_free(ptr[i]);
+		md_free(ptr[i]);
 	}
 }
 
@@ -1071,28 +1093,60 @@ static void copy_del(const operator_data_t* _data)
 	const auto data = CAST_DOWN(copy_data_s, _data);
 
 	operator_free(data->op);
-	md_free(data->ref);
 
-	for (unsigned int i = 0; i < data->N; i++)
+	for (int i = 0; i < data->N; i++)
 		xfree(data->strs[i]);
 
 	xfree(data->strs);
+	xfree(data->loc);
 	xfree(data);
 }
 
-const struct operator_s* operator_copy_wrapper_sameplace(unsigned int N, const long* strs[N], const struct operator_s* op, const void* ref)
+static const struct graph_s* copy_wrapper_graph_create(const struct operator_s* op)
 {
-	assert(N == operator_nr_args(op));
+	auto data = CAST_DOWN(copy_data_s, op->data);
+
+	auto subgraph = operator_get_graph(data->op);
+	auto result = create_graph_container(op, "coppy wrapper", subgraph);
+
+	return result;
+}
+
+static const struct operator_s* operator_copy_wrapper_generic(int N, const long* strs[N], enum COPY_LOCATION loc[N], const struct operator_s* op, int device, bool copy_output)
+{
+	assert(N == (int)operator_nr_args(op));
+
+	bool keep_strs = true;
+	for (int i = 0; i < N; i++)
+		keep_strs = keep_strs && (NULL == strs[i]);
+
+	// merge gpu wrapper with copy wrpper (stides)
+	if (keep_strs && (NULL != CAST_MAYBE(copy_data_s, op->data))) {
+
+		auto data = CAST_DOWN(copy_data_s, op->data);
+
+		enum COPY_LOCATION loc2[N];
+		for (int i = 0; i < N; i++)
+			loc2[i] = (CL_SAMEPLACE == data->loc[i]) ? loc[i] : data->loc[i];
+			
+		return operator_copy_wrapper_generic(N, data->strs, loc2, data->op, device, data->copy_output);
+	}
 
 	PTR_ALLOC(struct copy_data_s, data);
 	SET_TYPEID(copy_data_s, data);
-	data->op = operator_ref(op);
 
 	unsigned int D[N];
 	const long* dims[N];
 	const long* (*strs2)[N] = TYPE_ALLOC(const long*[N]);
 
-	for (unsigned int i = 0; i < N; i++) {
+	data->op = operator_ref(op);
+
+	data->N = N;
+	data->strs = *strs2;
+	data->loc = *TYPE_ALLOC(enum COPY_LOCATION[N]);
+	data->copy_output = copy_output;
+
+	for (int i = 0; i < N; i++) {
 
 		const struct iovec_s* io = operator_arg_domain(op, i);
 
@@ -1100,7 +1154,7 @@ const struct operator_s* operator_copy_wrapper_sameplace(unsigned int N, const l
 		dims[i] = io->dims;
 
 		long (*strsx)[io->N] = TYPE_ALLOC(long[io->N]);
-		md_copy_strides(io->N, *strsx, strs[i]);
+		md_copy_strides(io->N, *strsx, (NULL == strs[i]) ? io->strs : strs[i]);
 		(*strs2)[i] = *strsx;
 
 		// check for trivial strides
@@ -1108,137 +1162,76 @@ const struct operator_s* operator_copy_wrapper_sameplace(unsigned int N, const l
 		long tstrs[io->N];
 		md_calc_strides(io->N, tstrs, io->dims, CFL_SIZE);
 
-		for (unsigned int i = 0; i < io->N; i++)
+		for (int i = 0; i < io->N; i++)
 			if (1 != io->dims[i])
 				assert(io->strs[i] == tstrs[i]);
+
+		data->loc[i] = loc[i];
 	}
 
-	data->N = N;
-	data->strs = *strs2;
-	data->ref = (NULL == ref) ? NULL : md_alloc_sameplace(1, MD_DIMS(1), FL_SIZE, ref);
-
-	return operator_generic_create2(N, op->io_flags, D, dims, *strs2, CAST_UP(PTR_PASS(data)), copy_fun, copy_del, NULL);
+	return operator_generic_create2(N, op->io_flags, D, dims, *strs2, CAST_UP(PTR_PASS(data)), copy_fun, copy_del, copy_wrapper_graph_create);
 }
 
+const struct operator_s* operator_cpu_wrapper(const struct operator_s* op)
+{
+	int ref = 1;
+	int N = operator_nr_args(op);
+
+	const long* strs[N];
+
+	for (int i = 0; i < N; i++) {
+
+		auto dom = operator_arg_domain(op, i);
+		assert(md_check_equal_dims(dom->N, MD_STRIDES(dom->N, dom->dims, dom->size), dom->strs, ~0));
+		strs[i] = dom->strs;
+	}
+
+	return operator_copy_wrapper_sameplace(N, strs, op, &ref);
+}
+
+
+const struct operator_s* operator_copy_wrapper_sameplace(unsigned int N, const long* strs[N], const struct operator_s* op, const void* ref)
+{
+	enum COPY_LOCATION loc[N];
+	for (int i = 0; i < (int)N; i++) {
+
+	#ifdef USE_CUDA
+		if (cuda_ondevice(ref))
+			loc[i] = CL_DEVICE;
+		else
+	#endif
+			loc[i] = CL_CPU;
+		
+		if (NULL == ref)
+			loc[i] = CL_SAMEPLACE;
+	}
+
+	assert(N == operator_nr_args(op));
+
+	return operator_copy_wrapper_generic(N, strs, loc, op, -2 /*=no change of device*/, false);
+}
+
+
 const struct operator_s* operator_copy_wrapper(unsigned int N, const long* strs[N], const struct operator_s* op)
 {
 	return operator_copy_wrapper_sameplace(N, strs, op, NULL);
 }
 
-
-
-
-struct gpu_data_s {
-
-	INTERFACE(operator_data_t);
-
-	long move_flags;
-	const struct operator_s* op;
-};
-
-static DEF_TYPEID(gpu_data_s);
-
-
-
-#if defined(USE_CUDA) && defined(_OPENMP)
-#include <omp.h>
-#define MAX_CUDA_DEVICES 16
-omp_lock_t gpulock[MAX_CUDA_DEVICES];
-#endif
-
-
-static void gpuwrp_fun(const operator_data_t* _data, unsigned int N, void* args[N])
-{
-#if defined(USE_CUDA) && defined(_OPENMP)
-	const auto data = CAST_DOWN(gpu_data_s, _data);
-	const auto op = data->op;
-	void* gpu_ptr[N];
-
-	assert(N == operator_nr_args(op));
-
-	debug_printf(DP_DEBUG1, "GPU start.\n");
-
-	int nr_cuda_devices = MIN(cuda_num_devices(), MAX_CUDA_DEVICES);
-	int gpun = omp_get_thread_num() % nr_cuda_devices;
-
-	cuda_set_device(gpun);
-
-	for (unsigned int i = 0; i < N; i++) {
-
-		if (!MD_IS_SET(data->move_flags, i)) {
-
-			gpu_ptr[i] = args[i];
-			continue;
-		}
-
-		const struct iovec_s* io = operator_arg_domain(op, i);
-
-		if (op->io_flags[i])
-			gpu_ptr[i] = md_alloc_gpu(io->N, io->dims, io->size);
-		else
-			gpu_ptr[i] = md_gpu_move(io->N, io->dims, args[i], io->size);
-	}
-
-	omp_set_lock(&gpulock[gpun]);
-	operator_generic_apply_unchecked(op, N, gpu_ptr);
-	omp_unset_lock(&gpulock[gpun]);
-
-	for (unsigned int i = 0; i < N; i++) {
-
-		if (!MD_IS_SET(data->move_flags, i))
-			continue;
-
-		const struct iovec_s* io = operator_arg_domain(op, i);
-
-		if (op->io_flags[i])
-			md_copy(io->N, io->dims, args[i], gpu_ptr[i], io->size);
-
-		md_free(gpu_ptr[i]);
-	}
-
-	debug_printf(DP_DEBUG1, "GPU end.\n");
-
-#else
-	UNUSED(_data); UNUSED(N); UNUSED(args);
-	error("Both OpenMP and CUDA are necessary for automatic GPU execution. At least one was not found.\n");
-#endif
-}
-
-static void gpuwrp_del(const operator_data_t* _data)
-{
-	const auto data = CAST_DOWN(gpu_data_s, _data);
-
-	operator_free(data->op);
-
-	xfree(data);
-}
-
 const struct operator_s* operator_gpu_wrapper2(const struct operator_s* op, long move_flags)
 {
 	unsigned int N = operator_nr_args(op);
 	assert(N <= 8 * sizeof(move_flags));
 
-	unsigned int D[N];
-	const long* dims[N];
+	enum COPY_LOCATION loc[N];
 	const long* strs[N];
 
-	for (unsigned int i = 0; i < N; i++) {
+	for (int i = 0; i < (int)N; i++) {
 
-		const struct iovec_s* io = operator_arg_domain(op, i);
-
-		D[i] = io->N;
-		dims[i] = io->dims;
-		strs[i] = io->strs;
+		loc[i] = (MD_IS_SET(move_flags, i)) ? CL_DEVICE : CL_SAMEPLACE;
+		strs[i] = NULL;
 	}
 
-	// op = operator_ref(op);
-	PTR_ALLOC(struct gpu_data_s, data);
-	SET_TYPEID(gpu_data_s, data);
-
-	data->move_flags = move_flags;
-	data->op = operator_ref(op);
-
-	return operator_generic_create2(N, op->io_flags, D, dims, strs, CAST_UP(PTR_PASS(data)), gpuwrp_fun, gpuwrp_del, NULL);
+	return operator_copy_wrapper_generic(N, strs, loc, op, -1 /*select gpu by thread*/, false);
 }
 
 
@@ -1248,6 +1241,108 @@ const struct operator_s* operator_gpu_wrapper(const struct operator_s* op)
 }
 
 
+const struct operator_s* operator_assign_gpu(const struct operator_s* op, int device)
+{
+	int N = op->N;
+	enum COPY_LOCATION loc[N];
+	const long* strs[N];
+
+	for (int i = 0; i < N; i++) {
+
+		loc[i] = CL_DEVICE;
+		strs[i] = NULL;
+	}
+
+	return operator_copy_wrapper_generic(N, strs, loc, op, device, false);
+}
+
+
+struct vptr_wrapper_s {
+
+	INTERFACE(operator_data_t);
+
+	const struct operator_s* op;
+	struct vptr_hint_s* hint;
+};
+
+static DEF_TYPEID(vptr_wrapper_s);
+
+
+static void vptrw_apply(const operator_data_t* _data, unsigned int N, void* args[N])
+{
+	const auto d = CAST_DOWN(vptr_wrapper_s, _data);
+
+	void* targs[N];
+	for (int i = 0; i < (int)N; i++) {
+
+		auto iov = operator_arg_domain(d->op, i);
+		if (is_vptr(args[i]))
+			targs[i] = args[i];
+		else
+			targs[i] = vptr_wrap(iov->N, iov->dims, iov->size, args[i], d->hint, false, operator_get_io_flags(d->op)[i]);
+	}
+
+	operator_generic_apply_unchecked(d->op, N, targs);
+
+	for (int i = 0; i < (int)N; i++)
+		if (targs[i] != args[i])
+			md_free(targs[i]);
+}
+
+static void vptrw_free(const operator_data_t* _data)
+{
+	const auto d = CAST_DOWN(vptr_wrapper_s, _data);
+	operator_free(d->op);
+	vptr_hint_free(d->hint);
+	xfree(d);
+}
+
+const struct operator_s* operator_vptr_wrapper(const struct operator_s* op, struct vptr_hint_s* hint)
+{
+	if (NULL == hint)
+		return operator_ref(op);
+
+	int A = operator_nr_args(op);
+	int N = -1;
+
+	for (int i = 0; i < A; i++) {
+
+		auto iov = operator_arg_domain(op, i);
+
+		if (1 == iov->N && 1 == iov->dims[0])
+			continue;
+
+		if (-1 == N)
+			N = iov->N;
+
+		assert(N == iov->N);
+		assert(N == md_calc_blockdim(N, iov->dims, iov->strs, iov->size));
+	}
+
+	assert(0 < N);
+
+	PTR_ALLOC(struct vptr_wrapper_s, data);
+	SET_TYPEID(vptr_wrapper_s, data);
+
+	data->op = operator_ref(op);
+	data->hint = vptr_hint_ref(hint);
+
+	unsigned int D[N];
+	const long* op_dims[N];
+	const long* op_strs[A];
+
+	for (int j = 0; j < A; j++) {
+
+		auto iov = operator_arg_domain(op, j);
+		D[j] = iov->N;
+		op_dims[j] = iov->dims;
+		op_strs[j] = iov->strs;
+	}
+
+	return operator_generic_create2(A, op->io_flags, D, op_dims, op_strs, CAST_UP(PTR_PASS(data)), vptrw_apply, vptrw_free, NULL);
+}
+
+
 struct operator_combi_s {
 
 	INTERFACE(operator_data_t);
@@ -1267,7 +1362,7 @@ static void combi_apply(const operator_data_t* _data, unsigned int N, void* args
 
 	int off = N;
 
-	for (int i = (int)data->N - 1; 0 <= i; i--) {
+	for (int i = data->N - 1; 0 <= i; i--) {
 
 		int A = operator_nr_args(data->x[i]);
 
@@ -1353,6 +1448,16 @@ const struct operator_s* operator_combi_create(int N, const struct operator_s* x
 }
 
 
+const struct operator_s* operator_combi_create_FF(int N, const struct operator_s* x[N])
+{
+	auto ret = operator_combi_create(N, x);
+
+	for (int i = 0; i < N; i++)
+		operator_free(x[i]);
+
+	return ret;
+}
+
 
 
 struct operator_dup_s {
@@ -1460,7 +1565,12 @@ const struct operator_s* operator_dup_create(const struct operator_s* op, unsign
 	return operator_generic_create2(N - 1, io_flags, D, dims, strs, CAST_UP(PTR_PASS(data)), dup_apply, dup_del, operator_dup_get_graph);
 }
 
-
+const struct operator_s* operator_dup_create_F(const struct operator_s* op, unsigned int a, unsigned int b)
+{
+	auto ret = operator_dup_create(op, a, b);
+	operator_free(op);
+	return ret;
+}
 
 
 // FIXME: we should reimplement link in terms of dup and bind (caveat: gpu; io_flags)
@@ -1604,6 +1714,13 @@ const struct operator_s* operator_link_create(const struct operator_s* op, unsig
 	return operator_generic_create2(N - 2, io_flags, D, dims, strs, CAST_UP(PTR_PASS(data)), link_apply, link_del, operator_link_get_graph);
 }
 
+const struct operator_s* operator_link_create_F(const struct operator_s* op, unsigned int o, unsigned int i)
+{
+	auto ret = operator_link_create(op, o, i);
+	operator_free(op);
+	return ret;
+}
+
 
 
 struct permute_data_s {
@@ -1704,6 +1821,27 @@ const struct operator_s* operator_permute(const struct operator_s* op, int N, co
 	return operator_generic_create2(N, io_flags, D, dims, strs, CAST_UP(PTR_PASS(data)), permute_fun, permute_del, operator_permute_get_graph);
 }
 
+const struct operator_s* operator_sort_args_F(const struct operator_s* op)
+{
+	int II = operator_nr_in_args(op);
+	int OO = operator_nr_out_args(op);
+
+	int perm[II + OO];
+
+	for(int j = 0, o = 0, i = OO; j < OO + II; j++) {
+
+		if (operator_get_io_flags(op)[j])
+			perm[o++] = j;
+		else
+			perm[i++] = j;
+	}
+
+	auto ret = operator_permute(op, OO + II, perm);
+	operator_free(op);
+
+	return ret;;
+}
+
 
 struct extract_data_s {
 
@@ -1802,7 +1940,9 @@ const struct operator_s* operator_extract_create2(const struct operator_s* op, i
 
 		assert((0 <= pos[i]) && (pos[i] < dimsa[i]));
 		assert(dims[a][i] + pos[i] <= dimsa[i]);
-		copy_needed = copy_needed || !((0 == strs[a][i]) || (strs[a][i] == strsa[i]));
+
+		if (!((0 == strs[a][i]) || (strs[a][i] == strsa[i])))
+			copy_needed = true;
 	}
 
 	assert(trivial_strides || !copy_needed);
@@ -2380,6 +2520,7 @@ list_t operator_get_list(const struct operator_s* op) {
 	auto data_perm = CAST_MAYBE(permute_data_s, op->data);
 	auto data_plus = CAST_MAYBE(operator_plus_s, op->data);
 	auto data_copy = CAST_MAYBE(copy_data_s, op->data);
+	auto data_attach = CAST_MAYBE(attach_data_s, op->data);
 
 	if (NULL != data_combi) {
 
@@ -2417,11 +2558,6 @@ list_t operator_get_list(const struct operator_s* op) {
 		return operator_get_list(data_perm->op);
 	}
 
-	if (NULL != data_perm) {
-
-		return operator_get_list(data_perm->op);
-	}
-
 	if (NULL != data_plus) {
 
 		list_t result = operator_get_list(data_plus->a);
@@ -2434,6 +2570,11 @@ list_t operator_get_list(const struct operator_s* op) {
 		return operator_get_list(data_copy->op);
 	}
 
+	if (NULL != data_attach) {
+
+		return operator_get_list(data_attach->op);
+	}
+
 	list_t result = list_create();
 	list_append(result, (void*)op);
 
