@@ -28,6 +28,8 @@
 #include "misc/misc.h"
 #include "misc/debug.h"
 
+#include "num/rand.h"
+
 #include "vecops.h"
 
 
@@ -79,6 +81,12 @@ static void clear(long N, float* vec)
 		vec[i] = 0.;
 }
 
+static void sadd(long N, float* vec, float src)
+{
+	for (long i = 0; i < N; i++)
+		vec[i] += src;
+}
+
 static double dot(long N, const float* vec1, const float* vec2)
 {
 	double res = 0.;
@@ -90,6 +98,16 @@ static double dot(long N, const float* vec1, const float* vec2)
 	return res;
 }
 
+static complex double zdot(long N, const complex float* vec1, const complex float* vec2)
+{
+	complex double res = 0.;
+
+	for (long i = 0; i < N; i++)
+		res += vec1[i] * conjf(vec2[i]);
+
+	return res;
+}
+
 /**
  * Compute l2 norm of vec
  *
@@ -194,6 +212,18 @@ static void add(long N, float* dst, const float* src1, const float* src2)
 		dst[i] = src1[i] + src2[i];
 }
 
+static void sadd_update(long N, float val, float* dst, const float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = src[i] + val;
+}
+
+static void zsadd(long N, complex float val, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = src[i] + val;
+}
+
 static void sub(long N, float* dst, const float* src1, const float* src2)
 {
 	for (long i = 0; i < N; i++)
@@ -213,15 +243,20 @@ static void vec_div(long N, float* dst, const float* src1, const float* src2)
 		dst[i] = (src2[i] == 0) ? 0.f : src1[i] / src2[i];
 }
 
+static void sdiv(long N, float* dst, float src1, const float* src2)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = (src2[i] == 0) ? 0.f : src1 / src2[i];
+}
+
 static void fmac(long N, float* dst, const float* src1, const float* src2)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] += src1[i] * src2[i];
 	//dst[i] = fmaf(src1[i], src2[i], dst[i]);
-		
 }
 
-static void fmac2(long N, double* dst, const float* src1, const float* src2)
+static void fmacD(long N, double* dst, const float* src1, const float* src2)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] += src1[i] * src2[i];
@@ -242,7 +277,7 @@ static void zmul(long N, complex float* dst, const complex float* src1, const co
 static void zdiv(long N, complex float* dst, const complex float* src1, const complex float* src2)
 {
 	for (long i = 0; i < N; i++)
-		dst[i] = (src2[i] == 0) ? 0.f : src1[i] / src2[i];
+		dst[i] = (src2[i] == 0.) ? 0. : (src1[i] / src2[i]);
 }
 
 static void zpow(long N, complex float* dst, const complex float* src1, const complex float* src2)
@@ -257,7 +292,7 @@ static void zfmac(long N, complex float* dst, const complex float* src1, const c
 		dst[i] += src1[i] * src2[i];
 }
 
-static void zfmac2(long N, complex double* dst, const complex float* src1, const complex float* src2)
+static void zfmacD(long N, complex double* dst, const complex float* src1, const complex float* src2)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] += src1[i] * src2[i];
@@ -275,12 +310,19 @@ static void zfmacc(long N, complex float* dst, const complex float* src1, const
 		dst[i] += src1[i] * conjf(src2[i]);
 }
 
-static void zfmacc2(long N, complex double* dst, const complex float* src1, const complex float* src2)
+static void zfmaccD(long N, complex double* dst, const complex float* src1, const complex float* src2)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] += src1[i] * conjf(src2[i]);
 }
 
+static void zfsq2(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] += crealf(src[i]) * crealf(src[i]) + cimagf(src[i]) * cimagf(src[i]);
+}
+
+
 static void zconj(long N, complex float* dst, const complex float* src)
 {
 	for (long i = 0; i < N; i++)
@@ -329,6 +371,24 @@ static void zexpj(long N, complex float* dst, const complex float* src)
 		dst[i] = cexpf(1.I * src[i]);
 }
 
+static void zlog(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = (src[i] == (complex float)0.) ? 0. : clogf(src[i]);
+}
+
+static void vec_exp(long N, float* dst, const float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = expf(src[i]);
+}
+
+static void vec_log(long N, float* dst, const float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = (src[i] == 0.) ? 0. : logf(src[i]);
+}
+
 static void zarg(long N, complex float* dst, const complex float* src)
 {
 	for (long i = 0; i < N; i++)
@@ -341,11 +401,46 @@ static void zabs(long N, complex float* dst, const complex float* src)
 		dst[i] = cabsf(src[i]);
 }
 
+static void zatanr(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = atan(crealf(src[i])) + 0.I;
+}
+
+static void zsin(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = csinf(src[i]);
+}
+
+static void zcos(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = ccosf(src[i]);
+}
+
+static void zsinh(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = csinhf(src[i]);
+}
+
+static void zcosh(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = ccoshf(src[i]);
+}
+
+static void zacosr(long N, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = acosf(crealf(src[i])) + 0.I;
+}
 
 static void zmax(long N, complex float* dst, const complex float* src1, const complex float* src2)
 {
 	for (long i = 0; i < N; i++)
-		dst[i] = (crealf(src1[i]) > crealf(src2[i])) ? src1[i] : src2[i];
+		dst[i] = MAX(crealf(src1[i]), crealf(src2[i]));
 }
 
 
@@ -369,12 +464,25 @@ static void min(long N, float* dst, const float* src1, const float* src2)
 }
 
 
+static void smin(long N, float val, float* dst, const float* src1)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = MIN(src1[i], val);
+}
+
+
 static void zsmax(long N, float val, complex float* dst, const complex float* src)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] = MAX(crealf(src[i]), val);
 }
 
+static void zsmin(long N, float val, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = MIN(crealf(src[i]), val);
+}
+
 
 static void vec_pow(long N, float* dst, const float* src1, const float* src2)
 {
@@ -393,14 +501,14 @@ static void vec_sqrt(long N, float* dst, const float* src)
 static void vec_zle(long N, complex float* dst, const complex float* src1, const complex float* src2)
 {
 	for (long i = 0; i < N; i++)
-		dst[i] = (crealf(src1[i]) <= crealf(src2[i]));
+		dst[i] = (crealf(src1[i]) <= crealf(src2[i])) ? 1. : 0.;
 }
 
 
 static void vec_le(long N, float* dst, const float* src1, const float* src2)
 {
 	for (long i = 0; i < N; i++)
-		dst[i] = (src1[i] <= src2[i]);
+		dst[i] = (src1[i] <= src2[i]) ? 1. : 0.;
 }
 
 /**
@@ -472,10 +580,10 @@ static void softthresh(long N, float lambda, float* d, const float* x)
 static float klargest_complex_partsort( unsigned int N,  unsigned int k, const complex float* ar)
 {
 	assert(k <= N);
-	
+
 	complex float* tmp =  (complex float*)xmalloc(N * sizeof(complex float));
 	copy(2 * N, (float*)tmp, (float*)ar);
-	
+
 	float thr = quickselect_complex(tmp, N, k);
 
 	xfree(tmp);
@@ -486,9 +594,9 @@ static float klargest_complex_partsort( unsigned int N,  unsigned int k, const c
 /**
  * Hard thesholding, y = HT(x, thr).
  * computes the thresholded vector, y = x * (abs(x) >= t(kmax))
- * 
+ *
  * @param N number of elements
- * @param k threshold parameter, index of kth largest element of sorted x 
+ * @param k threshold parameter, index of kth largest element of sorted x
  * @param d pointer to destination, y
  * @param x pointer to input
  */
@@ -496,7 +604,7 @@ static float klargest_complex_partsort( unsigned int N,  unsigned int k, const c
 static void zhardthresh(long N,  unsigned int k, complex float* d, const complex float* x)
 {
 	float thr = klargest_complex_partsort(N, k, x);
-   
+
 	for (long i = 0; i < N; i++) {
 
 		float norm = cabsf(x[i]);
@@ -587,6 +695,107 @@ static void zfftmod(long N, complex float* dst, const complex float* src, unsign
 }
 
 
+static void pdf_gauss(long N, float mu, float sig, float* dst, const float* src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = expf(- (src[i] - mu) * (src[i] - mu) / (2 * sig * sig)) / (sqrtf(2 * M_PI) * sig);
+}
+
+
+static void vec_real(long N, float* dst, const _Complex float* src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = crealf(src[i]);
+}
+
+static  void vec_imag(long N, float* dst, const _Complex float* src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = cimagf(src[i]);
+}
+
+static void vec_zcmpl_real(long N, _Complex float* dst, const float* src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = src[i];
+}
+
+static void vec_zcmpl_imag(long N, _Complex float* dst, const float* src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = src[i] * I;
+}
+
+static void vec_zcmpl(long N, _Complex float* dst, const float* real_src, const float* imag_src)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = real_src[i] + imag_src[i] * I;
+}
+
+static void vec_zfill(long N, _Complex float val, _Complex float* dst)
+{
+	for (int i = 0; i < N; i ++)
+		dst[i] = val;
+}
+
+
+static void xpay_bat(long Bi, long N, long Bo, const float* beta, float* a, const float* x)
+{
+	for (long bi = 0; bi < Bi; bi++) {
+
+		for (long bo = 0; bo < Bo; bo++) {
+
+			for (long i = 0; i < N; i++) {
+
+				long idx = 2 * bi + 2 * Bi * i + 2 * Bi * N * bo;
+				long idx_beta = bi + Bi * bo;
+
+				a[idx] = x[idx] + beta[idx_beta] * a[idx];
+				a[idx + 1] = x[idx + 1] + beta[idx_beta] * a[idx + 1];
+			}
+		}
+	}
+
+}
+
+static void dot_bat(long Bi, long N, long Bo, float* dst, const float* src1, const float* src2)
+{
+	for (long bi = 0; bi < Bi; bi++) {
+
+		for (long bo = 0; bo < Bo; bo++) {
+
+			double ret = 0.;
+
+			for (long i = 0; i < N; i++) {
+
+				long idx = 2 * bi + 2 * Bi * i + 2 * Bi * N * bo;
+				ret += src1[idx] * src2[idx] + src1[idx + 1] * src2[idx + 1];
+			}
+
+			dst[bi + Bi * bo] =(float)ret;
+		}
+	}
+}
+
+static void axpy_bat(long Bi, long N, long Bo, float* a, const float* alpha, const float* x)
+{
+	for (long bi = 0; bi < Bi; bi++) {
+
+		for (long bo = 0; bo < Bo; bo++) {
+
+			for (long i = 0; i < N; i++) {
+
+				long idx = 2 * bi + 2 * Bi * i + + 2 * Bi * N * bo;
+				long idx_alpha = bi + Bi * bo;
+
+				a[idx] += alpha[idx_alpha] * x[idx];
+				a[idx + 1] += alpha[idx_alpha] * x[idx + 1];
+			}
+		}
+	}
+
+}
+
 
 /*
  * If you add functions here, please also add to gpuops.c/gpukrnls.cu
@@ -600,12 +809,15 @@ const struct vec_ops cpu_ops = {
 	.zsum = zsum,
 	.zl1norm = zl1norm,
 
+	.zdot = zdot,
+
 	.add = add,
 	.sub = sub,
 	.mul = mul,
 	.div = vec_div,
 	.fmac = fmac,
-	.fmac2 = fmac2,
+	.fmacD = fmacD,
+	.sadd = sadd_update,
 
 	.smul = smul,
 
@@ -620,21 +832,33 @@ const struct vec_ops cpu_ops = {
 	.zmul = zmul,
 	.zdiv = zdiv,
 	.zfmac = zfmac,
-	.zfmac2 = zfmac2,
+	.zfmacD = zfmacD,
 	.zmulc = zmulc,
 	.zfmacc = zfmacc,
-	.zfmacc2 = zfmacc2,
+	.zfmaccD = zfmaccD,
+	.zfsq2 = zfsq2,
 
 	.zsmax = zsmax,
+	.zsmin = zsmin,
 	.zsmul = zsmul,
+	.zsadd = zsadd,
 
 	.zpow = zpow,
 	.zphsr = zphsr,
 	.zconj = zconj,
 	.zexpj = zexpj,
 	.zexp = zexp,
+	.zlog = zlog,
 	.zarg = zarg,
 	.zabs = zabs,
+	.zatanr = zatanr,
+
+	.zsin = zsin,
+	.zcos = zcos,
+	.zacosr = zacosr,
+
+	.zsinh = zsinh,
+	.zcosh = zcosh,
 
 	.zcmp = zcmp,
 	.zdiv_reg = zdiv_reg,
@@ -652,6 +876,19 @@ const struct vec_ops cpu_ops = {
 	.softthresh_half = softthresh_half,
 	.zhardthresh = zhardthresh,
 	.zhardthresh_mask = zhardthresh_mask,
+
+	.exp = vec_exp,
+	.log = vec_log,
+
+	.pdf_gauss=pdf_gauss,
+
+	.real = vec_real,
+	.imag = vec_imag,
+	.zcmpl_real = vec_zcmpl_real,
+	.zcmpl_imag = vec_zcmpl_imag,
+	.zcmpl = vec_zcmpl,
+
+	.zfill = vec_zfill,
 };
 
 
@@ -675,7 +912,27 @@ struct vec_iter_s {
 	void (*xpay)(long N, float alpha, float* a, const float* x);
 	void (*axpy)(long N, float* a, float alpha, const float* x);
 	void (*axpbz)(long N, float* out, const float a, const float* x, const float b, const float* z);
+	void (*fmac)(long N, float* a, const float* x, const float* y);
+
+	void (*mul)(long N, float* a, const float* x, const float* y);
+	void (*div)(long N, float* a, const float* x, const float* y);
+	void (*sqrt)(long N, float* a, const float* x);
+
+	void (*smax)(long N, float alpha, float* a, const float* x);
+	void (*smin)(long N, float alpha, float* a, const float* x);
+	void (*sadd)(long N, float* x, float y);
+	void (*sdiv)(long N, float* a, float x, const float* y);
+	void (*le)(long N, float* a, const float* x, const float* y);
+
 	void (*zmul)(long N, complex float* dst, const complex float* src1, const complex float* src2);
+	void (*zsmax)(long N, float val, complex float* dst, const complex float* src1);
+
+	void (*rand)(long N, float* dst);
+
+	void (*xpay_bat)(long Bi, long N, long Bo, const float* beta, float* a, const float* x);
+	void (*dot_bat)(long Bi, long N, long Bo, float* dst, const float* src1, const float* src2);
+	void (*axpy_bat)(long Bi, long N, long Bo, float* a, const float* alpha, const float* x);
+
 };
 
 
@@ -696,6 +953,18 @@ const struct vec_iter_s cpu_iter_ops = {
 	.sub = sub,
 	.swap = swap,
 	.zmul = zmul,
+	.fmac = fmac,
+	.sqrt = vec_sqrt,
+	.sdiv = sdiv,
+	.sadd = sadd,
+	.mul = mul,
+	.div = vec_div,
+	.smax = smax,
+	.smin = smin,
+	.rand = gaussian_rand_vec,
+	.le = vec_le,
+
+	.xpay_bat = xpay_bat,
+	.dot_bat = dot_bat,
+	.axpy_bat = axpy_bat,
 };
-
-
