@@ -23,7 +23,8 @@
 
 #include "linops/someops.h"
 #include "linops/linop.h"
-#include "linops/waveop.h"
+
+#include "wavelet/wavthresh.h"
 
 #include "iter/iter.h"
 #include "iter/prox.h"
@@ -42,18 +43,27 @@
 
 
 
-static const char usage_str[] = "<kspace> <sensitivities> <output>";
 static const char help_str[] = "Perform POCSENSE reconstruction.";
 
 	
 
 int main_pocsense(int argc, char* argv[argc])
 {
+	const char* ksp_file = NULL;
+	const char* sens_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &ksp_file, "kspace"),
+		ARG_INFILE(true, &sens_file, "sensitivities"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
 	float alpha = 0.;
 	int maxiter = 50;
 	bool l1wav = false;
 	float lambda = -1.;
-	bool use_gpu = false;
 	bool use_admm = false;
 	float admm_rho = -1.;
 	int l1type = 2;
@@ -63,12 +73,12 @@ int main_pocsense(int argc, char* argv[argc])
 		OPT_INT('i', &maxiter, "iter", "max. number of iterations"),
 		OPT_FLOAT('r', &alpha, "alpha", "regularization parameter"),
 		OPT_INT('l', &l1type, "1/-l2", "toggle l1-wavelet or l2 regularization"),
-		OPT_SET('g', &use_gpu, "()"),
+		OPT_SET('g', &bart_use_gpu, "()"),
 		OPT_FLOAT('o', &lambda, "", "()"),
 		OPT_FLOAT('m', &admm_rho, "", "()"),
 	};
 
-	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	if (1 == l1type)
 		l1wav = true;
@@ -79,13 +89,13 @@ int main_pocsense(int argc, char* argv[argc])
 		error("Unknown regularization type.");
 
 	
-	unsigned int N = DIMS;
+	int N = DIMS;
 
 	long dims[N];
 	long ksp_dims[N];
 
-	complex float* kspace_data = load_cfl(argv[1], N, ksp_dims);
-	complex float* sens_maps = load_cfl(argv[2], N, dims);
+	complex float* kspace_data = load_cfl(ksp_file, N, ksp_dims);
+	complex float* sens_maps = load_cfl(sens_file, N, dims);
 
 
 	for (int i = 0; i < 4; i++)	// sizes2[4] may be > 1
@@ -94,7 +104,7 @@ int main_pocsense(int argc, char* argv[argc])
 
 	assert(1 == ksp_dims[MAPS_DIM]);
 
-	(use_gpu ? num_init_gpu : num_init)();
+	num_init_gpu_support();
 
 
 	
@@ -106,7 +116,7 @@ int main_pocsense(int argc, char* argv[argc])
 	// -----------------------------------------------------------
 	// memory allocation
 	
-	complex float* result = create_cfl(argv[3], N, ksp_dims);
+	complex float* result = create_cfl(out_file, N, ksp_dims);
 	complex float* pattern = md_alloc(N, dims1, CFL_SIZE);
 
 
@@ -123,20 +133,24 @@ int main_pocsense(int argc, char* argv[argc])
 	// l1-norm threshold operator
 	
 	const struct operator_p_s* thresh_op = NULL;
-	const struct linop_s* wave_op = NULL;
 
 	if (l1wav) {
 
 		long minsize[DIMS] = { [0 ... DIMS - 1] = 1 };
-		minsize[0] = MIN(ksp_dims[0], 16);
-		minsize[1] = MIN(ksp_dims[1], 16);
-		minsize[2] = MIN(ksp_dims[2], 16);
 
-		long strs[DIMS];
-		md_calc_strides(DIMS, strs, ksp_dims, CFL_SIZE);
+		unsigned long flags = 0;
 
-		wave_op = linop_wavelet_create(DIMS, FFT_FLAGS, ksp_dims, strs, minsize, false);
-		thresh_op = prox_unithresh_create(DIMS, wave_op, alpha, COIL_FLAG);
+		for (int i = 0; i < DIMS; i++) {
+
+			if ((1 < ksp_dims[i]) && MD_IS_SET(FFT_FLAGS, i)) {
+
+				flags = MD_SET(flags, i);
+				minsize[i] = MIN(ksp_dims[i], 16);
+			}
+		}
+
+		bool randshift = false;
+		thresh_op = prox_wavelet_thresh_create(DIMS, ksp_dims, flags, COIL_FLAG, WAVELET_DAU2, minsize, alpha, randshift);
 	}
 #if 0
 	else {
@@ -181,7 +195,7 @@ int main_pocsense(int argc, char* argv[argc])
 	
 	fftmod(N, ksp_dims, FFT_FLAGS, kspace_data, kspace_data);
 
-	if (use_gpu)
+	if (bart_use_gpu)
 #ifdef USE_CUDA
 		pocs_recon_gpu2(italgo, iconf, (const struct linop_s**)ops2, dims, thresh_op, alpha, lambda, result, sens_maps, pattern, kspace_data);
 #else
@@ -204,9 +218,6 @@ int main_pocsense(int argc, char* argv[argc])
 	if (NULL != thresh_op)
 		operator_p_free(thresh_op);
 
-	if (NULL != wave_op)
-		linop_free(wave_op);
-
 	unmap_cfl(N, ksp_dims, result);
 	unmap_cfl(N, ksp_dims, kspace_data);
 	unmap_cfl(N, dims, sens_maps);
