@@ -17,40 +17,45 @@
 
 #include "misc/mmio.h"
 #include "misc/misc.h"
+#include "misc/opts.h"
 
 #ifndef DIMS
 #define DIMS 16
 #endif
 
-static const char usage_str[] = "scale <input1> <input2> <output>";
-static const char help_str[] = "Multiply input1 with scale factor and add input2.\n";
+static const char help_str[] = "Multiply input1 with scale factor and add input2.";
 
 
-int main_saxpy(int argc, char* argv[])
+int main_saxpy(int argc, char* argv[argc])
 {
-	mini_cmdline(&argc, argv, 4, usage_str, help_str);
+	complex float scale = 0;
+	const char* in1_file = NULL;
+	const char* in2_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_CFL(true, &scale, "scale"),
+		ARG_INFILE(true, &in1_file, "input1"),
+		ARG_INFILE(true, &in2_file, "input2"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+	const struct opt_s opts[] = {};
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
-	complex float scale;
-
-	if (0 != parse_cfl(&scale, argv[1])) {
-		
-		fprintf(stderr, "ERROR: %s is not a complex number.\n", argv[1]);
-		exit(1);
-	}
-
 	const int N = DIMS;
 	long dims1[N];
 	long dims2[N];
 
-	complex float* data1 = load_cfl(argv[2], N, dims1);
-	complex float* data2 = load_cfl(argv[3], N, dims2);
+	complex float* data1 = load_cfl(in1_file, N, dims1);
+	complex float* data2 = load_cfl(in2_file, N, dims2);
 
 	for (int i = 0; i < N; i++)
 		assert(dims1[i] == dims2[i]);
 
-	complex float* out = create_cfl(argv[4], N, dims2);
+	complex float* out = create_cfl(out_file, N, dims2);
 
 	#pragma omp parallel for
 	for (long i = 0; i < md_calc_size(N, dims1); i++)
@@ -59,7 +64,8 @@ int main_saxpy(int argc, char* argv[])
 	unmap_cfl(N, dims1, data1);
 	unmap_cfl(N, dims2, data2);
 	unmap_cfl(N, dims2, out);
-	exit(0);
+
+	return 0;
 }
 
 
