@@ -28,6 +28,7 @@ struct sense_conf {
 	float gamma;	// ..
 	float cclambda;
 	bool bpsense;
+	bool precond;
 };
 
 
@@ -36,9 +37,9 @@ extern const struct sense_conf sense_defaults;
 struct operator_s;
 struct operator_p_s;
 
-extern const struct operator_p_s* sense_recon_create(const struct sense_conf* conf, const long dims[DIMS],
+extern const struct operator_p_s* sense_recon_create(const struct sense_conf* conf,
 		  const struct linop_s* sense_op,
-		  const long pat_dims[DIMS], const complex float* pattern,
+		  const long pat_dims[DIMS],
 		  italgo_fun2_t italgo, iter_conf* iconf,
 		  const complex float* init,
 		  unsigned int num_funs,
