@@ -31,7 +31,6 @@
 #include "linops/someops.h"
 #include "linops/grad.h"
 #include "linops/sum.h"
-#include "linops/sampling.h"
 
 #include "iter/iter.h"
 #include "iter/iter2.h"
@@ -102,8 +101,8 @@ struct reg_s {
 
 	enum { L1WAV, TV, LLR, MLR, IMAGL1, IMAGL2, L1IMG, L2IMG } xform;
 
-	unsigned int xflags;
-	unsigned int jflags;
+	unsigned long xflags;
+	unsigned long jflags;
 
 	float lambda;
 };
@@ -141,13 +140,13 @@ static bool opt_reg(void* ptr, char c, const char* optarg)
 		if (strcmp(rt, "W") == 0) {
 
 			regs[r].xform = L1WAV;
-			int ret = sscanf(optarg, "%*[^:]:%d:%d:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%lu:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
 			assert(3 == ret);
 		}
 		else if (strcmp(rt, "L") == 0) {
 
 			regs[r].xform = LLR;
-			int ret = sscanf(optarg, "%*[^:]:%d:%d:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%lu:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
 			assert(3 == ret);
 		}
 		else if (strcmp(rt, "M") == 0) {
@@ -158,20 +157,20 @@ static bool opt_reg(void* ptr, char c, const char* optarg)
 			regs[r].lambda = regs[0].lambda;
 
 			regs[0].xform = MLR;
-			int ret = sscanf(optarg, "%*[^:]:%d:%d:%f", &regs[0].xflags, &regs[0].jflags, &regs[0].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%lu:%f", &regs[0].xflags, &regs[0].jflags, &regs[0].lambda);
 			assert(3 == ret);
 		}
 		else if (strcmp(rt, "T") == 0) {
 
 			regs[r].xform = TV;
-			int ret = sscanf(optarg, "%*[^:]:%d:%d:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%lu:%f", &regs[r].xflags, &regs[r].jflags, &regs[r].lambda);
 			assert(3 == ret);
 			p->algo = ADMM;
 		}
 		else if (strcmp(rt, "R1") == 0) {
 
 			regs[r].xform = IMAGL1;
-			int ret = sscanf(optarg, "%*[^:]:%d:%f", &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%f", &regs[r].jflags, &regs[r].lambda);
 			assert(2 == ret);
 			regs[r].xflags = 0u;
 			p->algo = ADMM;
@@ -179,7 +178,7 @@ static bool opt_reg(void* ptr, char c, const char* optarg)
 		else if (strcmp(rt, "R2") == 0) {
 
 			regs[r].xform = IMAGL2;
-			int ret = sscanf(optarg, "%*[^:]:%d:%f", &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%f", &regs[r].jflags, &regs[r].lambda);
 			assert(2 == ret);
 			regs[r].xflags = 0u;
 			p->algo = ADMM;
@@ -187,7 +186,7 @@ static bool opt_reg(void* ptr, char c, const char* optarg)
 		else if (strcmp(rt, "I") == 0) {
 
 			regs[r].xform = L1IMG;
-			int ret = sscanf(optarg, "%*[^:]:%d:%f", &regs[r].jflags, &regs[r].lambda);
+			int ret = sscanf(optarg, "%*[^:]:%lu:%f", &regs[r].jflags, &regs[r].lambda);
 			assert(2 == ret);
 			regs[r].xflags = 0u;
 		}
@@ -257,9 +256,6 @@ int main_sqpics(int argc, char* argv[argc])
 
 	// Initialize default parameters
 
-
-	bool use_gpu = false;
-
 	bool randshift = true;
 	unsigned int maxiter = 30;
 	float step = -1.;
@@ -308,7 +304,7 @@ int main_sqpics(int argc, char* argv[argc])
 		OPT_UINT('i', &maxiter, "iter", "max. number of iterations"),
 		OPT_INFILE('t', &traj_file, "file", "k-space trajectory"),
 		OPT_CLEAR('n', &randshift, "disable random wavelet cycle spinning"),
-		OPT_SET('g', &use_gpu, "use GPU"),
+		OPT_SET('g', &bart_use_gpu, "use GPU"),
 		OPT_INFILE('p', &pat_file, "file", "pattern or weights"),
 		OPT_SELECT('I', enum algo_t, &ropts.algo, IST, "(select IST)"),
 		OPT_UINT('b', &llr_blk, "blk", "Lowrank block size"),
@@ -369,11 +365,11 @@ int main_sqpics(int argc, char* argv[argc])
 	assert(1 == ksp_dims[MAPS_DIM]);
 
 
-	(use_gpu ? num_init_gpu : num_init)();
+	num_init_gpu_support();
 
 	// print options
 
-	if (use_gpu)
+	if (bart_use_gpu)
 		debug_printf(DP_INFO, "GPU reconstruction\n");
 
 	if (map_dims[MAPS_DIM] > 1)
