@@ -16,15 +16,8 @@
  * 3:492-526 (2010)
  */
 
-#include <stdlib.h>
-#include <assert.h>
-#include <complex.h>
-#include <assert.h>
-#include <stdbool.h>
-
 #include "num/multind.h"
 #include "num/flpmath.h"
-#include "num/iovec.h"
 #include "num/ops_p.h"
 #include "num/init.h"
 
@@ -52,14 +45,14 @@ static const char help_str[] = "Perform total generalized variation denoising al
 int main_tgv(int argc, char* argv[argc])
 {
 	float lambda = 0.;
-	int flags = -1;
+	unsigned long flags = 0;
 	const char* in_file = NULL;
 	const char* out_file = NULL;
 
 	struct arg_s args[] = {
 
 		ARG_FLOAT(true, &lambda, "lambda"),
-		ARG_INT(true, &flags, "flags"),
+		ARG_ULONG(true, &flags, "flags"),
 		ARG_INFILE(true, &in_file, "input"),
 		ARG_OUTFILE(true, &out_file, "output"),
 	};
