@@ -39,7 +39,7 @@
 
 
 // FIXME: consider moving this to a more accessible location?
-static void wthresh(unsigned int D, const long dims[D], float lambda, unsigned int flags, complex float* out, const complex float* in)
+static void wthresh(int D, const long dims[D], float lambda, unsigned long flags, complex float* out, const complex float* in)
 {
 	long minsize[D];
 	md_singleton_dims(D, minsize);
@@ -47,16 +47,16 @@ static void wthresh(unsigned int D, const long dims[D], float lambda, unsigned i
 	long course_scale[3] = MD_INIT_ARRAY(3, 16);
 	md_copy_dims(3, minsize, course_scale);
 
-	unsigned int wflags = 7; // FIXME
+	unsigned long wflags = 7; // FIXME
 
-	for (unsigned int i = 0; i < 3; i++)
+	for (int i = 0; i < 3; i++)
 		if (dims[i] < minsize[i])
 			wflags = MD_CLEAR(wflags, i);
 
 	long strs[D];
 	md_calc_strides(D, strs, dims, CFL_SIZE);
 
-	const struct linop_s* w = linop_wavelet_create(D, wflags, dims, strs, minsize, false);
+	const struct linop_s* w = linop_wavelet_create(D, wflags, dims, strs, WAVELET_DAU2, minsize, false);
 	const struct operator_p_s* p = prox_unithresh_create(D, w, lambda, flags);
 
 	operator_p_apply(p, 1., D, dims, out, D, dims, in);
@@ -65,7 +65,7 @@ static void wthresh(unsigned int D, const long dims[D], float lambda, unsigned i
 }
 
 
-static void lrthresh(unsigned int D, const long dims[D], int llrblk, float lambda, unsigned int flags, complex float* out, const complex float* in)
+static void lrthresh(int D, const long dims[D], int llrblk, float lambda, unsigned long flags, complex float* out, const complex float* in)
 {
 	long blkdims[MAX_LEV][D];
 
@@ -80,10 +80,10 @@ static void lrthresh(unsigned int D, const long dims[D], int llrblk, float lambd
 }
 
 
-static void dfthresh(unsigned int D, const long dims[D], float lambda, complex float* out, const complex float* in)
+static void dfthresh(int D, const long dims[D], float lambda, complex float* out, const complex float* in)
 {
-	long minsize[3];
-	md_singleton_dims(3, minsize);
+	long minsize[D];
+	md_singleton_dims(D, minsize);
 
 	long coarse_scale[3] = MD_INIT_ARRAY(3, 16);
 	md_min_dims(3, ~0u, minsize, dims, coarse_scale);
@@ -102,7 +102,7 @@ static void dfthresh(unsigned int D, const long dims[D], float lambda, complex f
 	operator_p_free(p);
 }
 
-static void hard_thresh(unsigned int D, const long dims[D], float lambda, complex float* out, const complex float* in)
+static void hard_thresh(int D, const long dims[D], float lambda, complex float* out, const complex float* in)
 {
 	long size = md_calc_size(DIMS, dims);
 
@@ -111,7 +111,7 @@ static void hard_thresh(unsigned int D, const long dims[D], float lambda, comple
 		out[i] = (cabsf(in[i]) > lambda) ? in[i] : 0.;
 }
 
-static void binary_thresh(unsigned int D, const long dims[D], float lambda, complex float* out, const complex float* in)
+static void binary_thresh(int D, const long dims[D], float lambda, complex float* out, const complex float* in)
 {
 	long size = md_calc_size(DIMS, dims);
 
@@ -120,18 +120,37 @@ static void binary_thresh(unsigned int D, const long dims[D], float lambda, comp
 		out[i] = (cabsf(in[i]) > lambda) ? 1. : 0.;
 }
 
+static void binary_thresh_max(unsigned int D, const long dims[D], float lambda, complex float* out, const complex float* in)
+{
+	long size = md_calc_size(DIMS, dims);
+
+#pragma omp parallel for
+	for (long i = 0; i < size; i++)
+		out[i] = (cabsf(in[i]) < lambda) ? 1. : 0.;
+}
+
 
 
-static const char usage_str[] = "lambda <input> <output>";
 static const char help_str[] = "Perform (soft) thresholding with parameter lambda.";
 
 
 
 int main_threshold(int argc, char* argv[argc])
 {
-	unsigned int flags = 0;
+	float lambda = 0.;
+	const char* in_file = NULL;
+	const char* out_file = NULL;
+
+	struct arg_s args[] = {
+
+		ARG_FLOAT(true, &lambda, "lambda"),
+		ARG_INFILE(true, &in_file, "input"),
+		ARG_OUTFILE(true, &out_file, "output"),
+	};
+
+	unsigned long flags = 0;
         
-	enum th_type { NONE, WAV, LLR, DFW, MPDFW, HARD, BINARY } th_type = NONE;
+	enum th_type { NONE, WAV, LLR, DFW, MPDFW, HARD, BINARY, BINARY_MAX } th_type = NONE;
 	int llrblk = 8;
 
 
@@ -141,52 +160,54 @@ int main_threshold(int argc, char* argv[argc])
 		OPT_SELECT('W', enum th_type, &th_type, WAV, "daubechies wavelet soft-thresholding"),
 		OPT_SELECT('L', enum th_type, &th_type, LLR, "locally low rank soft-thresholding"),
 		OPT_SELECT('D', enum th_type, &th_type, DFW, "divergence-free wavelet soft-thresholding"),
-		OPT_SELECT('B', enum th_type, &th_type, BINARY, "thresholding with binary output"),
-		OPT_UINT('j', &flags, "bitmask", "joint soft-thresholding"),
+		OPT_SELECT('B', enum th_type, &th_type, BINARY, "thresholding with binary output where (val>lambda)"),
+		OPT_SELECT('M', enum th_type, &th_type, BINARY_MAX, "thresholding with binary output where (val<lambda)"),
+		OPT_ULONG('j', &flags, "bitmask", "joint soft-thresholding"),
 		OPT_INT('b', &llrblk, "blocksize", "locally low rank block size"),
 	};
 
-	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
 
 	num_init();
 
 	const int N = DIMS;
 	long dims[N];
-	complex float* idata = load_cfl(argv[2], N, dims);
-	complex float* odata = create_cfl(argv[3], N, dims);
-
-	float lambda = atof(argv[1]);
+	complex float* idata = load_cfl(in_file, N, dims);
+	complex float* odata = create_cfl(out_file, N, dims);
 
 	switch (th_type) {
 
-		case WAV:
-			wthresh(N, dims, lambda, flags, odata, idata);
-			break;
+	case WAV:
+		wthresh(N, dims, lambda, flags, odata, idata);
+		break;
 
-		case LLR:
-			lrthresh(N, dims, llrblk, lambda, flags, odata, idata);
-			break;
-                        
-		case DFW:
-			dfthresh(N, dims, lambda, odata, idata);
-			break;
+	case LLR:
+		lrthresh(N, dims, llrblk, lambda, flags, odata, idata);
+		break;
+ 
+	case DFW:
+		dfthresh(N, dims, lambda, odata, idata);
+		break;
 
-		case HARD:
-			hard_thresh(N, dims, lambda, odata, idata);
-			break;
+	case HARD:
+		hard_thresh(N, dims, lambda, odata, idata);
+		break;
 
-		case BINARY:
-			binary_thresh(N, dims, lambda, odata, idata);
-			break;
+	case BINARY:
+		binary_thresh(N, dims, lambda, odata, idata);
+		break;
 
-		default:
-			md_zsoftthresh(N, dims, lambda, flags, odata, idata);
+	case BINARY_MAX:
+		binary_thresh_max(N, dims, lambda, odata, idata);
+		break;
 
+	default:
+		md_zsoftthresh(N, dims, lambda, flags, odata, idata);
 	}
 
-
 	unmap_cfl(N, dims, idata);
 	unmap_cfl(N, dims, odata);
+
 	return 0;
 }
 
