@@ -56,7 +56,10 @@
 #include "wavelet/wavthresh.h"
 #include "lowrank/lrthresh.h"
 
-static const char usage_str[] = "<maps> <wave> <phi> <reorder> <table> <output>";
+#include "grecon/optreg.h"
+#include "grecon/italgo.h"
+
+
 static const char help_str[]  = 
 	"Perform a wave-shuffling reconstruction.\n\n"
 	"Conventions:\n"
@@ -97,13 +100,18 @@ static void print_opdims(const struct linop_s* op)
 {
 	const struct iovec_s* domain   = linop_domain(op);
 	const struct iovec_s* codomain = linop_codomain(op);
+
 	debug_printf(DP_INFO, "\tDomain:   [");
+
 	for (long k = 0; k < domain->N; k ++)
 		debug_printf(DP_INFO, "%6ld", domain->dims[k]);
+
 	debug_printf(DP_INFO, "]\n");
 	debug_printf(DP_INFO, "\tCodomain: [");
+
 	for (long k = 0; k < codomain->N; k ++)
 		debug_printf(DP_INFO, "%6ld", codomain->dims[k]);
+
 	debug_printf(DP_INFO, "]\n");
 }
 
@@ -121,9 +129,11 @@ static void construct_mask(
 	long t = 0;
 
 	for (int i = 0; i < n; i++) {
+
 		y = lround(creal(reorder[i]));
 		z = lround(creal(reorder[i + n]));
 		t = lround(creal(reorder[i + 2 * n]));
+
 		mask[(y + z * sy) + t * sy * sz] = 1;
 	}
 }
@@ -175,10 +185,14 @@ static void kern_apply(const linop_data_t* _data, complex float* dst, const comp
 	perm_dims[3] = tk;
 	perm_dims[4] = sy;
 	perm_dims[5] = sz;
+
 	complex float* perm = md_alloc_sameplace(DIMS, perm_dims, CFL_SIZE, src);
-	unsigned int permute_order[DIMS] = {0, 3, 5, 6, 1, 2, 4, 7};
-	for (unsigned int i = 8; i < DIMS; i++)
+
+	int permute_order[DIMS] = { 0, 3, 5, 6, 1, 2, 4, 7 };
+
+	for (int i = 8; i < DIMS; i++)
 		permute_order[i] = i;
+
 	md_permute(DIMS, permute_order, perm_dims, perm, input_dims, src, CFL_SIZE);
 
 	long vec_dims[]     = {wx, nc, tf,  1};
@@ -246,15 +260,15 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 	long num_threads = 1;
 #endif
 
-	long vec_dims[]     = {wx, nc, tf,  1};
-	long phi_mat_dims[] = { 1,  1, tf, tk};
-	long phi_out_dims[] = {wx, nc,  1, tk};
-	long fmac_dims[]    = {wx, nc, tf, tk};
-	long line_dims[]    = {wx, nc,  1,  1};
-	long vthrd_dims[]   = {wx, nc, tf,  1, num_threads};
+	long vec_dims[]     = { wx, nc, tf,  1 };
+	long phi_mat_dims[] = { 1,  1, tf, tk };
+	long phi_out_dims[] = { wx, nc,  1, tk };
+	long fmac_dims[]    = { wx, nc, tf, tk };
+	long line_dims[]    = { wx, nc,  1,  1 };
+	long vthrd_dims[]   = { wx, nc, tf,  1, num_threads };
 
 	complex float* vec = md_alloc_sameplace(5, vthrd_dims, CFL_SIZE, dst);
-	md_clear(DIMS, vthrd_dims, vec, CFL_SIZE);
+	md_clear(5, vthrd_dims, vec, CFL_SIZE);
 
 	long vec_str[4];
 	md_calc_strides(4, vec_str, vec_dims, CFL_SIZE);
@@ -280,12 +294,16 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 		int t = -1;
 
 		if (0 == flags[k]) {
+
 			md_clear(4, vec_dims, vec + (wx * nc * tf * tid), CFL_SIZE);
 
 			for (int i = k; i < n; i ++) {
+
 				if ((y == lround(creal(data->reorder[i]))) && (z == lround(creal(data->reorder[i + n])))) {
+
 					flags[i] = 1;
 					t = lround(creal(data->reorder[i + 2 * n]));
+
 					md_copy(4, line_dims, (vec + (wx * nc * tf * tid) + t * wx * nc), (src + i * wx * nc), CFL_SIZE);
 				}
 			}
@@ -300,9 +318,12 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 	out_dims[2] = sz;
 	out_dims[3] = nc;
 	out_dims[6] = tk;
-	unsigned int permute_order[DIMS] = {0, 4, 5, 1, 6, 2, 3, 7};
-	for (unsigned int i = 8; i < DIMS; i++)
+
+	int permute_order[DIMS] = { 0, 4, 5, 1, 6, 2, 3, 7 };
+
+	for (int i = 8; i < DIMS; i++)
 		permute_order[i] = i;
+
 	md_permute(DIMS, permute_order, out_dims, dst, perm_dims, perm, CFL_SIZE);
 
 	md_free(vec);
@@ -441,11 +462,11 @@ static const struct linop_s* linop_kern_create(bool gpu_flag,
 	output_dims[1] = _table_dims[1];
 	output_dims[2] = _reorder_dims[0];
 
-	const struct linop_s* K = linop_create(DIMS, output_dims, DIMS, input_dims, CAST_UP(PTR_PASS(data)), kern_apply, kern_adjoint, kern_normal, NULL, kern_free);
-	return K;
+	return linop_create(DIMS, output_dims, DIMS, input_dims, CAST_UP(PTR_PASS(data)), kern_apply, kern_adjoint, kern_normal, NULL, kern_free);
 }
 
 struct multc_s {
+
 	INTERFACE(linop_data_t);
 
 	unsigned int nc;
@@ -491,6 +512,7 @@ static void multc_apply(const linop_data_t* _data, complex float* dst, const com
 	long single_map_dims[] = { [0 ... DIMS - 1] = 1 };
 	md_copy_dims(DIMS, single_map_dims, map_dims);
 	single_map_dims[COIL_DIM] = 1;
+
 	complex float* single_map = md_alloc_sameplace(DIMS, single_map_dims, CFL_SIZE, src);
 
 	complex float* buffer = md_alloc_sameplace(DIMS, sc_inp_dims, CFL_SIZE, src);
@@ -515,19 +537,26 @@ static void multc_apply(const linop_data_t* _data, complex float* dst, const com
 	md_calc_strides(DIMS, strides_sc_inp, sc_inp_dims, CFL_SIZE);
 
 	for (long k = 0; k < data->nc; k++) {
+
 		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
 		md_clear(DIMS, sc_inp_dims, buffer, CFL_SIZE);
+
 		pos[COIL_DIM] = k;
 		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
 		pos[COIL_DIM] = 0;
+
 		md_zfmac2(DIMS, zfmac_dims, strides_sc_inp, buffer, strides_src, src, strides_single_map, single_map);
+
 		operator_apply(fwd, DIMS, sc_out_dims, tbl  + (wx * n * k), DIMS, sc_inp_dims, buffer);
 	}
 
 	md_clear(DIMS, dst_dims, dst, CFL_SIZE);
-	unsigned int permute_order[DIMS] = {0, 2, 1};
-	for (unsigned int i = 3; i < DIMS; i++)
+
+	int permute_order[DIMS] = { 0, 2, 1 };
+
+	for (int i = 3; i < DIMS; i++)
 		permute_order[i] = i;
+
 	md_permute(DIMS, permute_order, dst_dims, dst, tbl_dims, tbl, CFL_SIZE);
 
 	md_free(single_map);
@@ -592,17 +621,23 @@ static void multc_adjoint(const linop_data_t* _data, complex float* dst, const c
 	md_clear(DIMS, dst_dims, dst, CFL_SIZE);
 	
 	for (long k = 0; k < data->nc; k++) {
+
+
 		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
 		md_clear(DIMS, sc_out_dims, buffer1, CFL_SIZE);
 		md_clear(DIMS, dst_dims, buffer2, CFL_SIZE);
 		md_clear(DIMS, tbl_dims, tbl, CFL_SIZE);
+
 		pos[1] = k;
 		md_slice(DIMS, 2, pos, src_dims, tbl, src, CFL_SIZE);
 		pos[1] = 0;
+
 		operator_apply(adj, DIMS, sc_out_dims, buffer1, DIMS, tbl_dims, tbl);
+
 		pos[COIL_DIM] = k;
 		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
 		pos[COIL_DIM] = 0;
+
 		md_zfmacc2(DIMS, dst_dims, strides_dst, buffer2, strides_sc_out, buffer1, strides_single_map, single_map);
 		md_zadd(DIMS, dst_dims, dst, dst, buffer2);
 	}
@@ -657,16 +692,22 @@ static void multc_normal(const linop_data_t* _data, complex float* dst, const co
 	md_calc_strides(DIMS, strides, dims, CFL_SIZE);
 
 	md_clear(DIMS, dims, dst, CFL_SIZE);
+
 	for (long k = 0; k < data->nc; k++) {
+
 		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
 		md_clear(DIMS, sc_dims, buffer1, CFL_SIZE);
 		md_clear(DIMS, sc_dims, buffer2, CFL_SIZE);
 		md_clear(DIMS, dims, buffer3, CFL_SIZE);
+
 		pos[COIL_DIM] = k;
 		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
 		pos[COIL_DIM] = 0;
+
 		md_zfmac2(DIMS, dims, strides_sc, buffer1, strides, src, strides_single_map, single_map);
+
 		operator_apply(nrm, DIMS, sc_dims, buffer2, DIMS, sc_dims, buffer1);
+
 		md_zfmacc2(DIMS, dims, strides, buffer3, strides_sc, buffer2, strides_single_map, single_map);
 		md_zadd(DIMS, dims, dst, dst, buffer3);
 	}
@@ -831,8 +872,9 @@ static void construct_kernel(
 		}
 	}
 
-	unsigned int permute_order[DIMS] = {4, 1, 2, 5, 6, 7, 3, 0};
-	for (unsigned int i = 8; i < DIMS; i++)
+	int permute_order[DIMS] = { 4, 1, 2, 5, 6, 7, 3, 0 };
+
+	for (int i = 8; i < DIMS; i++)
 		permute_order[i] = i;
 
 	md_permute(DIMS, permute_order, kern_dims, kern, out_dims, out, CFL_SIZE);
@@ -875,76 +917,82 @@ static void fftmod_apply(long sy, long sz,
 	}
 }
 
-enum algo_t { CG, IST, FISTA };
-
-int main_wshfl(int argc, char* argv[])
+int main_wshfl(int argc, char* argv[argc])
 {
 	double start_time = timestamp();
 
-	float lambda    = 1E-5;
-	int   maxiter   = 300;
-	int   blksize   = 8;
-	float step      = 0.5;
-	float tol       = 1.E-3;
-	bool  llr       = false;
-	bool  wav       = false;
-	bool  fista     = false;
-	bool  hgwld     = false;
-	float cont      = 1;
-	float eval      = -1;
-	const char* fwd = NULL;
-	const char* x0  = NULL;
-	int   gpun      = -1;
-	bool  dcx       = false;
-	bool  pf        = false;
+	const char* maps_file = NULL;
+	const char* wave_file = NULL;
+	const char* phi_file = NULL;
+	const char* reorder_file = NULL;
+	const char* table_file = NULL;
+	const char* out_file = NULL;
 
-	const struct opt_s opts[] = {
-		OPT_FLOAT( 'r', &lambda,  "lambda", "Soft threshold lambda for wavelet or locally low rank."),
-		OPT_INT(   'b', &blksize, "blkdim", "Block size for locally low rank."),
-		OPT_INT(   'i', &maxiter, "mxiter", "Maximum number of iterations."),
-		OPT_FLOAT( 's', &step,    "stepsz", "Step size for iterative method."),
-		OPT_FLOAT( 'c', &cont,    "cntnu",  "Continuation value for IST/FISTA."),
-		OPT_FLOAT( 't', &tol,     "toler",  "Tolerance convergence condition for iterative method."),
-		OPT_FLOAT( 'e', &eval,    "eigvl",  "Maximum eigenvalue of normal operator, if known."),
-		OPT_STRING('F', &fwd,     "frwrd",  "Go from shfl-coeffs to data-table. Pass in coeffs path."),
-		OPT_STRING('O', &x0,      "initl",  "Initialize reconstruction with guess."),
-		OPT_INT(   'g', &gpun,    "gpunm",  "GPU device number."),
-		OPT_SET(   'f', &fista,             "Reconstruct using FISTA instead of IST."),
-		OPT_SET(   'H', &hgwld,             "Use hogwild in IST/FISTA."),
-		OPT_SET(   'v', &dcx,               "Split coefficients to real and imaginary components."),
-		OPT_SET(   'w', &wav,               "Use wavelet."),
-		OPT_SET(   'l', &llr,               "Use locally low rank across temporal coefficients."),
-		OPT_SET(   'p', &pf,                "Use locally low rank and real-imaginary components for partial fourier."),
+	struct arg_s args[] = {
+
+		ARG_INFILE(true, &maps_file, "maps"),
+		ARG_INFILE(true, &wave_file, "wave"),
+		ARG_INFILE(true, &phi_file, "phi"),
+		ARG_INFILE(true, &reorder_file, "reorder"),
+		ARG_INFILE(true, &table_file, "table"),
+		ARG_OUTFILE(true, &out_file, "output"),
 	};
 
-	cmdline(&argc, argv, 6, 6, usage_str, help_str, ARRAY_SIZE(opts), opts);
+	struct opt_reg_s ropts;
+	opt_reg_init(&ropts);
 
-	if (pf)
-		dcx = true;
+	int   maxiter   = 30;
+	int   cgiter    = 10;
+	int   blksize   = 8;
+	float rho       = 1;
+	float eval      = -1;
+	float tol       = 1E-3;
+	bool  hgwld     = false;
+	bool  ksp       = false;
+	const char* fwd = NULL;
+	const char* x0  = NULL;
+	bool  dcx       = false;
+
+	const struct opt_s opts[] = {
+		{ 'R', NULL, true, OPT_SPECIAL, opt_reg, &ropts, "<T>:A:B:C", "Generalized regularization options. (-Rh for help)" },
+		OPT_INT(    'b', &blksize, "blkdim",    "Block size for locally low rank."),
+		OPT_INT(    'i', &maxiter, "mxiter",    "Maximum number of iterations."),
+		OPT_INT(    'j', &cgiter,  "cgiter",    "Maximum number of CG iterations in ADMM."),
+		OPT_FLOAT(  's', &rho,     "admrho",    "ADMM Rho value."),
+		OPT_FLOAT(  'e', &eval,    "eigval",    "Eigenvalue to scale step size. (Optional.)"),
+		OPT_INFILE( 'F', &fwd,     "frwrd",     "Go from shfl-coeffs to data-table. Pass in coeffs path."),
+		OPT_INFILE( 'O', &x0,      "initl",     "Initialize reconstruction with guess."),
+		OPT_FLOAT(  't', &tol,     "toler",     "Tolerance convergence condition for FISTA."),
+		OPT_SET(    'g', &bart_use_gpu,         "Use GPU."),
+		OPT_SET(    'K', &ksp,                  "Go from data-table to shuffling basis k-space."),
+		OPT_SET(    'H', &hgwld,                "Use hogwild."),
+		OPT_SET(    'v', &dcx,                  "Split coefficients to real and imaginary components."),
+	};
+
+	cmdline(&argc, argv, ARRAY_SIZE(args), args, help_str, ARRAY_SIZE(opts), opts);
+
+	struct admm_conf admm = { false, false, false, rho, cgiter };
 
 	debug_printf(DP_INFO, "Loading data... ");
 
 	long maps_dims[DIMS];
-	complex float* maps = load_cfl(argv[1], DIMS, maps_dims);
+	complex float* maps = load_cfl(maps_file, DIMS, maps_dims);
 
 	long wave_dims[DIMS];
-	complex float* wave = load_cfl(argv[2], DIMS, wave_dims);
+	complex float* wave = load_cfl(wave_file, DIMS, wave_dims);
 
 	long phi_dims[DIMS];
-	complex float* phi = load_cfl(argv[3], DIMS, phi_dims);
+	complex float* phi = load_cfl(phi_file, DIMS, phi_dims);
 
 	long reorder_dims[DIMS];
-	complex float* reorder = load_cfl(argv[4], DIMS, reorder_dims);
+	complex float* reorder = load_cfl(reorder_file, DIMS, reorder_dims);
 
 	long table_dims[DIMS];
-	complex float* table = load_cfl(argv[5], DIMS, table_dims);
+	complex float* table = load_cfl(table_file, DIMS, table_dims);
 
 	debug_printf(DP_INFO, "Done.\n");
 
-	if (gpun >= 0)
-		num_init_gpu_device(gpun);
-	else
-		num_init();
+	num_init_gpu_support();
 
 	int wx = wave_dims[0];
 	int sx = maps_dims[0];
@@ -983,6 +1031,32 @@ int main_wshfl(int argc, char* argv[])
 	coeff_dims[6] = tk;
 	coeff_dims[8] = dcx ? 2 : 1;
 
+	if (ksp) {
+
+		const struct linop_s* Knc = linop_kern_create(bart_use_gpu, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, table_dims);
+		long ksp_dims[] = { [0 ... DIMS - 1] = 1 };
+		ksp_dims[0] = wx;
+		ksp_dims[1] = sy;
+		ksp_dims[2] = sz;
+		ksp_dims[3] = nc;
+		ksp_dims[6] = tk;
+		complex float* res = create_cfl(out_file, DIMS, ksp_dims);
+
+		operator_apply(Knc->adjoint, DIMS, ksp_dims, res, DIMS, table_dims, table);
+
+		linop_free(Knc);
+		md_free(kernel);
+
+		unmap_cfl(DIMS, maps_dims, maps);
+		unmap_cfl(DIMS, wave_dims, wave);
+		unmap_cfl(DIMS, phi_dims, phi);
+		unmap_cfl(DIMS, reorder_dims, reorder);
+		unmap_cfl(DIMS, table_dims, table);
+		unmap_cfl(DIMS, ksp_dims, res);
+
+		return 0;
+	}
+
 	debug_printf(DP_INFO, "Creating single channel linear operators:\n");
 
 	double t1;
@@ -1012,7 +1086,7 @@ int main_wshfl(int argc, char* argv[])
 	long single_channel_table_dims[] = { [0 ... DIMS - 1] = 1 };
 	md_copy_dims(DIMS, single_channel_table_dims, table_dims);
 	single_channel_table_dims[1] = 1;
-	const struct linop_s* K = linop_kern_create(gpun >= 0, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
+	const struct linop_s* K = linop_kern_create(bart_use_gpu, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tK:   %f seconds.\n", t2 - t1);
 
@@ -1021,14 +1095,6 @@ int main_wshfl(int argc, char* argv[])
 
 	debug_printf(DP_INFO, "Single channel forward operator information:\n");
 	print_opdims(A_sc);
-	if (eval < 0)	
-#ifdef USE_CUDA
-		eval = (gpun >= 0) ? estimate_maxeigenval_gpu(A_sc->normal) : estimate_maxeigenval(A_sc->normal);
-#else
-		eval = estimate_maxeigenval(A_sc->normal);
-#endif
-	debug_printf(DP_INFO, "\tMax eval: %.2e\n", eval);
-	step /= eval;
 
 	struct linop_s* A = linop_multc_create(nc, md, maps, A_sc);
 	debug_printf(DP_INFO, "Overall forward linear operator information:\n");
@@ -1037,17 +1103,20 @@ int main_wshfl(int argc, char* argv[])
 	if (fwd != NULL) {
 
 		debug_printf(DP_INFO, "Going from coefficients to data table... ");
+
 		complex float* coeffs_to_fwd = load_cfl(fwd, DIMS, coeff_dims);
-		complex float* table_forward = create_cfl(argv[6], DIMS, table_dims);
+		complex float* table_forward = create_cfl(out_file, DIMS, table_dims);
+
 		const struct linop_s* R      = linop_wavereshape_create(wx, sx, sy, sz, 1, tk);
 		const struct linop_s* CFx    = linop_fx_create( wx, sy, sz, 1, tk, true);
 		const struct linop_s* W      = linop_wave_create(wx, sy, sz, 1, tk, wave_dims[COEFF_DIM], wave);
 		const struct linop_s* CFyz   = linop_fyz_create(wx, sy, sz, 1, tk, true);
-		const struct linop_s* K      = linop_kern_create(gpun >= 0, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
+		const struct linop_s* K      = linop_kern_create(bart_use_gpu, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
 		struct linop_s* AC_sc = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
 			R, CFx), W), CFyz), K);
 		struct linop_s* AC = linop_multc_create(nc, md, maps, AC_sc);
 		operator_apply(AC->forward, DIMS, table_dims, table_forward, DIMS, coeff_dims, coeffs_to_fwd);
+
 		debug_printf(DP_INFO, "Done.\n");
 
 		debug_printf(DP_INFO, "Cleaning up... ");
@@ -1066,152 +1135,129 @@ int main_wshfl(int argc, char* argv[])
 	}
 
 	if (dcx) {
+
 		debug_printf(DP_INFO, "\tSplitting result into real and imaginary components.\n");
 		struct linop_s* tmp = A;
 		struct linop_s* dcxop = linop_decompose_complex_create(DIMS, ITER_DIM, linop_domain(A)->dims);
 
 		A = linop_chain(dcxop, tmp);
+		debug_printf(DP_INFO, "New operator information:\n");
+			print_opdims(A);
 
 		linop_free(dcxop);
 		linop_free(tmp);
 	}
 
 	debug_printf(DP_INFO, "Normalizing data table and applying fftmod to table and maps... ");
+
 	float norm = md_znorm(DIMS, table_dims, table);
 	md_zsmul(DIMS, table_dims, table, table, 1. / norm);
 	fftmod_apply(sy, sz, reorder_dims, reorder, table_dims, table, maps_dims, maps);
+
 	debug_printf(DP_INFO, "Done.\n");
 
-	const struct operator_p_s* T = NULL;
-	long blkdims[MAX_LEV][DIMS];
-	long minsize[] = { [0 ... DIMS - 1] = 1 };
-	minsize[0] = MIN(sx, 16);
-	minsize[1] = MIN(sy, 16);
-	minsize[2] = MIN(sz, 16);
-	unsigned int WAVFLAG = (sx > 1) * READ_FLAG | (sy > 1) * PHS1_FLAG | (sz > 2) * PHS2_FLAG;
+	debug_printf(DP_INFO, "Preparing reconstruction operator: ");
 
-	enum algo_t algo = CG;
-	if ((wav) || (llr) || (pf)) {
-		algo = (fista) ? FISTA : IST;
-		if (wav) {
-			debug_printf(DP_INFO, "Creating wavelet threshold operator... ");
-			T = prox_wavelet_thresh_create(DIMS, coeff_dims, WAVFLAG, 0u, minsize, lambda, true);
-		} else if (llr) {
-			debug_printf(DP_INFO, "Creating locally low rank threshold operator across coeff and real-imag... ");
-			llr_blkdims(blkdims, ~(COEFF_FLAG | ITER_FLAG), coeff_dims, blksize);
-			T = lrthresh_create(coeff_dims, true, ~(COEFF_FLAG | ITER_FLAG), (const long (*)[])blkdims, lambda, false, false, false);
-		} else {
-			assert(dcx);
-			debug_printf(DP_INFO, "Creating locally low rank threshold operator across real-imag... ");
-			llr_blkdims(blkdims, ~ITER_FLAG, coeff_dims, blksize);
-			T = lrthresh_create(coeff_dims, true, ~ITER_FLAG, (const long (*)[])blkdims, lambda, false, false, false);
-		}
-		debug_printf(DP_INFO, "Done.\n");
-	}
+	const struct operator_p_s* thresh_ops[NUM_REGS] = { NULL };
+	const struct linop_s* trafos[NUM_REGS] = { NULL };
 
+	opt_reg_configure(DIMS, coeff_dims, &ropts, thresh_ops, trafos, blksize, 1, "dau2", bart_use_gpu);
+	int nr_penalties = ropts.r;
+	struct reg_s* regs = ropts.regs;
+	bool fista = (nr_penalties == 1);
+
+	// FISTA variables.
+	float step = 0.5;
 	italgo_fun2_t italgo = iter2_call_iter;
 	struct iter_call_s iter2_data;
 	SET_TYPEID(iter_call_s, &iter2_data);
 	iter_conf* iconf = CAST_UP(&iter2_data);
+	struct iter_fista_conf fsconf = iter_fista_defaults;
 
-	struct iter_conjgrad_conf cgconf = iter_conjgrad_defaults;
-	struct iter_fista_conf    fsconf = iter_fista_defaults;
-	struct iter_ist_conf      isconf = iter_ist_defaults;
+	// ADMM variables.
+	struct iter it;
 
-	switch(algo) {
+	if (fista) {
 
-		case IST:
+		if (eval < 0) {
+#ifdef USE_CUDA
+			eval = bart_use_gpu ? estimate_maxeigenval_gpu(A_sc->normal) : estimate_maxeigenval(A_sc->normal);
+#else
+			eval = estimate_maxeigenval(A_sc->normal);
+#endif
+		}
+		step /= eval;
+		debug_printf(DP_INFO, "\tAlgorithm:      FISTA.\n");
+		debug_printf(DP_INFO, "\tMax eigenvalue: %.2e\n", eval);
+		debug_printf(DP_INFO, "\tStep:           %.2e\n", step);
+		debug_printf(DP_INFO, "\tTolerance:      %.2e\n", tol);
 
-			debug_printf(DP_INFO, "Using IST.\n");
-			debug_printf(DP_INFO, "\tLambda:             %0.2e\n", lambda);
-			debug_printf(DP_INFO, "\tMaximum iterations: %d\n", maxiter);
-			debug_printf(DP_INFO, "\tStep size:          %0.2e\n", step);
-			debug_printf(DP_INFO, "\tHogwild:            %d\n", (int) hgwld);
-			debug_printf(DP_INFO, "\tTolerance:          %0.2e\n", tol);
-			debug_printf(DP_INFO, "\tContinuation:       %0.2e\n", cont);
+		fsconf.maxiter      = maxiter;
+		fsconf.step         = step;
+		fsconf.hogwild      = hgwld;
+		fsconf.tol          = tol;
 
-			isconf              = iter_ist_defaults;
-			isconf.step         = step;
-			isconf.maxiter      = maxiter;
-			isconf.tol          = tol;
-			isconf.continuation = cont;
-			isconf.hogwild      = hgwld;
+		iter2_data.fun   = iter_fista;
+		iter2_data._conf = CAST_UP(&fsconf);
 
-			iter2_data.fun   = iter_ist;
-			iter2_data._conf = CAST_UP(&isconf);
+	} else {
 
-			break;
-
-		case FISTA:
-
-			debug_printf(DP_INFO, "Using FISTA.\n");
-			debug_printf(DP_INFO, "\tLambda:             %0.2e\n", lambda);
-			debug_printf(DP_INFO, "\tMaximum iterations: %d\n", maxiter);
-			debug_printf(DP_INFO, "\tStep size:          %0.2e\n", step);
-			debug_printf(DP_INFO, "\tHogwild:            %d\n", (int) hgwld);
-			debug_printf(DP_INFO, "\tTolerance:          %0.2e\n", tol);
-			debug_printf(DP_INFO, "\tContinuation:       %0.2e\n", cont);
-
-			fsconf              = iter_fista_defaults;
-			fsconf.maxiter      = maxiter;
-			fsconf.step         = step;
-			fsconf.hogwild      = hgwld;
-			fsconf.tol          = tol;
-			fsconf.continuation = cont;
-
-			iter2_data.fun   = iter_fista;
-			iter2_data._conf = CAST_UP(&fsconf);
-
-			break;
-
-		default:
-		case CG:
-
-			debug_printf(DP_INFO, "Using CG.\n");
-			debug_printf(DP_INFO, "\tMaximum iterations: %d\n", maxiter);
-			debug_printf(DP_INFO, "\tTolerance:          %0.2e\n", tol);
-
-			cgconf          = iter_conjgrad_defaults;
-			cgconf.maxiter  = maxiter;
-			cgconf.l2lambda = 0;
-			cgconf.tol      = tol;
-
-			iter2_data.fun   = iter_conjgrad;
-			iter2_data._conf = CAST_UP(&cgconf);
-
-			break;
+		debug_printf(DP_INFO, "\tAlgorithm: ADMM\n.");
+		debug_printf(DP_INFO, "\tRho:       %.2e\n.", rho);
 
+		it = italgo_config(ALGO_ADMM, nr_penalties, regs, maxiter, step, hgwld, false, admm, 1, false);
 	}
 
 	complex float* init = NULL;
 	if (x0 != NULL) {
+
 		debug_printf(DP_INFO, "Loading in initial guess... ");
+
 		init = load_cfl(x0, DIMS, coeff_dims);
+
 		debug_printf(DP_INFO, "Done.\n");
 	}
 
 	debug_printf(DP_INFO, "Reconstruction... ");
-	complex float* recon = create_cfl(argv[6], DIMS, coeff_dims);
-	struct lsqr_conf lsqr_conf = { 0., gpun >= 0 };
+
+	complex float* recon = create_cfl(out_file, DIMS, coeff_dims);
+
+	struct lsqr_conf lsqr_conf = lsqr_defaults;
+	lsqr_conf.lambda = 0.;
+	lsqr_conf.it_gpu = bart_use_gpu;
 	double recon_start = timestamp();
-	const struct operator_p_s* J = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*) init, A, NULL, 1, &T, NULL, NULL);
+
+	const struct operator_p_s* J = fista ?
+		lsqr2_create(&lsqr_conf, italgo,    iconf,    (const float*) init, A, NULL, nr_penalties, thresh_ops, NULL,   NULL):
+		lsqr2_create(&lsqr_conf, it.italgo, it.iconf, (const float*) init, A, NULL, nr_penalties, thresh_ops, trafos, NULL);
 	operator_p_apply(J, 1., DIMS, coeff_dims, recon, DIMS, table_dims, table);
+	md_zsmul(DIMS, coeff_dims, recon, recon, norm);
+
 	double recon_end = timestamp();
+
 	debug_printf(DP_INFO, "Done.\nReconstruction time: %f seconds.\n", recon_end - recon_start);
 
 	debug_printf(DP_INFO, "Cleaning up and saving result... ");
+
 	operator_p_free(J);
+
+	italgo_config_free(it);
+
 	linop_free(A);
 	linop_free(A_sc);
+
 	md_free(kernel);
+
 	unmap_cfl(DIMS, maps_dims, maps);
 	unmap_cfl(DIMS, wave_dims, wave);
 	unmap_cfl(DIMS, phi_dims, phi);
 	unmap_cfl(DIMS, reorder_dims, reorder);
 	unmap_cfl(DIMS, table_dims, table);
 	unmap_cfl(DIMS, coeff_dims, recon);
+
 	if (x0 != NULL)
 		unmap_cfl(DIMS, coeff_dims, init);
+
 	debug_printf(DP_INFO, "Done.\n");
 
 	double end_time = timestamp();
