@@ -1,4 +1,3 @@
 % set Matlab path and TOOLBOX_PATH environment variable
 addpath(fullfile(pwd, 'matlab'));
-setenv('TOOLBOX_PATH', pwd);
-
+setenv('BART_TOOLBOX_PATH', pwd);
