@@ -3,6 +3,8 @@
 
 #include "num/multind.h"
 #include "num/rand.h"
+#include "num/flpmath.h"
+
 
 #include "utest.h"
 
@@ -162,3 +164,95 @@ static bool test_md_reshape(void)
 UT_REGISTER_TEST(test_md_reshape);
 
 
+static bool test_compress(void)
+{
+	enum { N = 1 };
+	long dims[N] = { 31 };
+
+	complex float* _ptr1 = md_alloc(N, dims, CFL_SIZE);
+	md_gaussian_rand(N, dims, _ptr1);
+
+	dims[0] = 62;
+
+	float* ptr1 = (float*)_ptr1;
+	md_sgreatequal(N, dims, ptr1, ptr1, 0.);
+
+	void* compress = md_compress(N, dims, ptr1);
+
+	float* ptr2 = md_alloc(N, dims, FL_SIZE);
+	md_decompress(N, dims, ptr2, compress);
+
+	float err = md_nrmse(N, dims, ptr2, ptr1);
+
+	md_free(ptr1);
+	md_free(ptr2);
+	md_free(compress);
+
+	
+	UT_ASSERT(err < UT_TOL);
+}
+
+UT_REGISTER_TEST(test_compress);
+
+
+static float test_md_reflectpad_center(int D,
+		const long odims[D], const complex float* expect,
+		const long idims[D], const complex float* in)
+{
+	complex float* t = md_alloc(D, odims, sizeof(complex float));
+
+	md_reflectpad_center(D, odims, t, idims, in, sizeof(complex float));
+
+	float err = md_znrmse(D, odims, expect, t);
+
+	md_free(t);
+
+	return err;
+}
+
+static bool test_md_reflectpad_center_1(void)
+{
+	const long idims[] = { 3, 2 };
+
+	const complex float in[] = {
+		1, 2, 3,
+		4, 5, 6,
+	};
+
+	const long odims[] = { 4, 3 };
+
+	const complex float good[] = {
+		1, 1, 2, 3,
+		4, 4, 5, 6,
+		4, 4, 5, 6,
+	};
+
+	UT_ASSERT(UT_TOL > test_md_reflectpad_center(2, odims, good, idims, in));
+}
+
+UT_REGISTER_TEST(test_md_reflectpad_center_1);
+
+
+static bool test_md_reflectpad_center_2(void)
+{
+	const long idims[] = { 2, 2 };
+
+	complex float in[] = {
+		1, 2,
+		4, 5,
+	};
+
+	const long odims[] = { 9, 4 };
+
+	complex float good[] = {
+		2, 2, 1, 1, 2, 2, 1, 1, 2,
+		2, 2, 1, 1, 2, 2, 1, 1, 2,
+		5, 5, 4, 4, 5, 5, 4, 4, 5,
+		5, 5, 4, 4, 5, 5, 4, 4, 5,
+	};
+
+	UT_ASSERT(UT_TOL > test_md_reflectpad_center(2, odims, good, idims, in));
+}
+
+UT_REGISTER_TEST(test_md_reflectpad_center_2);
+
