@@ -13,6 +13,8 @@
 #include "misc/debug.h"
 #include "misc/misc.h"
 
+#include "num/mpi_ops.h"
+
 #include "utest.h"
 
 
@@ -42,6 +44,10 @@ UTESTS
 
 int main(int argc, char* argv[])
 {
+#ifdef USE_MPI
+	init_mpi(&argc, &argv);
+#endif
+
 	UNUSED(argc);
 
 	int num_tests_run = 0;
@@ -61,6 +67,8 @@ int main(int argc, char* argv[])
 
 	debug_printf(good ? DP_INFO : DP_ERROR, "%20s: %2d/%2d passed.\n", argv[0], num_tests_pass, num_tests_run);
 
+	deinit_mpi();
+
 	exit(good ? 0 : 1);
 }
 
